{ͻ
                              UNIT MosAppMatrix                             
 ͹
  CREATOR....: Cheveau Frdric & PLANAS Jean-Pierre                        
  UPDATED BY.: Cheveau Frdric & PLANAS Jean-Pierre                        
  CREATED THE: 04.01.2002                                                   
  COUNTRY....: Montpellier - (FRANCE)                                       
  DESCRIPTION: Manage a DOS text program that process some files.           
  AUTONOMOUS.: Yes                                                          
  UNIT SIZE..: 60 Lines and (640 Bytes compiled)                            
  LAST UPDATE: 24.12.1999 at 20:51                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosAppMatrix;

INTERFACE

Uses
     Crt,                               // Replace it by MosCrt when all bugs will be fixed
     Dos,                               // Pascal Units
     Classes,
     MosStrings,                        // MOS Units
     MosDisks,
     MosFiles,
     MosFilesTree;

Type

TParamMode = (                          // Parameters modes definition
   MMAutoParam,
   MMMandatoryParam
   );

TMatrixMode = (                         // Manage a source only or source/target application
   MMNoSource,
   MMSourceOnly,
   MMSourceTarget
   );

TSwitchType = (                         // Select the switch type '/' or '-'
   STSlash,
   STDec
   );

TAppResult = (                          // Results during process
   ARNoErrors,
   ARSourceNotSpecified,
   ARTargetNotSpecified,
   ARNoSourceOrTargetAllowed,
   ARTooMuchParameters,
   ARUnknownSwitches,
   ARCannotRemoveSwitch,
   ARParamAlreadyDefined,
   ARErrorDuringProcess,
   ARForceExit
   );

//****************************************************************************
//****************************************************************************
//****************************************************************************

TApplicationMatrix = Class;             // Forward;

TCommandsLine = Class                   // Commands line management
   Private
      fSwitch        :Char;             // Char for switch
      fSwitchType    :TSwitchType;      // Select switch '/' or '-'
      fFreeParamCount:Integer;          // Count the free parameters in commands line (params without '/')
      fParamList     :TStringList;      // Manage the parameters list

// The followed variable is used only to link the TCommandsLine class
// to the TApplicationMatrix class and access to the AppError variable

      fAppMatrix     :TApplicationMatrix;

      Function  GetParamIndex(Param:String):Integer;
      Procedure SetSwitchType(pSwitchType:TSwitchType);
      Procedure AddNewParam(TypeP,Param,Synopsis,Description:String; pVisible:Boolean);
      Function  RemoveParamEx(Param:String):Integer;

   Protected
      Property AppMatrix:TApplicationMatrix Read fAppMatrix      Write fAppMatrix;
      Property ParamList     :TStringList   Read fParamList;

   Public
      Property SwitchType    :TSwitchType   Read fSwitchType     Write SetSwitchType;
      Property Switch        :Char          Read fSwitch;
      Property FreeParamCount:Integer       Read fFreeParamCount Write fFreeParamCount;

      Constructor Create;
      Destructor  Destroy; Override;

      Function  CheckAllParameters:String;
      Function  GetSourceOrTargetParam(pTypeToGet:TMatrixMode):String;
      Function  TestSwitch(Param:String):Boolean;
      Function  TestAndGetSwitchResult(Param:String):String;

      Function  IsParamAvailable(Param:String):Boolean;
      Procedure AddNewCommand   (Param,Synopsis,Description:String; pVisible:Boolean);
      Procedure AddNewSwitch    (Param,Synopsis,Description:String; pVisible:Boolean);
      Function  RemoveParam     (Param:String):Integer;
   End;

//****************************************************************************
//****************************************************************************
//****************************************************************************

TApplicationMatrix = Class(TFileRecurse)// Application squeletton class
   Private
      fSwitches      :TCommandsLine;    // Switches management
      fVersion       :String;           // Version of application
      fReleaseDate   :String;           // Date of last modification
      fAuthor        :String;           // Author of the program
      fCopyright     :String;           // Program copyright

      fSPath,fSMask  :String;           // Source path & mask
      fTPath,fTMask  :String;           // Target path & mask
      fSource        :String;           // Source file(s)
      fTarget        :String;           // Target file(s)
      fTimeInit      :TDateTime;        // To calcule elapsed time
      fTimeEnd       :TDateTime;
      fDoneOne       :Boolean;          // If at least a file is done
      fOldDirectory  :String;           // Previous scanned directory
      fFilesCount    :Longint;          // Count processed files
      fDirsCount     :Longint;          // Count processed directories
      fConflictsCount:Longint;          // Count files conflicts
      fAppError      :TAppResult;       // Application errors
      fMatrixMode    :TMatrixMode;      // Manage a source only or source/target application
      fParamMode     :TParamMode;       // Parameters mode
      fBackupConflict:TFileConflict;    // Manage the files conflicts
      fBackupFiles   :Boolean;          // Create a backup for each file
      fEnableBackup  :Boolean;          // Enable or disable file backup
      fRecurseDirs   :Boolean;          // Recurse directories
      fEnableRecurse :Boolean;          // Enable or disable file recurse

      Procedure SetEnableBackup(pEnableBackup:Boolean);
      Procedure SetEnableRecurse(pEnableRecurse:Boolean);
      Procedure SetMatrixMode(pMatrixMode:TMatrixMode);
      Procedure SetParamMode(pParamMode:TParamMode);
      Procedure SetAppError(pAppError:TAppResult);
      Function  GetElapsedTime:String;
      Procedure DisplayHelpCommandsEx(TypeToDisplay:String);

   Protected
      Procedure OnDisplayAuthor; Virtual;
      Procedure OnDisplayAppHeader; Virtual;
      Procedure OnDisplayHelp; Virtual;
      Procedure OnDisplayHelpExplainText; Virtual;
      Procedure OnDisplayHelpSynopsis; Virtual;
      Procedure OnDisplayHelpCommands; Virtual;
      Procedure OnDisplayHelpSwitches; Virtual;
      Procedure OnDisplayHelpExample; Virtual;
      Procedure OnDisplayHelpCustomText; Virtual;
      Procedure OnDisplaySummary; Virtual;
      Procedure OnDisplayDebugInfos; Virtual;

      Procedure OnDefineSwitches; Virtual;
      Procedure OnDefineCommands; Virtual;
      Procedure OnCheckSwitches; Virtual;
      Procedure OnValidateData(Var pSource,pTarget:String); Virtual;
      Procedure OnBeforeProcess; Virtual;
      Procedure OnAfterProcess; Virtual;

   Public
      Property MatrixMode    :TMatrixMode   Read fMatrixMode     Write SetMatrixMode;
      Property ParamMode     :TParamMode    Read fParamMode      Write SetParamMode;
      Property AppError      :TAppResult    Read fAppError       Write SetAppError;
      Property EnableRecurse :Boolean       Read fEnableRecurse  Write SetEnableRecurse;
      Property EnableBackup  :Boolean       Read fEnableBackup   Write SetEnableBackup;

      Property Switches      :TCommandsLine Read fSwitches       Write fSwitches;
      Property BackupConflict:TFileConflict Read fBackupConflict Write fBackupConflict;
      Property Version       :String        Read fVersion        Write fVersion;
      Property ReleaseDate   :String        Read fReleaseDate    Write fReleaseDate;
      Property Author        :String        Read fAuthor         Write fAuthor;
      Property Copyright     :String        Read fCopyright      Write fCopyright;

      Property ElapsedTime   :String        Read GetElapsedTime;
      Property Source        :String        Read fSource;
      Property SourceMask    :String        Read fSMask;
      Property SourcePath    :String        Read fSPath;
      Property Target        :String        Read fTarget;
      Property TargetMask    :String        Read fTMask;
      Property TargetPath    :String        Read fTPath;
      Property FilesCount    :Longint       Read fFilesCount;
      Property DirsCount     :Longint       Read fDirsCount;
      Property ConflictsCount:Longint       Read fConflictsCount;
      Property BackupFiles   :Boolean       Read fBackupFiles;
      Property RecurseDirs   :Boolean       Read fRecurseDirs;

      Constructor Create;
      Destructor  Destroy; Override;
      Procedure   Run; Virtual;
      Procedure   OnFileFound(CurrentFile:String); Override;
      Procedure   OnDirectoryFound(pDirectory:String); Virtual;
   End;

IMPLEMENTATION

Uses
    SysUtils;

Const

    SepPipe = '|';

Constructor TCommandsLine.Create;
Begin
   fFreeParamCount := 0;

   fParamList := TStringList.Create;
End;

Destructor TCommandsLine.Destroy;
Begin
   fParamList.Free;
End;

Procedure TCommandsLine.SetSwitchType(pSwitchType:TSwitchType);
Begin
   fSwitchType := pSwitchType;
   Case fSwitchType of
      STSlash : fSwitch := '/';
      STDec   : fSwitch := '-';
      Else fSwitch := '/';
   End;
End;

Function TCommandsLine.CheckAllParameters:String;
Var A,B          :Integer;
    St           :String;
    PLine        :String;
    Found        :Boolean;
    ErrParams    :String;

Begin
   ErrParams := '';

   For A := 1 to Str_ParamCount(Str_GetCommandLine) do
   Begin
      St := Str_ParamStr(Str_GetCommandLine,A);
      If Pos(Switch,St) > 0 Then         // It's a defined switch...
      Begin
         Delete(St, Pos(Switch,St), 1);  // Remove the switch char (usually '/')
         If Pos('=',St) > 0 Then St := Copy(St, 1, Pos('=',St) - 1); // Remove all after the '=' char if exist
         Found := FALSE;
         For B := 0 to fParamList.Count - 1 do   // Search in param list
         Begin
            PLine := fParamList[B];
            If Upper(Str_Get(PLine,2,SepPipe)) = Upper(St) Then Found := TRUE;
         End;
         If Not Found Then ErrParams := ErrParams + ' ' + Switch + St;
      End;
   End;
   Result := ErrParams;
End;

Function TCommandsLine.GetSourceOrTargetParam(pTypeToGet:TMatrixMode):String;
Var A            :Integer;
    St           :String;

Begin
   Result := '';
   fFreeParamCount := 0;

   For A := 1 to Str_ParamCount(Str_GetCommandLine) do
   Begin
      St := Str_ParamStr(Str_GetCommandLine,A);
      If Pos(Switch,St) <= 0 Then
      Begin
         Inc(fFreeParamCount);
         If (pTypeToGet = MMSourceOnly)   And (fFreeParamCount = 1) Then Result := Dsk_FExpand(St);
         If (pTypeToGet = MMSourceTarget) And (fFreeParamCount = 2) Then Result := Dsk_FExpand(St);
      End;
   End;

   If fFreeParamCount > 2 Then Result := ''; // More than 2 parameter means an error !
End;

Function TCommandsLine.GetParamIndex(Param:String):Integer;
Var A            :Integer;
Begin
   Result := -1;
   For A := 0 to fParamList.Count - 1 do
   Begin
      If Upper(Str_Get(fParamList[A],2,SepPipe)) = Upper(Param) Then Result := A;
   End;
End;

Function TCommandsLine.IsParamAvailable(Param:String):Boolean;
Begin
   If GetParamIndex(Param) >= 0 Then Result := TRUE
                                Else Result := FALSE;
End;

Function TCommandsLine.RemoveParam(Param:String):Integer;
Begin
   Result := 0;
   If (Param = '?') Or (Upper(Param) = 'AUTHOR') Or
      (Upper(Param) = 'DEBUG') Or (Param = 'S') Or (Param = 'B')  Then
   Begin                        // Some switches cannot be removed...
      AppMatrix.AppError := ARCannotRemoveSwitch;
   End;

   RemoveParamEx(Param);        // Call inner "RemoveParam"
End;

Function TCommandsLine.RemoveParamEx(Param:String):Integer;
Var Index:Integer;
Begin
   Index := GetParamIndex(Param);
   If Index >= 0 Then
   Begin
      fParamList.Delete(Index); // Remove the parameter from list
   End;
   Result := fParamList.Count;
End;

Procedure TCommandsLine.AddNewParam(TypeP,Param,Synopsis,Description:String; pVisible:Boolean);
Begin
   If GetParamIndex(Param) < 0 Then // Parameter not already defined...
   Begin
      fParamList.Add(TypeP + SepPipe + Param + SepPipe +
         Synopsis + SepPipe + Description + SepPipe +
         Str_B2S(pVisible));
   End
   Else AppMatrix.AppError := ARParamAlreadyDefined;
End;

Procedure TCommandsLine.AddNewCommand(Param,Synopsis,Description:String; pVisible:Boolean);
Begin
   AddNewParam('C',Param,Synopsis,Description,pVisible);
End;

Procedure TCommandsLine.AddNewSwitch (Param,Synopsis,Description:String; pVisible:Boolean);
Begin
   AddNewParam('S',Param,Synopsis,Description,pVisible);
End;

Function TCommandsLine.TestSwitch(Param:String):Boolean;
Var TheParam:String;

Begin
   Result := FALSE;

   TheParam := Switch + Param;
   If Not IsParamAvailable(Param) Then Exit;         // Parameter is not in authorised parameters list
   If Parameter(TheParam,0) > 0 Then Result := TRUE  // Parameter is in the command line
End;

Function TCommandsLine.TestAndGetSwitchResult(Param:String):String;
Var TheParam:String;

Begin
   Result := '';
   TheParam := Switch + Param;
   If Not IsParamAvailable(Param) Then Exit;         // Parameter is not in authorised parameters list
   If Parameter(TheParam,0) > 0 Then
   Begin
      If TheParam <> Switch + Param Then Result := TheParam;
   End;
End;

{ͻ
                                 Authors                                    
 ͹
  OUT....: Display more informations about authors.                         
 ͼ}

Procedure TApplicationMatrix.OnDisplayAuthor;
Begin
   If fAuthor = 'JP.Planas' Then
   Begin
      Writeln('PLANAS Jean-Pierre as SYBARIS');
      Writeln('7 Rue Jean Mac');
      Writeln('75011 Paris');
      Writeln('FRANCE');
      Writeln('E-Mail : sybaris@club-internet.fr');
      Writeln('Software engineer (Cross Systems Company)');
      Writeln('E-Mail : jp.planas@cross-systems.com');
      AppError := ARForceExit;   // Exit with closing all ressources
   End;

   If fAuthor = 'F.Cheveau' Then
   Begin
      Writeln('CHEVEAU Frdric as ZuulCoder');
      Writeln('68 Avenue de la Justice de Castelnau');
      Writeln('Res. Les Terrasses d''Occitanie. Bat H');
      Writeln('34090 MONTPELLIER');
      Writeln('FRANCE');
      Writeln('E-Mail : zuulcoder@hotmail.com');
      Writeln('Software engineer (Timeless Company)');
      Writeln('E-Mail : frederic.cheveau@vcstimeless.fr');
      AppError := ARForceExit;   // Exit with closing all ressources
   End;
End;

Procedure TApplicationMatrix.OnDisplayDebugInfos;
Var A,B           :Integer;
    St            :String;
    SwitchesCount :Integer;
    CommandsCount :Integer;

    PLine         :String;
    Found         :Boolean;
    ErrParams     :String;
    CountAllParams:Integer;
    LineToDisplay :String;

Begin
//****************************************************************************

   Writeln('Commands line informations');
   Writeln;
   Writeln('Input = ', Str_GetCommandLine);
   Writeln;

   ErrParams := '';

   With Switches do
   Begin
      CountAllParams := Str_ParamCount(Str_GetCommandLine);

      For A := 1 to CountAllParams do
      Begin
         St := Str_ParamStr(Str_GetCommandLine,A);
         LineToDisplay := '   ' + St;

         If Pos(Switch,St) > 0 Then         // It's a defined switch...
         Begin
            Delete(St, Pos(Switch,St), 1);  // Remove the switch char (usually '/')
            If Pos('=',St) > 0 Then
            Begin
               St := Copy(St, 1, Pos('=',St) - 1); // Remove all after the '=' char if exist
               LineToDisplay := LineToDisplay + Str_Spaces(15 - Length(LineToDisplay)) + 'Return value';
            End
            Else
            Begin
               LineToDisplay := LineToDisplay + Str_Spaces(15 - Length(LineToDisplay)) + 'Switch';
            End;
            Found := FALSE;
            For B := 0 to fParamList.Count - 1 do   // Search in param list
            Begin
               PLine := fParamList[B];
               If Upper(Str_Get(PLine,2,SepPipe)) = Upper(St) Then Found := TRUE;
            End;
            If Not Found Then
            Begin
               ErrParams := ErrParams + ' ' + Switch + St;
            End
            Else
            Begin
               LineToDisplay := LineToDisplay + Str_Spaces(33 - Length(LineToDisplay)) + 'Defined';
            End;
         End
         Else
         Begin
            LineToDisplay := LineToDisplay + Str_Spaces(15 - Length(LineToDisplay)) + 'Free parameter';
         End;
         Writeln(LineToDisplay);
      End;

      Writeln;
      Writeln('Parameters count          = ',  CountAllParams);
      Writeln('Parameter switch char     = "', Switch,'"');
      Writeln('Parameters not registered = "', Trim(ErrParams),'"');
      Writeln;
   End;
   Readkey;

//****************************************************************************

   Writeln('Registered informations');

   SwitchesCount := 0;
   CommandsCount := 0;

   With Switches do
   Begin
      For A := 0 to ParamList.Count - 1 do    // Display registered switches
      Begin
         If Str_Get(ParamList[A],1,SepPipe) = 'C' Then Inc(CommandsCount);
         If Str_Get(ParamList[A],1,SepPipe) = 'S' Then Inc(SwitchesCount);
      End;

      Writeln;
      Writeln('Registered commands (', CommandsCount,')');
      For A := 0 to ParamList.Count - 1 do    // Display registered switches
      Begin
         St := ParamList[A];
         If Str_Get(St,1,SepPipe) = 'C' Then Writeln('   ', St);
      End;
      Writeln;
      Writeln('Registered switches (', SwitchesCount,')');
      For A := 0 to ParamList.Count - 1 do    // Display registered switches
      Begin
         St := ParamList[A];
         If Str_Get(St,1,SepPipe) = 'S' Then Writeln('   ', St);
      End;
   End;
   Writeln;

//****************************************************************************

   Writeln('Properties informations');
   Writeln;
(*

Display here the :
   MatrixMode     := MMNoSource;       // No source is needed
   ParamMode      := MMAutoParam;      // Set source parameter if not specified

   Using then RTTI to display types const as strings.
*)
End;

{ͻ
                                   HELP                                     
 ͹
  OUT....: Display the application help                                     
 ͼ}

Procedure TApplicationMatrix.OnDisplayHelpExplainText;
Begin
   Writeln('Describe your application here');
End;

Procedure TApplicationMatrix.OnDisplayHelpSynopsis;
Begin
   With Switches do
   Begin
      Writeln('Synopsis: ', Get_AppName, ' <File(s)> ', Switch , 'Commands ', Switch , 'Switches');
      Writeln('Inputfile <File(s)>     : File(s) to be processed');
   End;
End;

Procedure TApplicationMatrix.DisplayHelpCommandsEx(TypeToDisplay:String);
Var A            :Integer;
    StLib        :String;
    StType       :String;
    StSynopsis   :String;
    StDescription:String;
    StVisible    :String;
    StFinal      :String;
    Count        :Integer;

Begin
   With Switches do
   Begin
      If TypeToDisplay = 'C' Then StLib := 'Commands: '
                             Else StLib := 'Switches: ';
      Count := 0;
      For A := 0 to fParamList.Count - 1 do
      Begin
         StType        := Str_Get(fParamList[A],1,SepPipe);
         StSynopsis    := Str_Get(fParamList[A],3,SepPipe);
         StDescription := Str_Get(fParamList[A],4,SepPipe);
         StVisible     := Str_Get(fParamList[A],5,SepPipe);

         If (StType = TypeToDisplay) And (StVisible = Str_B2S(TRUE)) Then
         Begin
            Inc(Count);
            If Count = 1 Then StFinal := StLib
                         Else StFinal := '';
            StFinal := StFinal + Str_Spaces(10 - Length(StFinal)) + Switch + StSynopsis;
            StFinal := StFinal + Str_Spaces(24 - Length(StFinal)) + ': ' + StDescription;
            Writeln(StFinal);
         End;
      End;
   End;
End;

Procedure TApplicationMatrix.OnDisplayHelpCommands;
Begin
   DisplayHelpCommandsEx('C');
End;

Procedure TApplicationMatrix.OnDisplayHelpSwitches;
Begin
   DisplayHelpCommandsEx('S');
End;

Procedure TApplicationMatrix.OnDisplayHelpExample;
Begin
   With Switches do
   Begin
      Writeln('Example : ', Get_AppName, ' FILE.* ', Switch , 'S');
   End;
End;

Procedure TApplicationMatrix.OnDisplayHelpCustomText;
Begin
End;

Procedure TApplicationMatrix.OnDisplayAppHeader;
Begin
   Writeln(Get_AppName,' ',fVersion,' - ',fCopyright,' ',fAuthor);
   Writeln('Released the ',fReleaseDate);
   Writeln;
End;

Procedure TApplicationMatrix.OnDisplayHelp;
Begin
   With Switches do
   Begin
      OnDisplayHelpExplainText; // Override this function to implement your own text...
      Writeln;
      OnDisplayHelpSynopsis;    // Override this function to implement your own synopsis...
      Writeln;
      OnDisplayHelpCommands;    // Override this function to implement your own commands...
      OnDisplayHelpSwitches;    // Override this function to implement your own switches...
      OnDisplayHelpExample;     // Override this function to implement your own example...
      Writeln;
      Writeln('Note    : Please note that aliases and jockers are supported.');
      Writeln('          Commands and switches are not case sensitives.');
      Writeln('          Redirection to a file possible with > and >>.');
      Writeln('          Support long filenames (Windows and NT OS).');
      OnDisplayHelpCustomText;  // Override this function to display a special text...
      AppError := ARForceExit;// Exit with closing all ressources
   End;
End;

{ͻ
                  PROCEDURE CALLED FOR EACH DIRECTORY                       
 ͼ}

Procedure TApplicationMatrix.OnDirectoryFound(pDirectory:String);
Begin
   fOldDirectory := pDirectory;
   Inc(fDirsCount);
End;

{ͻ
                  PROCEDURE CALLED FOR EACH FILES                           
 ͼ}

Procedure TApplicationMatrix.OnFileFound(CurrentFile:String);
Var FD         :DirStr;                // FSplit
    FN         :NameStr;
    FE         :ExtStr;
    BackupName :String;

Begin                                  // Manage base functions like backup file etc...
   Try
      FSplit(CurrentFile, FD,FN,FE);   // Split Filename

      If fBackupFiles Then             // Backup the file if needs...
      Begin
         BackupName := FD + FN + '.BAK';
         If File_TestExist(BackupName) Then
         Begin
            If fBackupConflict = FCReplaceFile Then
            Begin                      // Replace file
               File_Erase(BackupName);
               File_Copy(CurrentFile, BackupName);
               Inc(fConflictsCount);   // Conflicts counter
            End;
         End
         Else
         Begin                         // Just copy file
            File_Copy(CurrentFile, BackupName);
         End;
      End;

      If fOldDirectory <> FD Then      // Directory changed...
      Begin
         OnDirectoryFound(FD);
      End;

   Finally
      Inc(fFilesCount);                // Count processed files
      fDoneOne := TRUE;                // At less one file has been processed !
   End;
End;

Function TApplicationMatrix.GetElapsedTime:String;
Var  ST       :TSystemTime;
     StHour   :String;
     StMinute :String;
     StSecond :String;
     StMilli  :String;

Begin
   Result := '';
   If fTimeEnd = 0 Then Exit; // Process not terminated !

   DateTimeToSystemTime(FTimeEnd - fTimeInit, ST);

   With ST do
   Begin
      StHour   := Str_FillZero(Hour, 2);
      StMinute := Str_FillZero(Minute, 2);
      StSecond := Str_FillZero(Second, 2);
      StMilli  := Str_I2S(Millisecond);
      Result := StHour + ':' + StMinute + ':' + StSecond + '.' + StMilli;
   End;
End;

Procedure TApplicationMatrix.SetMatrixMode(pMatrixMode:TMatrixMode);
Begin
   fMatrixMode := pMatrixMode;

// MMAutoParam et MMSourceTarget incompatibles...
   If (fMatrixMode = MMSourceTarget) And (fParamMode = MMAutoParam) Then ParamMode := MMMandatoryParam;

// If MMNoSource is needed the MMAutoParam must be set...
   If (fMatrixMode = MMNoSource) Then ParamMode := MMAutoParam;
End;

Procedure TApplicationMatrix.SetParamMode(pParamMode:TParamMode);
Begin
// MMAutoParam et MMSourceTarget incompatibles...
   If (fMatrixMode = MMSourceTarget) And (pParamMode = MMAutoParam) Then Exit;

   fParamMode := pParamMode;
End;

Procedure TApplicationMatrix.SetEnableBackup(pEnableBackup:Boolean);
Begin
   fEnableBackup := pEnableBackup;

   If fEnableBackup Then
   Begin
      fBackupFiles := TRUE;
      Switches.AddNewSwitch('B','B','Create backup file',TRUE);
   End
   Else
   Begin
      fBackupFiles := FALSE;
      Switches.RemoveParamEx('B');
   End;
End;

Procedure TApplicationMatrix.SetEnableRecurse(pEnableRecurse:Boolean);
Begin
   fEnableRecurse := pEnableRecurse;

   If fEnableRecurse Then
   Begin
      fRecurseDirs := TRUE;
      Switches.AddNewSwitch('S','S','Recurse subdirectories',TRUE);
   End
   Else
   Begin
      fRecurseDirs := FALSE;
      Switches.RemoveParamEx('S');
   End;
End;

Procedure TApplicationMatrix.SetAppError(pAppError:TAppResult);
Var AppMessage:String;

Begin
   fAppError := pAppError;

   Case fAppError of
      ARNoErrors                : AppMessage := '';
      ARSourceNotSpecified      : AppMessage := 'Source parameter not found...';
      ARTargetNotSpecified      : AppMessage := 'Target parameter not found...';
      ARNoSourceOrTargetAllowed : AppMessage := 'No source or target allowed...';
      ARTooMuchParameters       : AppMessage := 'Too much parameters...';
      ARUnknownSwitches         : AppMessage := ' '; // Need no empty string !
      ARCannotRemoveSwitch      : AppMessage := 'Switch is not removable...';
      ARParamAlreadyDefined     : AppMessage := 'Parameter already defined...';
      ARErrorDuringProcess      : AppMessage := 'Error during process...';
      ARForceExit               : AppMessage := ' '; // Need no empty string !
      Else AppMessage := '';
   End;

   If AppMessage <> '' Then
   Begin
      If Trim(AppMessage) <> '' Then Writeln(AppMessage);
      Self.Free;
      Halt(Ord(fAppError));
   End;
End;

Constructor TApplicationMatrix.Create;
Begin
   Inherited;

   fParamMode      := MMMandatoryParam;    // All parameters must be specified
   fMatrixMode     := MMSourceOnly;        // Manage a source only application

   fVersion        := 'v1.00';
   fReleaseDate    := '05.01.2002';
   fCopyright      := '(C)1994-2002';      // Program copyright
   fAuthor         := 'F.Cheveau';         // Author of the program

   fDoneOne        := FALSE;               // Initialisations...
   fOldDirectory   := '';
   fFilesCount     := 0;
   fDirsCount      := 0;
   fConflictsCount := 0;
   fSource         := '';                  // Source file(s)
   fSPath          := '';                  // Source path & msk
   fSMask          := '';
   fTarget         := '';                  // Target file(s)
   fTPath          := '';                  // Target path & msk
   fTMask          := '';
   fTimeInit       := Now;                 // To calcule elapsed time
   fTimeEnd        := 0;

   fSwitches       := TCommandsLine.Create;// Switches management
   Switches.SwitchType := STSlash;         // Select switch '/'
   Switches.AppMatrix  := Self;            // Link Switch object with AppMatrix object

   fBackupConflict := FCIgnore;            // No files conflicts

   SetEnableBackup(TRUE);                  // Enable backup switch
   SetEnableRecurse(TRUE);                 // Enable recurse switch
   fBackupFiles    := FALSE;               // Do not create backup
   fRecurseDirs    := FALSE;               // Do not recurse directories

   TextColor(7);                           // Define base colors
   TextBackground(0);
End;

Destructor TApplicationMatrix.Destroy;
Begin
   fSwitches.Free;
End;

Procedure TApplicationMatrix.OnDisplaySummary;
Begin
   Writeln('Elapsed time : ',GetElapsedTime);
End;

Procedure TApplicationMatrix.OnCheckSwitches;
Begin
End;

Procedure TApplicationMatrix.OnValidateData(Var pSource,pTarget:String);
Begin
   pSource := pSource;
   pTarget := pTarget;
End;

Procedure TApplicationMatrix.OnBeforeProcess;
Begin
End;

Procedure TApplicationMatrix.OnAfterProcess;
Begin
End;

Procedure TApplicationMatrix.OnDefineCommands;
Begin
   With Switches do
   Begin
      AddNewCommand('Debug','Debug','Display information about application',FALSE);
      AddNewCommand('?','?','This help',TRUE);
      AddNewCommand('Author','Author','More information about the author',TRUE);
   End;
End;

Procedure TApplicationMatrix.OnDefineSwitches;
Begin
End;

{ͻ
                        MAIN PROGRAM (INITIALISATIONS)                      
 ͼ}

Procedure TApplicationMatrix.Run;
Var ErrParams:String;

Begin

{************************ STANDARD PARAMETERS *******************************}

   ClrScr;                                 // Clear Screen

   OnDisplayAppHeader;                     // Display application header

   OnDefineCommands;                       // Define commands
   OnDefineSwitches;                       // Define switches

   With Switches do
   Begin
      If TestSwitch('Debug') Then          // Display debug information
      Begin
         OnDisplayDebugInfos;
         AppError := ARForceExit;
      End;

      ErrParams := CheckAllParameters;     // Check for valid parameters
      If ErrParams <> '' Then
      Begin
         Writeln('Unknown specified switche(s) :', ErrParams);
         AppError := ARUnknownSwitches;
      End;

      If TestSwitch('Author') Then OnDisplayAuthor;  // Display author information
      If TestSwitch('?') Then OnDisplayHelp;         // Display help

      If EnableRecurse Then
      Begin
         If TestSwitch('S') Then fRecurseDirs := TRUE   // Recurse files directories
                            Else fRecurseDirs := FALSE;
      End;

      If EnableBackup Then
      Begin
         If TestSwitch('B') Then fBackupFiles := TRUE   // Create Backup for each file
                            Else fBackupFiles := FALSE;
      End;
   End;

   OnCheckSwitches; // Override this function to implement your own switches and commands

{***************************** COUNT PARAMETERS *****************************}

   fSource := Switches.GetSourceOrTargetParam(MMSourceOnly);   // Get the source param if exist
   fTarget := Switches.GetSourceOrTargetParam(MMSourceTarget); // Get the target param if exist

   If fMatrixMode = MMNoSource Then             // No sources or target
   Begin
      If fSource <> '' Then AppError := ARNoSourceOrTargetAllowed;
   End;

   If fMatrixMode = MMSourceOnly Then           // Source mode only
   Begin
      If fParamMode = MMMandatoryParam Then     // All parameters must be specified
      Begin
         If Switches.FreeParamCount = 0 Then AppError := ARSourceNotSpecified;
         If Switches.FreeParamCount > 1 Then AppError := ARTooMuchParameters;
      End;

      If fParamMode = MMAutoParam Then          // Set source as '*.*' automaticaly
      Begin
         If Switches.FreeParamCount = 0 Then
         Begin
            fSource := Dsk_FExpand('*.*');
            Switches.FreeParamCount := 1;
         End;
         If Switches.FreeParamCount > 1 Then AppError := ARTooMuchParameters;
      End;
   End;

   If fMatrixMode = MMSourceTarget Then         // Source and target mode
   Begin
      If fParamMode = MMMandatoryParam Then     // All parameters must be specified
      Begin
         If Switches.FreeParamCount = 0 Then AppError := ARSourceNotSpecified;
         If Switches.FreeParamCount = 1 Then AppError := ARTargetNotSpecified;
         If Switches.FreeParamCount > 2 Then AppError := ARTooMuchParameters;
      End;
//Autoparam and SourceTarget incompatibles...
   End;

{**************************** MANAGEMENT OF ALIAS ***************************}

   OnValidateData(fSource,fTarget);        // Override this function to execute actions before finding files...

   File_Split(fSource, fSPath, fSMask);
   File_Split(fTarget, fTPath, fTMask);

   fSPath := Dsk_ValideDirectory(fSPath);
   fTPath := Dsk_ValideDirectory(fTPath);

{***************************** CALL FIND_FILES ******************************}

   OnBeforeProcess;                        // Override this function to execute actions before finding files...

   File_Find(fSPath + fSMask, fRecurseDirs); // Search all files...

   OnAfterProcess;                         // Override this function to execute actions after processing files...

{****************************** END OF PROGRAM ******************************}

   If Not fDoneOne                         // No files found !
     Then Writeln('File(s) "',fSMask,'" not found...');

   fTimeEnd := Now;                        // Get end time...

   OnDisplaySummary;                       // Override this function to display your own summary...

   TextColor(7);                           // The End...
   TextBackground(0);
End;

{****************************************************************************}

Begin
End.
