{ͻ
                               UNIT MosBevels                               
 ͹
  CREATOR....: CHEVEAU Frdric                                             
  UPDATED BY.: CHEVEAU Frdric                                             
  CREATED THE: 08.05.2000                                                   
  COUNTRY....: Montpellier - Timeless (FRANCE)                              
  DESCRIPTION: Visual components "BevelBox, BevelText".                     
  AUTONOMOUS.: No                                                           
  UNIT SIZE..: 3.718 Lines and (93.136 Bytes compiled)                      
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosBevels;

{$Mode Objfpc}

INTERFACE

Uses
     Classes,
     MosControls,
     MosImages,
     MosTexts;

Type                                    {*Types definition*}

{****************************************************************************}

TBorderStyle = (                        {*ASingleBorder style*}
   BSNoLine,                            {*No line*}
   BSLine,                              {*Common line*}
   BSLoweredLine,                       {*3D lowered line*}
   BSRaisedLine);                       {*3D raised line*}

TSlideMethod = (                        {*Slide method*}
   SMTopToBottom,                       {*Top to bottom*}
   SMBottomToTop,                       {*Bottom to top*}
   SMLeftToRight,                       {*Left to right*}
   SMRightToLeft,                       {*Right to left*}
   SMLeftTopToRightBottom,              {*Left top to right bottom*}
   SMRightBottomToLeftTop,              {*Right bottom to left top*}
   SMRightTopToLeftBottom,              {*Right top to left bottom*}
   SMLeftBottomToRightTop);             {*Left bottom to right top*}

{****************************************************************************}
{*** ASingleBorder Class                                                  ***}
{****************************************************************************}

ASingleBorder = Class(TControl)       {*ASingleBorder class definition*}
   Private
      fColor            :Longint;       {*Border line color*}
      fLineSize         :Byte;          {*Border line size in pixels*}
      fLineStyle        :Byte;          {*Border line style (Solid, dot...)*}
         { 0 = SolidLn }
         { 1 = DottedLn}
         { 2 = CenterLn}
         { 3 = DashedLn}
         { 4 = UserBitLn 4 (User-defined line style)}
      fStyle            :TBorderStyle;  {*Border style*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetColor(C:Longint);      {*Set new line color*}
      Procedure SetLineSize(S:Byte);      {*Set new line size*}
      Procedure SetLineStyle(S:Byte);     {*Set new line style*}
      Procedure SetStyle(S:TBorderStyle); {*Set new border style*}

   Public
      Property  Color     :Longint        Read fColor     Write SetColor;
      Property  LineSize  :Byte           Read fLineSize  Write SetLineSize;
      Property  LineStyle :Byte           Read fLineStyle Write SetLineStyle;
      Property  Style     :TBorderStyle   Read fStyle     Write SetStyle;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pStyle:TBorderStyle); {*Create component with parameters*}
      Procedure   Draw; Override;                 {*Draw component*}
   End;

{****************************************************************************}
{*** ABorder Class                                                        ***}
{****************************************************************************}

ABorder = Class(TControl)               {*ABorder class definition*}
   Private
      fColor            :Longint;       {*Border background color*}
      fFilled           :Boolean;       {*Fill the border with color*}
      fInner            :ASingleBorder; {*Inner border line*}
      fOuter            :ASingleBorder; {*Outer border line*}
      fSize             :Byte;          {*Size between borders*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetColor(C:Longint);    {*Set new color*}
      Procedure SetFilled(F:Boolean);   {*Set filled border or no*}
      Procedure SetSize(W:Byte);        {*Set border width size*}

   Public
      Property  Color   :Longint       Read fColor        Write SetColor;
      Property  Filled  :Boolean       Read fFilled       Write SetFilled;
      Property  Size    :Byte          Read fSize         Write SetSize;
      Property  Inner   :ASingleBorder Read fInner;
      Property  Outer   :ASingleBorder Read fOuter;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pFilled:Boolean; pSize:Byte; OuterStyle,InnerStyle:TBorderStyle); {*Create component with parmeters*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   MoveTo(Xa,Ya:Integer); Override;{*Move component*}
      Procedure   Draw; Override;                 {*Draw component*}
   End;

{****************************************************************************}
{*** AShape Class                                                         ***}
{****************************************************************************}

AShape = Class(TControl)                {*AShape class definition*}
   Private
      fColor            :Longint;       {*Shape background color*}
      fVacuum           :Byte;          {*Space beetween shape and border*}
      fFilled           :Boolean;       {*Fill the shape with color*}
      fOpacity          :Integer;       {*Shape opacity (0 to 100%)*}
      fImage            :AImage;        {*Image in shape*}
      fShowImage        :Boolean;       {*Display image in shape*}
      fPattern          :String;        {*Use pattern to fill. Default=''*}
      fSlide            :TSlideMethod;  {*Slide method*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetColor(C:Longint);    {*Set new color*}
      Procedure SetVacuum(V:Byte);      {*Set the shape vacuum*}
      Procedure SetFilled(F:Boolean);   {*Set filled shape or no*}
      Procedure SetOpacity(O:Integer);  {*Set shape opacity*}
      Procedure SetPattern(P:String);   {*Set new pattern*}
      Procedure SetShowImage(S:Boolean);{*Set ShowImage value*}

   Public
      Property  Color        :Longint      Read fColor     Write SetColor;
      Property  Vacuum       :Byte         Read fVacuum    Write SetVacuum;
      Property  Filled       :Boolean      Read fFilled    Write SetFilled;
      Property  Opacity      :Integer      Read fOpacity   Write SetOpacity;
      Property  ShowImage    :Boolean      Read fShowImage Write SetShowImage;
      Property  Pattern      :String       Read fPattern   Write SetPattern;
      Property  Slide        :TSlideMethod Read fSlide     Write fSlide;
      Property  Image        :AImage       Read fImage;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint); {*Create component with parameters*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*Draw component*}
   End;

{****************************************************************************}
{*** TShade Class                                                         ***}
{****************************************************************************}

TShade = Class(TControl)              {*TShade class definition*}
   Private
      fFirstColor       :Longint;       {*First shade color*}
      fLastColor        :Longint;       {*Last shade color*}
      fDirection        :TDockOrientation;{*Fill the shape with color*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetDirection(D:TDockOrientation); {*Set new direction*}
      Procedure SetFirstColor(C:Longint);         {*Set first shade color*}
      Procedure SetLastColor(C:Longint);          {*Set last shade color*}

   Public
      Property  FirstColor   :Longint          Read fFirstColor  Write SetFirstColor;
      Property  LastColor    :Longint          Read fLastColor   Write SetLastColor;
      Property  Direction    :TDockOrientation Read fDirection   Write SetDirection;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pFirstColor,pLastColor:Longint; pDirection:TDockOrientation); {*Create component with parameters*}
      Procedure   Draw; Override;                 {*Draw component*}
   End;

{****************************************************************************}
{*** TBevelBox Class                                                      ***}
{****************************************************************************}

TBevelBox = Class(TVisual)              {*TBevelBox class definition*}
   Private
      fShape            :AShape;        {*Component shape*}
      fBorder           :ABorder;       {*Component border*}

   Public
      Property  Shape   :AShape      Read fShape;
      Property  Border  :ABorder     Read fBorder;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint); {*Create component with parameters*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pBorderWidth:Byte; pOuterStyle,pInnerStyle:TBorderStyle; pShapeFilled:Boolean); {*Create component with parameters*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Procedure   MoveTo(Xa,Ya:Integer); Override;{*Move component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** TPanel Class                                                         ***}
{****************************************************************************}

TPanel = Class(TBevelBox);              {*TPanel class definition*}

{****************************************************************************}
{*** TBevelText Class                                                     ***}
{****************************************************************************}

TBevelText = Class(TBevelBox)           {*TBevelText class definition*}
   Private
      fText             :Atext;         {*Text to display*}

   Public
      Property  Text    :AText       Read fText;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{}
{}
{}

IMPLEMENTATION

Uses
     MosBase,
     MosColors,
     MosConfig,
     MosGraph,
     MosMouse;

{****************************************************************************}
{*** ASingleBorder Class                                                  ***}
{****************************************************************************}

Procedure ASingleBorder.Define;
Begin
fColor    := CLLineDown;
fLineSize := 1;
fLineStyle:= SolidLn;
fStyle    := BSLine;

If Owner = Self Then Log.Add('Create new TSingleBorder ('+')',2);
End;

Constructor ASingleBorder.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);

Define;
End;

Constructor ASingleBorder.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pStyle:TBorderStyle);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);

Define;

fColor    := pColor;
fStyle    := pStyle;
End;

Procedure ASingleBorder.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;

VESA_SetColor(fColor);
VESA_SetLineStyle(fLineStyle,0,fLineSize);

Mse_Hide;

Case fStyle of                          {*Depending of the border style*}
   BSLine :                             {*Common line*}
      Begin
      VESA_Rectangle(X1,Y1,X2,Y2);
      End;
   BSLoweredLine :                      {*3D lowered line*}
      Begin
      VESA_Line(X1,Y1,X2,Y1); VESA_Line(X1,Y1,X1,Y2);
      VESA_SetColor(CLLineUp);
      VESA_Line(X1+1,Y2,X2,Y2); VESA_Line(X2,Y1,X2,Y2);
      End;
   BSRaisedLine :                       {*3D raised line*}
      Begin
      VESA_Line(X1,Y2,X2,Y2); VESA_Line(X2,Y1,X2,Y2);
      VESA_SetColor(CLLineUp);
      VESA_Line(X1,Y1,X2-1,Y1); VESA_Line(X1,Y1,X1,Y2);
      End;
   End;
Mse_Show;
End;

Procedure ASingleBorder.SetColor(C:Longint);
Begin
If C = fColor Then Exit;
fColor := C;
Invalidate;                             {*Redraw if needs*}
End;

Procedure ASingleBorder.SetLineSize(S:Byte);
Begin
If S = fLineSize Then Exit;
fLineSize := S;
Invalidate;                             {*Redraw if needs*}
End;

Procedure ASingleBorder.SetLineStyle(S:Byte);
Begin
If S = fLineStyle Then Exit;
fLineStyle := S;
Invalidate;                             {*Redraw if needs*}
End;

Procedure ASingleBorder.SetStyle(S:TBorderStyle);
Begin
If S = fStyle Then Exit;
fStyle := S;
Invalidate;                             {*Redraw if needs*}
End;

{****************************************************************************}
{*** ABorder Class                                                        ***}
{****************************************************************************}

Procedure ABorder.Define;
Begin
fColor   := CLLightGreen;
fFilled  := Yes;
fSize    := 5;

If Owner = Self Then Log.Add('Create new TBorder ('+')',2);
End;

Constructor ABorder.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fOuter := ASingleBorder.Create(AOwner);
fInner := ASingleBorder.Create(AOwner);

Define;

fInner.X1 := fInner.X1 + fSize;
fInner.Y1 := fInner.Y1 + fSize;
fInner.X2 := fInner.X2 - fSize;
fInner.Y2 := fInner.Y2 - fSize;
End;

Constructor ABorder.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pFilled:Boolean; pSize:Byte; OuterStyle,InnerStyle:TBorderStyle);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fOuter := ASingleBorder.Create(AOwner, pX1,pY1,pX2,pY2, CLLineDown, OuterStyle);
fInner := ASingleBorder.Create(AOwner, pX1+pSize,pY1+pSize,pX2-pSize,pY2-pSize, CLLineDown, InnerStyle);

Define;

fColor   := pColor;
fFilled  := pFilled;
fSize    := pSize;
End;

Procedure ABorder.MoveTo(Xa,Ya:Integer);
Begin
DrawOnModify := No;                     {*Do not redraw automaticaly*}
Inherited MoveTo(Xa,Ya);                {*Move main border without drawing*}
DrawOnModify := Yes;                    {*Redraw now automaticaly*}

fOuter.DrawOnModify := No;              {*Move outer border without drawing*}
fOuter.MoveTo(Xa,Ya);
fOuter.DrawOnModify := Yes;

fInner.DrawOnModify := No;              {*Move inner border without drawing*}
fInner.MoveTo(Xa+fSize,Ya+fSize);
fInner.DrawOnModify := Yes;

Invalidate;                             {*Redraw if needs*}
End;

Destructor ABorder.Destroy;
Begin
fInner.Destroy;
fOuter.Destroy;
Inherited Destroy;
End;

Procedure ABorder.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;                         {*Call the parent Draw method*}

If (fSize > 1) And (fFilled) Then
   Begin                                {*Fill border with color*}
   Mse_Hide;
   VESA_SetFillStyle(1,fColor);
   VESA_Bar(X1,Y1,X2,Y1+fSize);
   VESA_Bar(X1,Y2-fSize,X2,Y2);
   VESA_Bar(X1,Y1,X1+fSize,Y2);
   VESA_Bar(X2-fSize,Y1,X2,Y2);
   Mse_Show;
   End;

fOuter.Draw;                            {*Draw outer border*}
fInner.Draw;                            {*Draw inner border*}
End;

Procedure ABorder.SetColor(C:Longint);
Begin
If C = fColor Then Exit;
fColor := C;
Invalidate;                             {*Redraw if needs*}
End;

Procedure ABorder.SetFilled(F:Boolean);
Begin
If F = fFilled Then Exit;
fFilled := F;
Invalidate;                             {*Redraw if needs*}
End;

Procedure ABorder.SetSize(W:Byte);
Begin
If W > Width-1 Then W := Width-1;       {*Maximum width*}
If W = fSize Then Exit;                 {*Size not changed*}
fSize := W;

fInner.X1 := fOuter.X1 + fSize;
fInner.Y1 := fOuter.Y1 + fSize;
fInner.X2 := fOuter.X2 - fSize;
fInner.Y2 := fOuter.Y2 - fSize;

Invalidate;                             {*Redraw if needs*}
End;

{****************************************************************************}
{*** AShape Class                                                         ***}
{****************************************************************************}

Procedure AShape.Define;
Begin
fColor   := CLLightGrey;
fVacuum  := 0;
fFilled  := Yes;
fOpacity := 100;
fPattern := '';
fImage.Width  := Width;
fImage.Height := Height;
fShowImage    := No;

If Owner = Self Then Log.Add('Create new TShape ('+')',2);
End;

Constructor AShape.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fImage := AImage.Create(AOwner);

Define;
End;

Constructor AShape.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fImage := AImage.Create(AOwner, '',pX1,pY1);

Define;

fColor   := pColor;
End;

Destructor AShape.Destroy;
Begin
fImage.Destroy;
Inherited Destroy;
End;

Procedure AShape.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;
{Il faut grer l'opacit... Transparence avec le fond}

If fFilled Then                         {*Fill shape with color*}
   Begin
   Mse_Hide;
   VESA_SetFillStyle(1,fColor);
   VESA_Bar(X1+fVacuum, Y1+fVacuum, X2-fVacuum, Y2-fVacuum);
   Mse_Show;
   End;

If fPattern <> '' Then                  {*Fill shape with pattern*}
   Begin
{Display pattern}
   End;

If (fShowImage) And (fImage.FileName <> '') Then
   Begin                                {*Fill shape with image*}
   fImage.Draw;
   End;
End;

Procedure AShape.SetColor(C:Longint);
Begin
If C = fColor Then Exit;
fColor := C;
Invalidate;                             {*Redraw if needs*}
End;

Procedure AShape.SetVacuum(V:Byte);
Begin
If V = fVacuum Then Exit;
fVacuum := V;
Invalidate;                             {*Redraw if needs*}
End;

Procedure AShape.SetFilled(F:Boolean);
Begin
If F = fFilled Then Exit;
fFilled := F;
Invalidate;                             {*Redraw if needs*}
End;

Procedure AShape.SetOpacity(O:Integer);
Begin
If O = fOpacity Then Exit;
fOpacity := O;
Invalidate;                             {*Redraw if needs*}
End;

Procedure AShape.SetPattern(P:String);
Begin
If P = fPattern Then Exit;
fPattern := P;
Invalidate;                             {*Redraw if needs*}
End;

Procedure AShape.SetShowImage(S:Boolean);
Begin
If S = fShowImage Then Exit;
fShowImage := S;
Invalidate;                             {*Redraw if needs*}
End;

{****************************************************************************}
{*** TShade Class                                                         ***}
{****************************************************************************}

Procedure TShade.Define;
Begin
fFirstColor := CLBlue;
fLastColor  := CLLightBlue;
fDirection  := DOHorizontal;

If Owner = Self Then Log.Add('Create new TShade ('+')',2);
End;

Constructor TShade.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);

Define;
End;

Constructor TShade.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pFirstColor,pLastColor:Longint; pDirection:TDockOrientation);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);

Define;

fFirstColor := pFirstColor;
fLastColor  := pLastColor;
fDirection  := pDirection;
End;

Procedure TShade.Draw;
Var A                :Integer;          {*Loop*}
    Px               :Integer;          {*Position to draw*}
    QttPixel         :Integer;          {*Quantity of pixels*}
    Save_Color       :LongInt;
    R,G,B,R1,G1,B1,R2,G2,B2:Byte;       {*Color components*}

Begin
If Not Visible Then Exit;
Inherited Draw;

Save_Color:=VESA_GetColor;              {*Save current color*}
If fDirection = DOHorizontal Then QttPixel := Width + 1;
If fDirection = DOVertical   Then QttPixel := Height+ 1;

Mse_Hide;
VESA_GetRGBColor(fFirstColor,R1,G1,B1);
VESA_GetRGBColor(fLastColor, R2,G2,B2);
{R:=R1; G:=G1; B:=B1;}

For A:=0 to QttPixel-1 do
   Begin                                {*Draw shade*}
   If QttPixel-1 > 0 Then R := R1+(R2-R1)*A Div (QttPixel-1) Else R:=R1;
   If QttPixel-1 > 0 Then G := G1+(G2-G1)*A Div (QttPixel-1) Else G:=G1;
   If QttPixel-1 > 0 Then B := B1+(B2-B1)*A Div (QttPixel-1) Else B:=B1;
   VESA_SetColor(VESA_SetRGBColor(R,G,B));
   If fDirection = DOHorizontal Then    {*Horizontal shade*}
      Begin
      Px := A + X1;
      VESA_Line(Px,Y1,Px,Y2);
      End;
   If fDirection = DOVertical Then      {*Vertical shade*}
      Begin
      Px := A + Y1;
      VESA_Line(X1,Px,X2,Px);
      End;
   End;

Mse_Show;
VESA_SetColor(Save_Color)               {*Restore preview color*}
End;

Procedure TShade.SetDirection(D:TDockOrientation);
Begin
If D = fDirection Then Exit;
fDirection := D;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TShade.SetFirstColor(C:Longint);
Begin
If C = fFirstColor Then Exit;
fFirstColor := C;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TShade.SetLastColor(C:Longint);
Begin
If C = fLastColor Then Exit;
fLastColor := C;
Invalidate;                             {*Redraw if needs*}
End;

{****************************************************************************}
{*** TBevelBox Class                                                      ***}
{****************************************************************************}

Constructor TBevelBox.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fShape  := AShape.Create (AOwner);
fBorder := ABorder.Create(AOwner);
End;

Constructor TBevelBox.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fShape  := AShape.Create (AOwner, pX1,pY1,pX2,pY2, pColor);
fBorder := ABorder.Create(AOwner, pX1,pY1,pX2,pY2, pColor, Yes, 1, BSNoLine, BSNoLine);
End;

Constructor TBevelBox.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pBorderWidth:Byte; pOuterStyle,pInnerStyle:TBorderStyle; pShapeFilled:Boolean);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fShape  := AShape.Create (AOwner, pX1,pY1,pX2,pY2, pColor);
fBorder := ABorder.Create(AOwner, pX1,pY1,pX2,pY2, pColor, Yes, pBorderWidth, pOuterStyle,pInnerStyle);

fShape.Filled := pShapeFilled;
End;

Destructor TBevelBox.Destroy;
Begin
fBorder.Destroy;
fShape.Destroy;
Inherited Destroy;
End;

Procedure TBevelBox.MoveTo(Xa,Ya:Integer);
Begin
DrawOnModify := No;                     {*Do not redraw automaticaly*}
Inherited MoveTo(Xa,Ya);                {*Move main border without drawing*}
DrawOnModify := Yes;                    {*Redraw now automaticaly*}

//fBorder.DrawOnModify := No;             {*Move border without drawing*}
fBorder.MoveTo(Xa,Ya);
//fBorder.DrawOnModify := Yes;

fShape.DrawOnModify := No;              {*Move shape without drawing*}
//fShape.MoveTo(Xa,Ya);
fShape.DrawOnModify := Yes;

Invalidate;                             {*Redraw if needs*}
End;

Procedure TBevelBox.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;

//fShape.Draw;
//fBorder.Draw;
End;

Function TBevelBox.GetMsg:TCompMessage;
Begin
Result := CMNone;
If Not Enabled Then Exit;

Result := Inherited GetMsg;
End;

{****************************************************************************}
{*** TBevelText Class                                                     ***}
{****************************************************************************}

Constructor TBevelText.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fText := AText.Create(AOwner);
End;

Destructor TBevelText.Destroy;
Begin
fText.Destroy;
Inherited Destroy;
End;

Procedure TBevelText.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;

fText.Draw;
End;

Function TBevelText.GetMsg:TCompMessage;
Begin
Result := CMNone;
If Not Enabled Then Exit;

Result := Inherited GetMsg;
End;

{******************************* NO MAIN CODE *******************************}

Begin
End.
