{ͻ
                              UNIT MosButtons                               
 ͹
  CREATOR....: CHEVEAU Frdric                                             
  UPDATED BY.: CHEVEAU Frdric                                             
  CREATED THE: 08.05.2000                                                   
  COUNTRY....: Montpellier - Timeless (FRANCE)                              
  DESCRIPTION: Visual components "Buttons,Cyclers,CheckBox,RadioBox".       
  AUTONOMOUS.: No                                                           
  UNIT SIZE..: 3.718 Lines and (93.136 Bytes compiled)                      
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosButtons;

{$Mode Objfpc}

INTERFACE

Uses
     Classes,
     MosBevels,
     MosControls,
     MosImages,
     MosTexts;

Type                                    {*Types definition*}

{****************************************************************************}

TSensitiveStates = (                    {*Sensitive component states*}
   SSMouseOver,                         {*Mouse is over component*}
   SSPreSelected,                       {*Component only preselected*}
   SSSelected,                          {*Component selected*}
   SSUnSelected);                       {*Component unselected*}

{****************************************************************************}
{*** TSensitive Class                                                     ***}
{****************************************************************************}

TSensitive = Class(TVisual)             {*Sensitive class definition*}
   Private
      fState            :TSensitiveStates; {*Component states*}
      fImgMouseOver     :String;        {*Image when mouse is over*}
      fImgPreSelected   :String;        {*Image when preselected*}
      fImgSelected      :String;        {*Image when selected*}
      fImgUnSelected    :String;        {*Image when not selected*}
      fImage            :AImage;        {*Image component*}
      fBevel            :TBevelBox;     {*BevelBox component*}
      fUseImages        :Boolean;       {*Use images for displaying*}
      fLockDown         :Boolean;       {*Lock component when down (selected)*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetState(S:TSensitiveStates);{*Set the component state*}
      Procedure SetImgMouseOver(I:String);   {*Set new mouse over image*}
      Procedure SetImgPreSelected(I:String); {*Set new preselected image*}
      Procedure SetImgSelected(I:String);    {*Set new selected image*}
      Procedure SetImgUnSelected(I:String);  {*Set new unselected image*}
      Procedure SetUseImages(U:Boolean);     {*Define images or box to draw*}

      Property  Image         :AImage      Read fImage;
      Property  Bevel         :TBevelBox   Read fBevel;

   Public
      Property  State         :TSensitiveStates Read fState          Write SetState;
      Property  LockDown      :Boolean          Read fLockDown       Write fLockDown;
      Property  UseImages     :Boolean          Read fUseImages      Write SetUseImages;
      Property  ImgMouseOver  :String           Read fImgMouseOver   Write SetImgMouseOver;
      Property  ImgPreSelected:String           Read fImgPreSelected Write SetImgPreSelected;
      Property  ImgSelected   :String           Read fImgSelected    Write SetImgSelected;
      Property  ImgUnSelected :String           Read fImgUnSelected  Write SetImgUnSelected;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pLockDown:Boolean); {*Define component*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Procedure   DrawMouseOver;  Virtual;        {*Draw component when mouse is over*}
      Procedure   DrawPreSelected;Virtual;        {*Draw preselected component*}
      Procedure   DrawSelected;   Virtual;        {*Draw selected component*}
      Procedure   DrawUnSelected; Virtual;        {*Draw unselected component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** TCustomButton Class                                                  ***}
{****************************************************************************}

TCustomButton = Class(TSensitive)       {*Custom button component*}
   Private
      Procedure Define;                 {*Define base variables*}

   Public
      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pState:TSensitiveStates);{*Define component*}
      Procedure   DrawMouseOver;  Override;       {*Draw component when mouse is over*}
      Procedure   DrawPreSelected;Override;       {*Draw preselected component*}
      Procedure   DrawSelected;   Override;       {*Draw selected component*}
      Procedure   DrawUnSelected; Override;       {*Draw unselected component*}
   End;

{****************************************************************************}
{*** BButton Class                                                        ***}
{****************************************************************************}

BButton = Class(TCustomButton)          {*Button component*}
   Private
      fText             :AText;         {*Text to display*}

   Public
      Property  Text         :AText   Read fText;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pWidth,pHeight:Integer; pPenColor,pBgColor:Longint; pState:TSensitiveStates; pCaption:String);{*Define component*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   DrawMouseOver;  Override;       {*Draw component when mouse is over*}
      Procedure   DrawPreSelected;Override;       {*Draw preselected component*}
      Procedure   DrawSelected;   Override;       {*Draw selected component*}
      Procedure   DrawUnSelected; Override;       {*Draw unselected component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** BCycler Class                                                        ***}
{****************************************************************************}

BCycler = Class(BButton)                {*Cycler component*}
   Private
      fStrings          :String;        {*Strings to display*}
      fIndex            :Byte;          {*Index of displayed string*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetStrings(S:String);   {*Set new strings*}
      Procedure SetIndex(I:Byte);       {*Set new string index*}

   Public
      Property  Strings      :String  Read fStrings       Write SetStrings;
      Property  Index        :Byte    Read fIndex         Write SetIndex;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pWidth,pHeight:Integer; pPenColor,pBgColor:Longint; pState:TSensitiveStates; pStrings:String; pIndex:Byte);{*Define component*}
      Procedure   DrawMouseOver;  Override;       {*Draw component when mouse is over*}
      Procedure   DrawPreSelected;Override;       {*Draw preselected component*}
      Procedure   DrawSelected;   Override;       {*Draw selected component*}
      Procedure   DrawUnSelected; Override;       {*Draw unselected component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** TCustomBox Class                                                     ***}
{****************************************************************************}

TCustomBox = Class(TSensitive)          {*TCustomBox base component*}
   Private
      fText             :AText;         {*Text to display*}
      fTextPosition     :THPosition;    {*Text position*}
      fTextSelection    :Boolean;       {*Select component with text*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetTextPosition(P:THPosition);{*Set the text position*}

   Public
      Property  Text         :AText       Read fText;
      Property  TextPosition :THPosition  Read fTextPosition  Write SetTextPosition;
      Property  TextSelection:Boolean     Read fTextSelection Write fTextSelection;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1:Integer; pCaption:String; pState:TSensitiveStates; pTextPosition:THPosition);
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** BCheckBox Class                                                      ***}
{****************************************************************************}

BCheckBox = Class(TCustomBox)           {*CheckBox component*}
   Private
   Protected
   Public
      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1:Integer; pCaption:String; pState:TSensitiveStates; pTextPosition:THPosition);
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** BRadioBox Class                                                      ***}
{****************************************************************************}

BRadioBox = Class(TCustomBox)           {*RadioBox component*}
   Private
      fGroup            :Integer;       {*Component group*}

      Procedure Define;                 {*Define base variables*}

   Protected

   Public
      Property  Group        :Integer  Read fGroup    Write fGroup;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1:Integer; pCaption:String; pState:TSensitiveStates; pTextPosition:THPosition; pGroup:Integer);
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{}
{}
{}

IMPLEMENTATION

Uses
     MosBase,
     MosColors,
     MosConfig,
     MosStrings;

{****************************************************************************}
{*** TSensitive Class                                                     ***}
{****************************************************************************}

Procedure TSensitive.Define;
Begin
fState          := SSUnSelected;
fUseImages      := No;
fImgMouseOver   := '';
fImgPreSelected := '';
fImgSelected    := '';
fImgUnSelected  := '';

If Owner = Self Then Log.Add('Create new TSensitive ('+')',2);
End;

Constructor TSensitive.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fImage := AImage.Create(AOwner);
fBevel := TBevelBox.Create(AOwner);

Define;

fLockDown := No;
End;

Constructor TSensitive.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pLockDown:Boolean);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fImage := AImage.Create(AOwner, '',pX1,pY1);
fBevel := TBevelBox.Create(AOwner, pX1,pY1,pX2,pY2, CLLightGrey, 0, BSRaisedLine,BSNoLine,Yes);

Define;

fLockDown     := pLockDown;
fImage.Width  := Width;
fImage.Height := Height;
End;

Destructor TSensitive.Destroy;
Begin
fImage.Destroy;
fBevel.Destroy;
Inherited Destroy;
End;

Procedure TSensitive.SetState(S:TSensitiveStates);
Begin
If S = fState Then Exit;
fState := S;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TSensitive.SetUseImages(U:Boolean);
Begin
If U = fUseImages Then Exit;
fUseImages := U;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TSensitive.SetImgMouseOver(I:String);
Begin
If I=fImgMouseOver Then Exit;
fImgMouseOver:=I;
If (fState = SSMouseOver) And (Displayed) Then Draw;
End;

Procedure TSensitive.SetImgPreSelected(I:String);
Begin
If I=fImgPreSelected Then Exit;
fImgPreSelected:=I;
If (fState = SSPreSelected) And (Displayed) Then Draw;
End;

Procedure TSensitive.SetImgSelected(I:String);
Begin
If I=fImgSelected Then Exit;
fImgSelected:=I;
If (fState = SSSelected) And (Displayed) Then Draw;
End;

Procedure TSensitive.SetImgUnSelected(I:String);
Begin
If I=fImgUnSelected Then Exit;
fImgUnSelected:=I;
If (fState = SSUnSelected) And (Displayed) Then Draw;
End;

Procedure TSensitive.DrawMouseOver;
Var SaveColor        :Longint;
Begin
If Not Visible Then Exit;
If fUseImages Then
   Begin
   fImage.FileName := fImgMouseOver;
   fImage.LoadFromFile;
   fImage.Draw;
   End
Else
   Begin
   SaveColor := fBevel.Shape.Color;
   fBevel.Border.Outer.Style := BSRaisedLine;
   fBevel.Shape.Color := CLPaper;
   DrawOnModify := No;
   fBevel.Shape.Color := SaveColor;
   DrawOnModify := Yes;
   End;
End;

Procedure TSensitive.DrawPreSelected;
Var SaveColor        :Longint;
Begin
If Not Visible Then Exit;
If fUseImages Then
   Begin
   fImage.FileName := fImgPreSelected;
   fImage.LoadFromFile;
   fImage.Draw;
   End
Else
   Begin
   SaveColor := fBevel.Shape.Color;
   fBevel.Border.Outer.Style := BSRaisedLine;
   fBevel.Shape.Color := CLFlamand;
   DrawOnModify := No;
   fBevel.Shape.Color := SaveColor;
   DrawOnModify := Yes;
   End;
End;

Procedure TSensitive.DrawSelected;
Var SaveColor        :Longint;
Begin
If Not Visible Then Exit;
If fUseImages Then
   Begin
   fImage.FileName := fImgSelected;
   fImage.LoadFromFile;
   fImage.Draw;
   End
Else
   Begin
   SaveColor := fBevel.Shape.Color;
   fBevel.Border.Outer.Style := BSLoweredLine;
   fBevel.Shape.Color := CLGrey;
   DrawOnModify := No;
   fBevel.Shape.Color := SaveColor;
   DrawOnModify := Yes;
   End;
End;

Procedure TSensitive.DrawUnSelected;
Begin
If Not Visible Then Exit;
If fUseImages Then
   Begin
   fImage.FileName := fImgUnSelected;
   fImage.LoadFromFile;
   fImage.Draw;
   End
Else
   Begin
   fBevel.Border.Outer.Style := BSRaisedLine;
   fBevel.Shape.Draw;
   End;
End;

Procedure TSensitive.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;

Case fState of
   SSMouseOver   : DrawMouseOver;
   SSPreSelected : DrawPreSelected;
   SSSelected    : DrawSelected;
   SSUnSelected  : DrawUnSelected;
   End;
End;

Function TSensitive.GetMsg:TCompMessage;
Var MsgMouse:TCompMessage;

Begin
Result := CMNone;
If Not Enabled Then Exit;               {*Component disabled*}

MsgMouse := Mouse.GetMsg;
If MsgMouse = CMMouseOver Then
   Begin
   End;

{Case fState of
   SSMouseOver   : DrawMouseOver;
   SSPreSelected : DrawPreSelected;
   SSSelected    : DrawSelected;
   SSUnSelected  : DrawUnSelected;
   End;}

//Result := Inherited GetMsg;
End;

{****************************************************************************}
{*** TCustomBox Class                                                     ***}
{****************************************************************************}

Procedure TCustomBox.Define;
Begin
fTextPosition := TPOnLeft;

If Owner = Self Then Log.Add('Create new TCustomBox ('+')',2);
End;

Constructor TCustomBox.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fText := AText.Create(AOwner);
Define;
End;

Constructor TCustomBox.Create(AOwner:TComponent; pX1,pY1:Integer; pCaption:String; pState:TSensitiveStates; pTextPosition:THPosition);
Begin
Inherited Create(AOwner, pX1,pY1,pX1,pY1, Yes);
fText := AText.Create(AOwner, pX1,pY1, pCaption);

Define;

State         := pState;
fTextPosition := pTextPosition;
End;

Destructor TCustomBox.Destroy;
Begin
fText.Destroy;
Inherited Destroy;
End;

Procedure TCustomBox.SetTextPosition(P:THPosition);
Begin
If P = fTextPosition Then Exit;
fTextPosition := P;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TCustomBox.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;
End;

Function TCustomBox.GetMsg:TCompMessage;
Begin
Result:=CMNone;
If Not Enabled Then Exit;
//Result := Inherited GetMsg;
End;

{****************************************************************************}
{*** BCheckBox Class                                                      ***}
{****************************************************************************}

Constructor BCheckBox.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
End;

Constructor BCheckBox.Create(AOwner:TComponent; pX1,pY1:Integer; pCaption:String; pState:TSensitiveStates; pTextPosition:THPosition);
Begin
Inherited Create(AOwner, pX1,pY1, pCaption, pState, pTextPosition);
End;

Procedure BCheckBox.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;
End;

Function BCheckBox.GetMsg:TCompMessage;
Begin
Result:=CMNone;
If Not Enabled Then Exit;
//Result := Inherited GetMsg;
End;

{****************************************************************************}
{*** BRadioBox Class                                                      ***}
{****************************************************************************}

Procedure BRadioBox.Define;
Begin
fGroup := 0;

If Owner = Self Then Log.Add('Create new TRadioBox ('+')',2);
End;

Constructor BRadioBox.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
Define;
End;

Constructor BRadioBox.Create(AOwner:TComponent; pX1,pY1:Integer; pCaption:String; pState:TSensitiveStates; pTextPosition:THPosition; pGroup:Integer);
Begin
Inherited Create(AOwner, pX1,pY1, pCaption, pState, pTextPosition);

Define;

fGroup := pGroup;
End;

Procedure BRadioBox.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;
End;

Function BRadioBox.GetMsg:TCompMessage;
Begin
Result:=CMNone;
If Not Enabled Then Exit;
//Result := Inherited GetMsg;
End;

{****************************************************************************}
{*** TCustomButton Class                                                  ***}
{****************************************************************************}

Procedure TCustomButton.Define;
Begin
UseImages := No;

If Owner = Self Then Log.Add('Create new TCustomButton ('+')',2);
End;

Constructor TCustomButton.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);

Define;
End;

Constructor TCustomButton.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pColor:Longint; pState:TSensitiveStates);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2, No);

Define;

State             := pState;
Bevel.Shape.Color := pColor;
End;

Procedure TCustomButton.DrawMouseOver;
Begin
If Not Visible Then Exit;
Inherited DrawMouseOver;
End;

Procedure TCustomButton.DrawPreSelected;
Begin
If Not Visible Then Exit;
Inherited DrawPreSelected;
End;

Procedure TCustomButton.DrawSelected;
Begin
If Not Visible Then Exit;
Inherited DrawSelected;
End;

Procedure TCustomButton.DrawUnSelected;
Begin
If Not Visible Then Exit;
Inherited DrawUnSelected;
End;

{****************************************************************************}
{*** BButton Class                                                        ***}
{****************************************************************************}

Constructor BButton.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);

fText := AText.Create(AOwner);
End;

Constructor BButton.Create(AOwner:TComponent; pX1,pY1,pWidth,pHeight:Integer; pPenColor,pBgColor:Longint; pState:TSensitiveStates; pCaption:String);
Begin
Inherited Create(AOwner, pX1,pY1,pX1,pY1, pBgColor, pState);
fText := AText.Create(AOwner, pX1,pY1,pWidth, pPenColor, pCaption, TPOnCenter);

Width  := pWidth;
Height := pHeight;
End;

Destructor BButton.Destroy;
Begin
fText.Destroy;
Inherited Destroy;
End;

Procedure BButton.DrawMouseOver;
Begin
If Not Visible Then Exit;
Inherited DrawMouseOver;

If Not UseImages Then
   Begin
   fText.Draw;
   End;
End;

Procedure BButton.DrawPreSelected;
Begin
If Not Visible Then Exit;
Inherited DrawPreSelected;

If Not UseImages Then
   Begin
   fText.Draw;
   End;
End;

Procedure BButton.DrawSelected;
Begin
If Not Visible Then Exit;
Inherited DrawSelected;

If Not UseImages Then
   Begin
   fText.Draw;
   End;
End;

Procedure BButton.DrawUnSelected;
Begin
If Not Visible Then Exit;
Inherited DrawUnSelected;

If Not UseImages Then
   Begin
   fText.Draw;
   End;
End;

Function BButton.GetMsg:TCompMessage;
Begin
Result:=CMNone;
If Not Enabled Then Exit;

fText.GetMsg;                           {*Manage text*}

//Result := Inherited GetMsg;
End;

{****************************************************************************}
{*** BCycler Class                                                        ***}
{****************************************************************************}

Procedure BCycler.Define;
Begin
fIndex   := 1;                          {*Display first string*}
fStrings := 'Caption 1|Caption 2';

If Owner = Self Then Log.Add('Create new TCycler ('+')',2);
End;

Constructor BCycler.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);

Define;
End;

Constructor BCycler.Create(AOwner:TComponent; pX1,pY1,pWidth,pHeight:Integer; pPenColor,pBgColor:Longint; pState:TSensitiveStates; pStrings:String; pIndex:Byte);
Var St:String;
Begin
St := Str_Get(pStrings,fIndex,'|');
Inherited Create(AOwner, pX1,pY1,pWidth,pHeight, pPenColor,pBgColor, pState, St);

Define;

fStrings := pStrings;
fIndex   := pIndex;
End;

Procedure BCycler.SetStrings(S:String);
Begin
If S = fStrings Then Exit;
fStrings := S;
Invalidate;                             {*Redraw if needs*}
End;

Procedure BCycler.SetIndex(I:Byte);
Begin
If I = fIndex Then Exit;
fIndex := I;
Invalidate;                             {*Redraw if needs*}
End;

Procedure BCycler.DrawMouseOver;
Begin
If Not Visible Then Exit;
Inherited DrawMouseOver;

If Not UseImages Then
   Begin
   End;
End;

Procedure BCycler.DrawPreSelected;
Begin
If Not Visible Then Exit;
Inherited DrawPreSelected;

If Not UseImages Then
   Begin
   End;
End;

Procedure BCycler.DrawSelected;
Begin
If Not Visible Then Exit;
Inherited DrawSelected;

If Not UseImages Then
   Begin
   End;
End;

Procedure BCycler.DrawUnSelected;
Begin
If Not Visible Then Exit;
Inherited DrawUnSelected;

If Not UseImages Then
   Begin
   End;
End;

Function BCycler.GetMsg:TCompMessage;
Begin
Result:=CMNone;
If Not Enabled Then Exit;

//Result := Inherited GetMsg;
End;

{******************************* NO MAIN CODE *******************************}

Begin
End.
