{ͻ
                              UNIT MosControls                              
 ͹
  CREATOR....: CHEVEAU Frdric                                             
  UPDATED BY.: CHEVEAU Frdric                                             
  CREATED THE: 08.05.2000                                                   
  COUNTRY....: Montpellier - Timeless (FRANCE)                              
  DESCRIPTION: All MOS visual components base library.                      
  AUTONOMOUS.: No                                                           
  UNIT SIZE..: 3.718 Lines and (93.136 Bytes compiled)                      
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosControls;

{$Mode Objfpc}

INTERFACE

Uses
     Classes,
     MosConfig,
     MosGadgets,
     MosGraphics,
     MosTime;

Const
   MaxMessages  = 20;                   {*Maximum of managed messages*}

Type                                    {*Types definition*}

{****************************************************************************}

TDisplayHint = (                        {*Display Hint method*}
   DHTop,
   DHOver,
   DHCorner,
   DHMouse);

TAlign = (                              {*Alignment kind*}
   ALBottom,
   ALClient,
   ALLeft,
   ALNone,
   ALRight,
   ALTop);

TShiftState = Set of (ssShift, ssAlt, ssCtrl, ssLeft, ssRight, ssMiddle, ssDouble);

TMouseButton = (X_mbLeft, X_mbRight, X_mbMiddle);

(*TMouseButton = (                         {*Mouse buttons definition*}
   MBNone,                              {*No button clicked*}
   MBLeft,                              {*Left button*}
   MBCenter,                            {*Center button*}
   MBRight,                             {*Right button*}
   MBIgnore);                           {*Ignore mouse buttons*}*)

TCompMessage = (                        {*Components messages*}
   CMNone,                              {*No messages*}
  CMClosed,                            {*Closable component*}
  CMSwapped,                           {*Swappable component*}
  CMExtended,                          {*Extendable component*}
   CMResized,                           {*Component is resized*}
   CMScaled,                            {*Component is scaled*}
  CMActivated,                         {*Component activated*}
  CMInactivated,                       {*Component inactivated*}
   CMHidden,                            {*Hide component*}
   CMDisplayed,                         {*Display component*}
   CMMoved,                             {*Movable component*}
   CMDisabled,                          {*Component has been disabled*}
   CMEnabled,                           {*Component has been enabled*}
  CMClicked,                           {*Component clicked*}
  CMDblClicked,                        {*Component double clicked*}
   CMSelected,                          {*Component selected*}
  CMUnSelected,                        {*Radio Box set to unselected*}
  CMModified,                          {*Component (data) modified or changed*}
  CMKeyPressed,                        {*Key pressed (keys sensitive components)*}
  CMMouseOver,                         {*Mouse is over the component*}
  CMTextSelected,                      {*Text selected in InputText*}
  CMTextCopyied,                       {*Text copyied in InputText*}
  CMTextCutted,                        {*Text cutted in InputText*}
  CMTextPasted,                        {*Text pasted in InputText*}
  CMColumnMoved,                       {*Column moved in List component*}
  CMColumnResized,                     {*Column resized in List component*}
  CMCellModified,                      {*Cell modified in List component*}
  CMFirstFrame,                        {*First frame animation displayed*}
  CMLastFrame);                        {*Last frame animation displayed*}

{****************************************************************************}

TConstraints = Record                   {*Constraints for TControl*}
   MaxHeight            :Integer;
   MaxWidth             :Integer;
   MinHeight            :Integer;
   MinWidth             :Integer;
   End;

TAnchors = Record                       {*Anchors for TControl*}
   AKLeft               :Boolean;
   AKTop                :Boolean;
   AKRight              :Boolean;
   AKBottom             :Boolean;
   End;

TMouseEvent     = Procedure(Sender:TObject; Button:TMouseButton; Shift:TShiftState; X,Y:Integer) of Object;

TMouseMoveEvent = Procedure(Sender:TObject; Shift:TShiftState; X,Y:Integer) of Object;

{****************************************************************************}
{*** TMessage Class                                                       ***}
{****************************************************************************}

TMessagesList = Array[1..MaxMessages] of TCompMessage;

TMessage = Class(TObject)               {*Message management class*}
   Private
      fMessageList      :TMessagesList;
      fCount            :Integer;       {*Count messages*}

   Public
      Property MessageList:TMessagesList Read fMessageList;
      Property Count      :Integer       Read fCount;

      Constructor Create;               {*Create a TMessage class*}
      Function    PutToList(Msg:TCompMessage):Boolean; {*Put message into list*}
      Function    GetFromList:TCompMessage;       {*Get message from list*}
      Procedure   Clear;                          {*Flush messages from list*}
      Procedure   MoveFrom(NewList:TMessage);     {*Move messages from a list to current list*}
   End;

{****************************************************************************}
{*** TControl Class                                                       ***}
{****************************************************************************}

TControl = Class(TComponent)            {*TControl is visual component class*}
   Private
      fX1,fY1,fX2,fY2   :Integer;       {*Coordinates*}
      fEnabled          :Boolean;       {*Enable or disable component*}
      fVisible          :Boolean;       {*Display or hide component*}
      fDisplayed        :Boolean;       {*Component currently displayed*}
      fSaveBg           :Boolean;       {*Component background need to be refreshed*}
      fScale            :Integer;       {*Scale value for display*}
      fParent           :TControl;      {*Parent visual component*}
      fRefresh          :TRefresh;      {*Refresh screen under component*}
      fMessage          :TMessage;      {*Messages and events management*}
      fFocused          :Boolean;       {*Focus is on component*}
      fFocusEnabled     :Boolean;       {*This component can have a focus*}
      fDrawOnModify     :Boolean;       {*Draw or no after property modification*}
      fAlign            :TAlign;        {*Component alignment*}
      fControls         :TList;         {*Visual child controls list*}
      fTabOrder         :Integer;       {*Component tab number*}
      fTabStop          :Boolean;       {*Component can be selected by tab key*}
      fDebug            :Integer;       {*Debug value for miscellaneous use*}

      fConstraints      :TConstraints;  {*Component range size*}
      fAnchors          :TAnchors;      {*Component anchors management*}

      fOnChange         :TNotifyEvent;  {*Component events*}
      fOnClick          :TNotifyEvent;
      fOnDblClick       :TNotifyEvent;
      fOnMouseDown      :TMouseEvent;
      fOnMouseMove      :TMouseMoveEvent;
      fOnMouseUp        :TMouseEvent;

      Procedure Define;                 {*Define base variables*}
      Procedure FontChanged(Sender: TObject);
      Procedure Change;
      Procedure Click;
      Procedure DblClick;
      Procedure MouseMove(Shift:TShiftState; X,Y:Integer);
      Procedure MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
      Procedure MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);

      Function  GetChildControlCount:Integer;  {*Count all defined child controls*}
      Function  GetControlCount:Integer;       {*Count direct defined child controls*}
      Function  GetControl(Index:Integer):TControl; {*Get specified control*}
{     Procedure AlignControl(AControl:TControl);} {*To align controls*}

   Protected
      Property  DrawOnModify:Boolean  Read fDrawOnModify Write fDrawOnModify;

      Procedure SetX1(X:Integer); Virtual;     {*Set the component pos X1*}
      Procedure SetY1(Y:Integer); Virtual;     {*Set the component pos Y1*}
      Procedure SetX2(X:Integer); Virtual;     {*Set the component pos right*}
      Procedure SetY2(Y:Integer); Virtual;     {*Set the component pos bottom*}
      Procedure SetLeft(L:Integer); Virtual;   {*Set the component pos left*}
      Procedure SetTop(T:Integer); Virtual;    {*Set the component pos top*}
      Function  GetWidth:Integer;              {*Get the component width*}
      Procedure SetWidth(W:Integer); Virtual;  {*Set the component width*}
      Function  GetHeight:Integer;             {*Get the component height*}
      Procedure SetHeight(H:Integer); Virtual; {*Set the component height*}
      Procedure SetVisible(V:Boolean);         {*Set component visible or not*}
      Procedure SetEnabled(E:Boolean); Virtual;{*Enable or disable component*}
      Procedure SetTabOrder(O:Integer);Virtual;{*Set component tab order*}
      Procedure SetTabStop(T:Boolean);Virtual; {*Set component tab stop*}
      Procedure SetScale(S:Integer); Virtual;  {*Set component scale value*}
      Procedure SetSaveBg(S:Boolean); Virtual; {*Specify to save or not the background*}
      Procedure SetAlign(A:TAlign);            {*Modify component alignment*}
      Procedure SetDebug(D:Integer); Virtual;  {*Set component debug value*}
      Procedure UpdateAlignment; Virtual;      {*Updates coordinates with aligment*}
      Procedure SetParent(pParent:TControl); Virtual;{*Set the visual parent component*}
      Procedure GetParentsInfo(Var Count:Integer; Var Broken:Boolean; Var Point:TPoint);{*Get information on all parents*}
      Procedure GetChildInfo(Var Count:Integer); {*Get child control information*}
      Procedure GetTabOrderList(List:TList);   {*Get controls list sorted by TabOrder*}
      Function  GetTabOrder(Control:TControl):Integer; {*Get the control TabOrder*}
      Function  GetFocusedControl:TControl;    {*Get focused component in list*}
      Procedure RemoveFocus; Virtual;          {*Remove focus from component*}
      Procedure RemoveAllFocus; Virtual;       {*Remove all focus in current Window*}
      Function  GetControlExtents:TCoordinates; Virtual; {*Get child controls extents*}
      Procedure StoreBg;                       {*Store the component background*}
      Procedure RestoreBg;                     {*Restore the component background*}
      Procedure DrawControls;                  {*Draw all child controls*}
      Procedure Clip; Virtual;                 {*Clip in control coordinates*}
      Procedure UnClip; Virtual;               {*Restore full screen*}

      Function  FindNextControl(CurControl:TControl; GoForward,CheckTabStop,CheckParent:Boolean):TControl; {*Find next control in list*}
      Procedure ShowControl(AControl:TControl); Virtual; {*Show control*}
      Procedure ReloadDockedControl(Const AControlName:String; Var AControl:TControl); Dynamic; {**}
(*    Procedure AlignControls(AControl:TControl; Var Rect:TRect); Virtual; {Pour aligner les contrles}
      Function  IsControlMouseMsg(Var Message:TWMMouse): Boolean; {*Get mouse pos on control*}
      Procedure ChangeScale(M,D:Integer);      {*Change scale for one control*}
      Procedure ScaleControls(M,D:Integer)     {*Scale all controls*};*)

   Public
      Property  Displayed   :Boolean      Read fDisplayed;
      Property  Focused     :Boolean      Read fFocused;
      Property  Refresh     :TRefresh     Read fRefresh;
      Property  Message     :TMessage     Read fMessage;
      Property  SaveBg      :Boolean      Read fSaveBg       Write SetSaveBg;
      Property  Parent      :TControl     Read fParent       Write SetParent;
      Property  Constraints :TConstraints Read fConstraints  Write fConstraints;
      Property  Anchors     :TAnchors     Read fAnchors      Write fAnchors;
      Property  Debug       :Integer      Read fDebug        Write SetDebug;
      Property  ControlCount:Integer      Read GetControlCount;
      Property  ChildControlCount:Longint Read GetChildControlCount;
      Property  Controls[Index:Integer]:TControl Read GetControl;

      Constructor Create(AOwner:TComponent); Override; {*Create new component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer);{*Create new component with parameters*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   MoveTo(Xa,Ya:Integer); Virtual; {*Move component to another position*}
      Procedure   Draw; Virtual;                  {*Draw component*}
      Procedure   Hide; Virtual;                  {*Hide component*}
      Procedure   Invalidate; Virtual;            {*Redraw component if needs*}
      Function    GetMsg:TCompMessage; Virtual;   {*Get messages from component*}
      Function    CanDraw:Boolean; Virtual;       {*Test if component can be drawed*}
      Function    CanTest:Boolean; Virtual;       {*Test if component can be tested*}
      Function    CanFocus:Boolean; Virtual;      {*Test if component can be focused*}
      Procedure   SetFocus; Virtual;              {*Set focus on component*}
      Function    TestParents:Boolean;            {*Test if all parents are defined*}
      Function    TestParent:Boolean;             {*Test if the parent is defined*}
      Function    TestParentsVisible:Boolean;     {*Test if all parents are visible*}
      Function    TestParentsEnabled:Boolean;     {*Test if all parents are enabled*}
      Function    ParentsCount:Integer;           {*Count how many parents have component*}
      Function    GetChildLevel:Integer; Virtual; {*Count child levels deep*}
      Function    GetClientRect:TCoordinates; Virtual; {*Get client coordinates*}
      Function    GetScreenRect:TCoordinates; Virtual; {*Get real screen coordinates*}
      Function    GetParentRect:TCoordinates; Virtual; {*Get parent coordinates*}
      Function    GetScreenPoint:TPoint; Virtual; {*Get parent client point*}
      Procedure   InsertControl(AControl:TControl); {*Insert child control in list*}
      Procedure   RemoveControl(AControl:TControl); {*Remove child control from list*}
      Function    ContainsControl(Control:TControl):Boolean; {*Check if a child control is contained in current component*}
      Function    TabOrderCount(pCanFocus,pTabStop:Boolean):Integer; {*Count TabOrder controls*}
      Function    GetNextTab:TControl;            {*Get the next available Tab control*}
      Function    GetPreviousTab:TControl;        {*Get the previous available Tab control*}

(*    Procedure   UpdateControlState;             {*Update all child controls state*}
      Function    ControlAtPos(Const Pos:TPoint; AllowDisabled:Boolean):TControl; {*Return child control at Pos if exist*}
      Function    FindChildControl(Const ControlName:String):TControl; {*Find a child control by his name*}*)

   Published
      Property  X1          :Integer  Read fX1         Write SetX1;
      Property  Y1          :Integer  Read fY1         Write SetY1;
      Property  X2          :Integer  Read fX2         Write SetX2;
      Property  Y2          :Integer  Read fY2         Write SetY2;
      Property  Left        :Integer  Read fX1         Write SetLeft;
      Property  Top         :Integer  Read fY1         Write SetTop;
      Property  Right       :Integer  Read fX2         Write SetX2;
      Property  Bottom      :Integer  Read fY2         Write SetY2;
      Property  Width       :Integer  Read GetWidth    Write SetWidth;
      Property  Height      :Integer  Read GetHeight   Write SetHeight;
      Property  Visible     :Boolean  Read fVisible    Write SetVisible;
      Property  Enabled     :Boolean  Read fEnabled    Write SetEnabled Default Yes;
      Property  Scale       :Integer  Read fScale      Write SetScale;
      Property  Align       :TAlign   Read fAlign      Write SetAlign;
      Property  TabOrder    :Integer  Read fTabOrder   Write SetTabOrder;
      Property  TabStop     :Boolean  Read fTabStop    Write SetTabStop;

      Property  OnChange    :TNotifyEvent     Read fOnChange     Write fOnChange;
      Property  OnClick     :TNotifyEvent     Read fOnClick      Write fOnClick;
      Property  OnDblClick  :TNotifyEvent     Read fOnDblClick   Write fOnDblClick;
      Property  OnMouseMove :TMouseMoveEvent  Read fOnMouseMove  Write fOnMouseMove;
      Property  OnMouseUp   :TMouseEvent      Read fOnMouseUp    Write fOnMouseUp;
      Property  OnMouseDown :TMouseEvent      Read fOnMouseDown  Write fOnMouseDown;
   End;

{****************************************************************************}
{*** AMouseZone Class                                                     ***}
{****************************************************************************}

AMouseZone = Class(TControl)            {*Mouse Zone component*}
   Private
      fCursor           :String;        {*Mouse pointer file*}
      fButton           :Byte;          {*Mouse button action*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetCursor(C:String);    {*Set a new mouse pointer (cursor)*}

   Public
      Property Cursor   :String      Read fCursor   Write SetCursor;
      Property Button   :Byte        Read fButton   Write fButton;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pButton:Byte; pCursor:String); {*Define a Mouse Zone*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from Mouse Zone*}
   End;

{****************************************************************************}
{*** AHint Class                                                          ***}
{****************************************************************************}

AHint = Class(TControl)                 {*THint component*}
   Private
      fHintBox          :TControl;      {*Hint box coordinates*}
      fDisplayMethod    :TDisplayHint;  {*Display method*}
      fCaption          :String;        {*String to display*}
      fDelay            :Integer;       {*Delay before displaying in 1/100*}

      fQttLines         :Byte;          {*Number of lines in text*}
      fMaxCharInLine    :Integer;       {*Quantity of chars in the bigest line*}
      fTimer            :TClock;        {*Pointer to TClock structure*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Property    QttLines     :Byte    Read fQttLines;
      Property    MaxCharInLine:Integer Read fMaxCharInLine Write fMaxCharInLine;

      Procedure   SetCaption(C:String); {*Define new Caption*}
      Procedure   SetDelay(D:Integer);  {*Set new time before displaying*}

   Public
      Property HintBox      :TControl     Read fHintBox;
      Property Timer        :TClock       Read fTimer;
      Property Delay        :Integer      Read fDelay         Write SetDelay;
      Property Caption      :String       Read fCaption       Write SetCaption;
      Property DisplayMethod:TDisplayHint Read fDisplayMethod Write fDisplayMethod;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pDisplayMethod:TDisplayHint; pCaption:String); {*Define a Hint*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*New draw method*}
      Procedure   Hide; Override;                 {*New hide method*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from Hint*}
   End;

{****************************************************************************}
{*** TVisual Class                                                        ***}
{****************************************************************************}

TVisual = Class(TControl)               {*MOS evoluted visual component*}
   Private
      fMouse            :AMouseZone;    {*Mouse pointer manager*}
      fHint             :AHint;         {*Hint manager*}
      fSupportColor     :Longint;       {*Support background color*}

   Protected
      Property  SupportColor:Longint       Read fSupportColor;

      Procedure SetLeft(L:Integer); Override;   {*Set the component left*}
      Procedure SetTop(T:Integer); Override;    {*Set the component top*}
      Procedure SetWidth(W:Integer); Override;  {*Set the component width*}
      Procedure SetHeight(H:Integer); Override; {*Set the component height*}
      Procedure SetX1(X:Integer); Override;     {*Set the component X1*}
      Procedure SetY1(Y:Integer); Override;     {*Set the component Y1}
      Procedure SetX2(X:Integer); Override;     {*Set the component X2*}
      Procedure SetY2(Y:Integer); Override;     {*Set the component Y2*}

   Public
      Property Mouse    :AMouseZone  Read fMouse;
      Property Hint     :AHint       Read fHint;

      Constructor Create(AOwner:TComponent); Override; {*Define component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer);{*Create the component*}
      Destructor  Destroy; Override;              {*Destroy the component*}
      Procedure   MoveTo(Xa,Ya:Integer); Override;{*Move component to another position*}
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}

AButton = Class
   Private
      MyButton         :TButton;
   Protected
   Public
      Constructor Define(X,Y,Sx,Sy:Integer; CoP,CoB:Longint; Id:Word; St,MPointer,StHint:String); {*Define Button*}
      Constructor Create(X,Y,Sx,Sy:Integer; CoP,CoB:Longint; Id:Word; St:String);                 {*Define and display Button*}
      Constructor Create(X,Y,Sx,Sy:Integer; CoP,CoB:Longint; Id:Word; St,MPointer,StHint:String); {*Define and display Button with Hint*}
      Destructor  Free;                                   {*Free Button resources*}
      Procedure   Draw;                                   {*Draw Button*}
      Procedure   Freeze(Status:Boolean);                 {*Freeze or unfreeze Button*}
      Function    GetMsg:Byte;                            {*Get Messages from Button*}
   End;

ACycler = Class
   Private
      MyCycler         :TCycler;
   Protected
   Public
      Constructor Define(X,Y,Size,Num:Integer; CoP,CoB:LongInt; Id:Word; St,MPointer,SHint:String); {*Define Cycler*}
      Constructor Create(X,Y,Size,Num:Integer; CoP,CoB:LongInt; Id:Word; St:String);                {*Define and display Cycler*}
      Constructor Create(X,Y,Size,Num:Integer; CoP,CoB:LongInt; Id:Word; St,MPointer,SHint:String); {*Define and display Cycler with Hint*}
      Destructor  Free;                                   {*Free Cycler resources*}
      Procedure   Draw;                                   {*Draw Cycler*}
      Procedure   Freeze(Status:Boolean);                 {*Freeze or unfreeze Cycler*}
      Function    GetMsg:Byte;                            {*Get messages from Cycler*}
   End;

ARadioBox = Class
   Private
      MyRadioBox       :TRadioBox;
   Protected
   Public
      Constructor Define(X,Y,Style:Integer; Id,Group,PLab:Byte; Status:Boolean; Cin,MPointer,SHint:String); {*Define RadioBox*}
      Constructor Create(X,Y,Style:Integer; Id,Group,PLab:Byte; Status:Boolean; Cin:String);                {*Define and display RadioBox*}
      Constructor Create(X,Y,Style:Integer; Id,Group,PLab:Byte; Status:Boolean; Cin,MPointer,SHint:String); {*Define and display extended RadioBox*}
      Destructor  Free;                                   {*Free RadioBox resources*}
      Procedure   Draw;                                   {*Draw RadioBox*}
      Procedure   Freeze(Status:Boolean);                 {*Freeze or unfreeze RadioBox*}
      Function    GetMsg:Byte;                            {*Get messages from RadioBox*}
   End;

ASlider = Class
   Private
      MySlider         :TSlider;
   Protected
   Public
      Constructor Define(X1,Y1,X2,Y2:Integer;Quantity,Displayed,Value,Orientation:Longint; Support,Shape:Byte; Graduations:Word; Cursor,ButtonsPos:Byte; ButtonsDef:Boolean; Id:Word; ShapeColor,ShapeShadeCol1,ShapeShadeCol2,CursorColor:Longint;
                         ShapeImage,CursorImage,MPointer,SHint:String); {*Define Slider*}
      Constructor Create(X1,Y1,Size:Integer; Quantity,Value,Orientation:Longint; Graduations:Word; Cursor:Byte; MPointer,SHint:String);                {*Define and display graduations Slider*}
      Constructor Create(X1,Y1,Size:Integer; Quantity,Value,Orientation:Longint; Cursor:Byte; MPointer,SHint:String); {*Define and display value Slider*}
      Constructor Create(X1,Y1,Size:Integer; Quantity,Displayed,Value,Orientation:Longint; ButtonsPos:Byte; ButtonsDef:Boolean; ShapeColor:Longint; MPointer,SHint:String); {*Define and display classical Slider*}
      Constructor Create(X1,Y1,Size:Integer; Quantity,Displayed,Value,Orientation:Longint; Support,Shape:Byte; Graduations:Word; Cursor,ButtonsPos:Byte; ButtonsDef:Boolean; Id:Word; ShapeColor,ShapeShadeCol1,ShapeShadeCol2,CursorColor:Longint;
                         ShapeImage,CursorImage,MPointer,SHint:String); {*Define and display extended slider*}
      Destructor  Free;                                   {*Free RadioBox resources*}
      Procedure   Draw;                                   {*Draw Slider*}
      Procedure   Refresh;                                {*Update position and draw Slider again*}
      Procedure   DrawShape;                              {*Draw Slider shape*}
      Procedure   DrawSupport;                            {*Draw Slider support*}
      Procedure   DrawCursor;                             {*Draw Slider cursor*}
      Procedure   DrawGraduations;                        {*Draw Slider graduations*}
      Function    GetMsg:Byte;                            {*Get messages from RadioBox*}
   End;

AInputText = Class
   Private
      MyInputText      :TInputText;
   Protected
   Public
      Constructor Define(X,Y,Siz:Integer; Len:Byte; CoB,Co2:Longint;Sty:Byte; ID:Word; Var Cin:String); {*Define InputText*}
      Constructor Create(X,Y,Siz,Len:Integer; CoB,Co2:Longint; Sty:Byte; Id:Word; Var Cin:String);      {*Define and display InputText*}
      Destructor  Free;                                   {*Free ImputText resources*}
      Procedure   Draw;                                   {*Draw ImputText*}
      Procedure   Freeze(Status:Boolean);                 {*Freeze or unfreeze ImputText*}
      Function    GetMsg(Var Cin:String):Byte;            {*Get messages from ImputText*}
   End;

AInputSlider = Class
   Private
      MyInputSlider    :TInputSlider;
   Protected
   Public
      Constructor Create(X1,Y1,Sx:Integer; Col1,Col2,Style:Longint; Min,Max,Step:Longint; Var Value:Longint; Id:Word); {*Define and display InputSlider*}
      Destructor  Free;                                   {*Free ImputSlider resources*}
      Procedure   Draw;                                   {*Draw ImputSlider*}
      Procedure   Freeze(Status:Boolean);                 {*Freeze or unfreeze ImputSlider*}
      Function    GetMsg(Var VCin:Longint):Byte;          {*Get messages from ImputSlider*}
   End;

AColorPalette = Class
   Private
      MyColorPalette   :TColorPalette;
   Protected
      Procedure   ToggleCell(Index:Integer; Display:Boolean; Border:Byte); {*Toggle a ColorPalette cell*}
   Public
      Constructor Define(Px,Py,CellX,CellY,InterCell,Columns:Integer; FirstColor,LastColor:Longint; Count:Integer; PaletteFile:String; Selected,BorderType:Integer; Id:Word; MPointer:ShortString); {*Define ColorPalette*}
      Constructor Create(Px,Py,CellX,CellY,InterCell,Columns:Integer; FirstColor,LastColor:Longint; Count,Selected,BorderType:Integer; Id:Word; MPointer:ShortString); {*Define and display ColorPalette*}
      Constructor Create(Px,Py,CellX,CellY,InterCell,Columns:Integer; PaletteFile:String; Selected,BorderType:Integer; Id:Word; MPointer:ShortString); {*Define and display ColorPalette*}
      Destructor  Free;                                   {*Free ColorPalette resources*}
      Procedure   Draw;                                   {*Draw ColorPalette*}
      Procedure   Freeze(Status:Boolean);                 {*Freeze or unfreeze ColorPalette*}
      Function    GetMsg:Byte;                            {*Get messages from ColorPalette*}
      Function    GetSelected:Longint;                    {*Get RGB values of the selected color*}
   End;

{}
{}
{}

IMPLEMENTATION

Uses
     MosBase,
     MosColors,
     MosGraph,
     MosMouse,
     MosStrings,
     MosText,
     MosTools;

{****************************************************************************}
{*** TMessage Class                                                       ***}
{****************************************************************************}

Constructor TMessage.Create;
Begin
fCount := 0;
End;

Function TMessage.PutToList(Msg:TCompMessage):Boolean;
Begin
If fCount < MaxMessages Then
   Begin
   Inc(fCount);                         {*1 new message*}
   fMessageList[Count] := Msg;          {*Put message on the top of stack*}
   Result := Yes;
   End
Else
   Begin
   Result := No;                        {*List full !*}
   End;
End;

Procedure TMessage.Clear;
Begin
fCount := 0;
End;

Procedure TMessage.MoveFrom(NewList:TMessage);
Var Loop:Integer;
Begin
If Not Assigned(NewList) Then Exit;     {*List not created !*}

For Loop:=1 to NewList.Count do         {*Move each message to inner list*}
   Begin
   PutToList(NewList.GetFromList);
   End;
NewList.Clear;                          {*Clear all messages from NewList*}
End;

Function TMessage.GetFromList:TCompMessage;
Var Loop:Integer;

Begin
Result := CMNone;                       {*In case of non messages*}
If fCount < 1 Then Exit;                {*No messages in list*}

Result := fMessageList[1];              {*Return message*}

For Loop := 1 to fCount-1 do            {*Shift messages in list*}
   Begin
   fMessageList[Loop] := fMessageList[Loop + 1];
   End;
Dec(fCount);                            {*Decrease components count*}
End;

{****************************************************************************}
{*** TControl Class                                                       ***}
{****************************************************************************}

Procedure TControl.Define;
Begin
fParent       := NIL;
fEnabled      := Yes;
fVisible      := Yes;
fDisplayed    := No;
fScale        := 100;
fDrawOnModify := Yes;
fSaveBg       := No;
fFocused      := No;
fFocusEnabled := No;
fAlign        := ALNone;

If Owner = Self Then
   If Assigned(Log) Then                {*Object could not created...*}
      Log.Add('Create new TControl ('+')',2);
End;

Constructor TControl.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fRefresh    := TRefresh.Create;
fMessage    := TMessage.Create;
fControls   := TList.Create;
//fFont       := TFont.Create(AOwner);
//fFont.OnChange := FontChanged;

fX1 := 015;                             {*Default coordinates*}
fY1 := 015;
fX2 := 060;
fY2 := 040;

Define;                                 {*Define base variables*}
End;

Constructor TControl.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer);
Begin
Inherited Create(AOwner);
fRefresh    := TRefresh.Create;
fMessage    := TMessage.Create;
fControls   := TList.Create;

fX1 := pX1;                             {*User coordinates*}
fY1 := pY1;
fX2 := pX2;
fY2 := pY2;

Define;                                 {*Define base variables*}
End;

Destructor TControl.Destroy;
Begin
fControls.Destroy;
fMessage.Destroy;
fRefresh.Destroy;

Inherited Destroy;
End;

Procedure TControl.StoreBg;
Begin
If fSaveBg Then
   Begin
   fRefresh.GetImageFromScreen(T_Clip.X1+fX1, T_Clip.Y1+fY1, T_Clip.X1+fX2, T_Clip.Y1+fY2);
   End;
End;

Procedure TControl.RestoreBg;
Begin
If fSaveBg Then
   Begin
   fRefresh.PutImageToScreen(T_Clip.X1+fX1, T_Clip.Y1+fY1);
   End;
End;

Procedure TControl.SetSaveBg(S:Boolean);
Begin
{WARNING ! Check algo because when you modify fBgStore, if the Bg is stored
it could happend some problems!}
fSaveBg := S;
End;

Procedure TControl.MoveTo(Xa,Ya:Integer);
Var BgSaved:Boolean;
Begin
{Check if coordinates are valid...}
If fAlign <> ALNone Then Exit;          {*Alignment defined ?*}

BgSaved := fDisplayed;
If BgSaved Then Hide;
fX2 := Xa + Width;
fY2 := Ya + Height;
fX1 := Xa;
fY1 := Ya;
Message.PutToList(CMMoved);

If (BgSaved) And (fDrawOnModify) Then Draw; {*Only if background is saved and if we need to draw on move*}
End;

Procedure TControl.Invalidate;
Begin
If (Not fDisplayed) And (fDrawOnModify) Then
   Begin
   Draw;                                {*Redraw control*}
   End;
End;

Procedure TControl.Draw;
Begin
If Not fVisible Then Exit;              {*Component is not visible*}
If fDisplayed Then Exit;                {*Component is yet displayed*}

fDisplayed := Yes;                      {*Naturaly displayed now !*}
StoreBg;
If ControlCount > 0 Then DrawControls;  {*Draw all child controls*}
Message.PutToList(CMDisplayed);
End;

Procedure TControl.Hide;
Begin
If Not fVisible Then Exit;              {*Component is not visible*}
If Not fDisplayed Then Exit;            {*Component is not displayed*}

fDisplayed := No;
RestoreBg;
Message.PutToList(CMHidden);
End;

Procedure TControl.Clip;
{Var Screen           :TCoordinates;
    Client           :TCoordinates;}
Begin
{Client := GetClientRect;
Screen := GetScreenRect;}

{*Checker si les coord reelles sont dans le client*}

VESA_SetViewPort(X1,Y1,X2,Y2, ClipOn);
End;

Procedure TControl.UnClip;
Begin
VESA_SetViewPort(0,0,VESA_GetMaxX,VESA_GetMaxY, ClipOn);
End;

Procedure TControl.SetVisible(V:Boolean);
Begin
If fVisible = V Then Exit;              {*Don't change status*}
fVisible := V;
If Not fVisible Then Draw Else Hide;
End;

Procedure TControl.SetEnabled(E:Boolean);
Begin
If fEnabled = E Then Exit;              {*Don't change status*}
fEnabled := E;
If fEnabled Then
   Begin
   Draw;
   Message.PutToList(CMEnabled);
   End
Else
   Begin
   Col_SetZoneToGrey(fX1,fY1,fX2,fY2);
   Message.PutToList(CMDisabled);
   End;
End;

Procedure TControl.SetAlign(A:TAlign);
Begin
If A = fAlign Then Exit;
fAlign := A;

If Not TestParent Then fAlign := ALNone;{*Parent not found*}

UpdateAlignment;                        {*Update coordinates for alignment*}
Invalidate;                             {*Redraw control if needs*}
End;

Procedure TControl.UpdateAlignment;
Var Coord            :TCoordinates;

Begin
Coord := GetParentRect;                 {*Get parent coordinates*}

Case fAlign of
   ALBottom : Begin
              fY1 := Coord.Y2 - Height;
              fY2 := Coord.Y2;
              End;
   ALClient : Begin
              fX1 := Coord.X1; fY1 := Coord.Y1;
              fX2 := Coord.X2; fY2 := Coord.Y2;
              End;
   ALLeft   : Begin
              fX1 := Coord.X1;
              fX2 := Coord.X1 + Width;
              End;
   ALNone   : Begin                     {*No coordinates modification*}
              End;
   ALRight  : Begin
              fX1 := Coord.X1 - Width;
              fX2 := Coord.X2;
              End;
   ALTop    : Begin
              fY1 := Coord.Y1;
              fY2 := Coord.Y1 + Height;
              End;
   End;
End;

Function TControl.GetWidth:Integer;
Begin
Result := fX2 - fX1;
End;

Procedure TControl.SetWidth(W:Integer);
Begin
If W = Width Then Exit;
If W < 0 Then Exit;
SetX2(fX1 + W);                         {*Redraw component*}
End;

Function TControl.GetHeight:Integer;
Begin
Result := fY2 - fY1;
End;

Procedure TControl.SetHeight(H:Integer);
Begin
If H = Height Then Exit;
If H < 0 Then Exit;
SetY2(fY1 + H);                         {*Redraw component*}
End;

Procedure TControl.SetX1(X:Integer);
Var BgSaved:Boolean;
Begin
If X = fX1 Then Exit;
{If (X > fX2) And (fX1 = fX2) Then Exit;}

BgSaved := fDisplayed;
If BgSaved Then Hide;

fX1 := X;
{If fX1 > fX2 Then fX1 := fX2;}
UpdateAlignment;                        {*Update coordinates for alignment*}
If BgSaved Then Draw;
Message.PutToList(CMReSized);
End;

Procedure TControl.SetY1(Y:Integer);
Var BgSaved:Boolean;
Begin
If Y = fY1 Then Exit;
{If (Y > fY2) And (fY1 = fY2) Then Exit;}

BgSaved:=fDisplayed;
If BgSaved Then Hide;

fY1 := Y;
{If fY1 > fY2 Then fY1 := fY2;}
UpdateAlignment;                        {*Update coordinates for alignment*}
If BgSaved Then Draw;
Message.PutToList(CMReSized);
End;

Procedure TControl.SetX2(X:Integer);
Var BgSaved:Boolean;
Begin
If X = fX2 Then Exit;
{If (X < fX1) And (fX1 = fX2) Then Exit;}

BgSaved := fDisplayed;
If BgSaved Then Hide;

fX2 := X;
{If fX2 < fX1 Then fX2 := fX1;}
UpdateAlignment;                        {*Update coordinates for alignment*}
If BgSaved Then Draw;
Message.PutToList(CMReSized);
End;

Procedure TControl.SetY2(Y:Integer);
Var BgSaved:Boolean;
Begin
If Y = fY2 Then Exit;
{If (Y < fY1) And (fY1 = fY2) Then Exit;}

BgSaved := fDisplayed;
If BgSaved Then Hide;

fY2 := Y;
{If fY2 < fY1 Then fY2 := fY1;}
UpdateAlignment;                        {*Update coordinates for alignment*}
If BgSaved Then Draw;
Message.PutToList(CMReSized);
End;

Procedure TControl.SetLeft(L:Integer);
Var BgSaved:Boolean;
    WidthSaved:Integer;
Begin
If L = fX1 Then Exit;
BgSaved := fDisplayed;
If BgSaved Then Hide;

WidthSaved := Width;
fX1 := L;
fX2 := L + WidthSaved;

UpdateAlignment;                        {*Update coordinates for alignment*}
If BgSaved Then Draw;
Message.PutToList(CMReSized);
End;

Procedure TControl.SetTop(T:Integer);
Var BgSaved:Boolean;
    HeightSaved:Integer;
Begin
If T = fY1 Then Exit;
BgSaved:=fDisplayed;
If BgSaved Then Hide;

HeightSaved := Height;
fY1 := T;
fY2 := T + HeightSaved;

UpdateAlignment;                        {*Update coordinates for alignment*}
If BgSaved Then Draw;
Message.PutToList(CMReSized);
End;

Procedure TControl.SetScale(S:Integer);
Begin
If fScale = S Then Exit;
fScale := S;

X2 := Round(Real(fX2 * fScale) / 100);
Y2 := Round(Real(fY2 * fScale) / 100);

Draw;
Message.PutToList(CMScaled);
End;

Procedure TControl.SetDebug(D:Integer);
Begin
If D = fDebug Then Exit;
fDebug := D;
End;

Procedure TControl.SetTabOrder(O:Integer);
Begin
If O = fTabOrder Then Exit;
fTabOrder := O;
{Il faut parcourir tous les controles enfants du container pour redfinir les numros}
End;

Procedure TControl.SetTabStop(T:Boolean);
Begin
If T = fTabStop Then Exit;
fTabStop := T;
If (fFocused) And (Not fTabStop) Then   {*Component have focus*}
   Begin
{   SetFocus;} {faire un SetFocus sur le composant suivant dans la liste}
   Invalidate;                          {*Redraw component*}
   End;
End;

Function TControl.GetFocusedControl:TControl;
Var Loop             :Integer;
Begin
For Loop := 0 to ControlCount - 1 do    {*Control list*}
   Begin
   If Controls[Loop].Focused Then       {*Focused component found*}
      Begin
      Result := Controls[Loop];         {*Return control*}
      Exit;
      End;
   End;
Result := NIL;                          {*Focus not found !*}
End;

Procedure TControl.RemoveAllFocus;
Begin
{*Supprime le focus dans toute la fentre courrante active*}
End;

Procedure TControl.RemoveFocus;
Begin
If Not fFocused Then Exit;              {*Component have not focus !*}

fFocused := No;
Invalidate;                             {*Redraw component*}
End;

Procedure TControl.SetFocus;
Var MyControl        :TControl;

Begin
If fFocused Then Exit;                  {*Already focused*}
If Not CanFocus Then Exit;              {*Cannot set focus for this component*}

If fParent <> NIL Then                  {*Need parent to found focused component*}
   Begin
   MyControl := fParent.GetFocusedControl; {*Only parent have controls list*}
   If MyControl <> NIL Then MyControl.RemoveFocus; {*Remove focus on focused component*}
   fFocused := Yes;
   Invalidate;                          {*Redraw component*}
   End;
End;

Function TControl.CanFocus:Boolean;
Begin
If (fVisible) And (fEnabled) And (fFocusEnabled)
   Then Result := Yes
   Else Result := No;
End;

Function TControl.CanDraw:Boolean;
Begin
If (fVisible) And (TestParents)
   Then Result := Yes
   Else Result := No;
End;

Function TControl.CanTest:Boolean;
Begin
If (fEnabled) And (TestParents)
   Then Result := Yes
   Else Result := No;
End;

Procedure TControl.GetParentsInfo(Var Count:Integer; Var Broken:Boolean; Var Point:TPoint);
Var TempCoord        :TCoordinates;     {*Temporary coordinates for calcul*}
Begin
If Not (Self Is TApplication) Then      {*Reached the high level ?*}
   Begin
   If fParent = NIL Then                {*Link to previous parent is broken*}
      Begin
      Broken := Yes;
      End
   Else
      Begin                             {*Parent defined. Follow the link*}
      If fParent Is TControl Then
         Begin                          {*Parent is a TControl component ?*}
         Inc(Count);                    {*One new parent*}
         TempCoord := GetClientRect;    {*Get client coordinates*}
         Point.X := Point.X + TempCoord.X1;
         Point.Y := Point.Y + TempCoord.Y1;
         fParent.GetParentsInfo(Count,Broken,Point); {*Call parent method*}
         End;
      End;
   End;
End;

Function TControl.TestParentsVisible:Boolean;
Begin
Result := Yes; {*Temporaire... il faut remonter toute la parente et tester*}
End;

Function TControl.TestParentsEnabled:Boolean;
Begin
Result := Yes; {*Temporaire... il faut remonter toute la parente et tester*}
End;

Function TControl.TestParent:Boolean;
Begin
Result := fParent <> NIL;
End;

Function TControl.TestParents:Boolean;
Var Count            :Integer;
    Broken           :Boolean;
    Point            :TPoint;
Begin
Count   := 0;                           {*Not used to test parents*}
Broken  := No;
Point.X := 0;                           {*Initialise blank point*}
Point.Y := 0;
GetParentsInfo(Count,Broken,Point);
Result  := Not Broken;
End;

Function TControl.ParentsCount:Integer;
Var Count            :Integer;
    Broken           :Boolean;
    Point            :TPoint;
Begin
Count   := 0;
Broken  := No;                          {*Not used to count parents*}
Point.X := 0;                           {*Initialise blank point*}
Point.Y := 0;
GetParentsInfo(Count,Broken,Point);     {*Recursive search method in parents*}
Result  := Count;
End;

Function TControl.GetScreenRect:TCoordinates;
Var Count            :Integer;
    Broken           :Boolean;
    Point            :TPoint;
    Client           :TCoordinates;
Begin
Count   := 0;                           {*Not used to get screen coordinates*}
Broken  := No;
Point.X := 0;                           {*Initialise blank point*}
Point.Y := 0;
GetParentsInfo(Count,Broken,Point);

Client  := GetClientRect;
Result.X1 := Point.X;
Result.Y1 := Point.Y;
Result.X2 := Point.X + (Client.X2 - Client.X1);
Result.Y2 := Point.Y + (Client.Y2 - Client.Y1);
End;

Function TControl.GetScreenPoint:TPoint;
Var Coord            :TCoordinates;
Begin
Coord := GetScreenRect;
Result.X := Coord.X1;
Result.Y := Coord.Y1;
End;

Function TControl.GetParentRect:TCoordinates;
Begin
Result.X1 := 0;
Result.Y1 := 0;
Result.X2 := 0;
Result.Y2 := 0;
If fParent = NIL Then Exit;
Result := fParent.GetClientRect;
End;

Function TControl.GetClientRect:TCoordinates;
Begin
Result.X1 := fX1;
Result.Y1 := fY1;
Result.X2 := fX2;
Result.Y2 := fY2;
End;

Procedure TControl.SetParent(pParent:TControl);
Begin
If fParent <> pParent Then
   Begin
   If pParent = Self Then Raise EInvalidOperation.CreateFmt('Cannot assign parent to self',[]);
   If fParent <> NIL Then fParent.RemoveControl(Self); {*Remove from current parent list*}
   If pParent <> NIL Then pParent.InsertControl(Self); {*Insert in new parent list*}
   fParent := pParent;                  {*Assign new parent to variable*}
   End;
End;

Function TControl.GetChildLevel:Integer;
Begin
{*Count child level - profondeur des enfants (le plus profond)*}
   Result := 0;
End;

Procedure TControl.DrawControls;
Var Loop              :Longint;
    MyControl         :TControl;
Begin
For Loop := 0 to ControlCount - 1 do    {*Child controls list*}
   Begin
   MyControl := Controls[Loop];         {*Get control*}
   If MyControl <> NIL Then MyControl.Draw; {*Draw child control*}
   End;
End;

Procedure TControl.ReloadDockedControl(Const AControlName:String; Var AControl:TControl);
Begin
AControl := Owner.FindComponent(AControlName) As TControl;
End;

Function TControl.GetTabOrder(Control:TControl):Integer;
Begin
{*Renvois le numro de TabControl du composant spcifi*}
// To avoid warnings !
   Control := Control;
   Result := 0;
End;

Function TControl.TabOrderCount(pCanFocus,pTabStop:Boolean):Integer;
Begin
{*Compte le nombre de composants qui ont un TabOrder dfinit en fonction des paramtres*}
// To avoid warnings !
   pTabStop := pTabStop;
   pCanFocus := pCanFocus;
   Result := 0;
End;

Function TControl.GetNextTab:TControl;
Begin
// To avoid warnings !
   Result := NIL;
End;

Function TControl.GetPreviousTab:TControl;
Begin
// To avoid warnings !
   Result := NIL;
End;

Procedure TControl.GetTabOrderList(List:TList);
{Var Loop             :Integer;
    Control          :TControl;}
Begin
// To avoid warnings !
   List := List;

(*If fTabList <> NIL Then
   For Loop := 0 to fTabList.Count - 1 do
      Begin
      Control := fTabList[Loop];
      List.Add(Control);
      Control.GetTabOrderList(List);
      End;*)
{A recoder version MOS pour obtenir une liste trie par TabOrder de tous les contrles}
End;

Function TControl.FindNextControl(CurControl:TControl; GoForward,CheckTabStop,CheckParent:Boolean):TControl;
{A coder en tenant compte de TabStop et TabOrder et GetTabOrderList}
{Var I, StartIndex    :Integer;
    List             :TList;}
Begin
// To avoid warnings !
CurControl   := CurControl;
GoForward    := GoForward;
CheckTabStop := CheckTabStop;
CheckParent  := CheckParent;

(*  Result := nil;
  List := TList.Create;
  try
    GetTabOrderList(List);
    if List.Count > 0 then
    begin
      StartIndex := List.IndexOf(CurControl);
      if StartIndex = -1 then
        if GoForward then StartIndex := List.Count - 1 else StartIndex := 0;
      I := StartIndex;
      repeat
        if GoForward then
        begin
          Inc(I);
          if I = List.Count then I := 0;
        end else
        begin
          if I = 0 then I := List.Count;
          Dec(I);
        end;
        CurControl := List[I];
        if CurControl.CanFocus and
          (not CheckTabStop or CurControl.TabStop) and
          (not CheckParent or (CurControl.Parent = Self)) then
          Result := CurControl;
      until (Result <> nil) or (I = StartIndex);
    end;
  finally
    List.Free;
  end;*)
Result := NIL;
End;

Procedure TControl.ShowControl(AControl:TControl);
Begin
// To avoid warnings !
   AControl := AControl;

   If fParent <> NIL Then fParent.ShowControl(Self);
End;

Function TControl.GetControlExtents:TCoordinates;
Var I                :Integer;
Begin
Result := Coord(MaxInt, MaxInt, 0, 0);
For I := 0 to ControlCount - 1 do
   With Controls[I] do
   If Visible Then
      Begin
      If X1 < Result.X1 Then Result.X1 := X1;
      If Y1 < Result.Y1 Then Result.Y1 := Y1;
      If X2 > Result.X2 Then Result.X2 := X2;
      If Y2 > Result.Y2 Then Result.Y2 := Y2;
      End;
End;

Function TControl.ContainsControl(Control:TControl):Boolean;
Begin
While (Control <> NIL) And (Control <> Self) do
   Begin
   Control := Control.Parent;
   End;
Result := Control <> NIL;
End;

Procedure TControl.RemoveControl(AControl:TControl);
Begin
If AControl Is TControl Then
   Begin
   fControls.Delete(fControls.IndexOf(AControl));
   End;
AControl.fParent := NIL;
End;

Procedure TControl.InsertControl(AControl:TControl);
Begin
If AControl <> NIL Then
   Begin
   If AControl Is TControl Then
      Begin
      fControls.Add(AControl);
      End;
   AControl.fParent := Self;
   End;
End;

Function TControl.GetControl(Index:Integer):TControl;
Begin
Result := NIL;
If fControls <> NIL Then
   Begin
   If Index < GetControlCount - 1 Then Result := TControl(fControls[Index])
                                  Else Result := NIL;
   End;
End;

Function TControl.GetControlCount:Integer;
Begin
If fControls <> NIL Then Result := fControls.Count
                    Else Result := 0;
End;

Procedure Tcontrol.GetChildInfo(Var Count:Integer);
Var Loop             :Integer;
Begin
If GetControlCount > 0 Then             {*Child controls defined ?*}
   Begin
   For Loop := 0 to GetControlCount - 1  do {*Get all direct child controls*}
      Begin
      Inc(Count);                       {*One more child found*}
      If Controls[Loop] <> NIL Then     {*Control pointer defined*}
         Begin
         Controls[Loop].GetChildInfo(Count); {*Call child recursive method*}
         End;
      End;
   End;
End;

Function TControl.GetChildControlCount:Integer;
Var Count            :Integer;
Begin
Count := 0;
GetChildInfo(Count);                    {*Call recursive search method*}
Result := Count;                        {*Send result*}
End;

Procedure TControl.FontChanged(Sender:TObject);
Begin
// To avoid warnings !
   Sender := Sender;
End;

Function TControl.GetMsg:TCompMessage;
Begin
Result := CMNone;
If Not Enabled Then Exit;               {*Component disabled*}

Result := Message.GetFromList;
End;

Procedure TControl.Change;
Begin
If Assigned(fOnChange) Then fOnChange(Self);
End;

Procedure TControl.Click;
Begin
If Assigned(fOnClick) Then fOnClick(Self);
End;

Procedure TControl.DblClick;
Begin
If Assigned(fOnDblClick) Then fOnDblClick(Self);
End;

Procedure TControl.MouseMove(Shift:TShiftState; X,Y:Integer);
Begin
If Assigned(fOnMouseMove) Then fOnMouseMove(Self, Shift, X,Y);
End;

Procedure TControl.MouseUp(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
Begin
If Assigned(fOnMouseUp) Then fOnMouseUp(Self, Button, Shift, X,Y);
End;

Procedure TControl.MouseDown(Button:TMouseButton; Shift:TShiftState; X,Y:Integer);
Begin
If Assigned(fOnMouseDown) Then fOnMouseDown(Self, Button, Shift, X,Y);
End;

{****************************************************************************}
{*** AMouseZone Class                                                     ***}
{****************************************************************************}

Procedure AMouseZone.Define;
Begin
fButton     := MBLeft;
fCursor     := P_Current;

If Owner = Self Then Log.Add('Create new TMouseZone ('+fCursor+')',2);
End;

Constructor AMouseZone.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
Define;
End;

Constructor AMouseZone.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pButton:Byte; pCursor:String);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);

Define;
fButton := pButton;
fCursor := pCursor;
End;

Procedure AMouseZone.SetCursor(C:String);
Begin
If C = fCursor Then Exit;
fCursor := C;
End;

Function AMouseZone.GetMsg:TCompMessage;
Begin
Result := CMNone;
If Not Enabled Then Exit;               {*Component disabled*}

If Mse_TestBox(T_Clip.X1+X1, T_Clip.Y1+Y1, T_Clip.X1+X2, T_Clip.Y1+Y2, MBIgnore) Then
   Begin
//   Result:=CMMouseOver;
   If Mse_Clicked(Button) Then Message.PutToList(CMSelected);
   If Not Sys.OnMouseZone Then Mse_SetPointer(Cursor); {*Set new mouse pointer*}
   Sys.OnMouseZone:=Yes;
   End;

Result := Message.GetFromList;          {*Return messages*}
End;

{****************************************************************************}
{*** AHint Class                                                          ***}
{****************************************************************************}

Procedure AHint.Define;
Begin
fHintBox.SaveBg := Yes;
fTimer.Show     := No;                  {*Init Timer*}
fDisplayMethod  := DHMouse;
fDelay          := 060;                 {*1 sec to wait*}

If Owner = Self Then Log.Add('Create new Hint ('+')',2);
End;

Constructor AHint.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fHintBox := TControl.Create(AOwner);
Define;
End;

Constructor AHint.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pDisplayMethod:TDisplayHint; pCaption:String);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);

fHintBox := TControl.Create(AOwner, pX1,pY1,pX2,pY2);

Define;
fDisplayMethod := pDisplayMethod;
SetCaption(pCaption);                   {*Set new caption*}
End;

Destructor AHint.Destroy;
Begin
fHintBox.Hide;                          {*Hide Hint before destroy*}
fHintBox.Destroy;
Inherited Destroy;
End;

Procedure AHint.SetDelay(D:Integer);
Begin
If D = fDelay Then Exit;
If D < 0 Then Exit;
If D > 50000 Then Exit;
fDelay := D;
End;

Procedure AHint.SetCaption(C:String);
Var A,L:Integer;

Begin
If C = fCaption Then Exit;              {*Strings are equal*}

If C[Length(C)]='|' Then Dec(C[0]);     {*Delete the end "|" char*}
fQttLines:= Str_Count(C,'|');
fCaption := C;

fMaxCharInLine := 0;
For A:=1 to fQttLines do
   Begin
   L := Length(Str_Get(fCaption,A,'|'));
   If fMaxCharInLine < L Then fMaxCharInLine := L;
   End;

Invalidate;                             {*Redraw if needs*}
End;

Procedure AHint.Draw;
Var mX,mY:Word;                         {*Mouse coordinates*}
    Context:TScreenContext;             {*Screen context*}
    A:Integer;
    CursorSizeX      :Integer;
    CursorSizeY      :Integer;

Begin
//fText    := AText.Create(AOwner);
//fText.Destroy;

If Not Visible Then Exit;
If fCaption = '' Then Exit;
Inherited Draw;                         {*Draw master zone*}

CursorSizeX := 10;                      {*Mouse pointer size*}
CursorSizeY := 16;

//Bevel(X1,Y1,X2,Y2, ClLightGreen);

fHintBox.Width  := fMaxCharInLine*(LaF+1)+6;
fHintBox.Height := fQttLines*(HaF)+4;   {*Calcule height an width*}

If fDisplayMethod = DHTop Then          {*Display on top of mouse zone*}
   Begin
   fHintBox.Left := Left; fHintBox.Top := Top - fHintBox.Height - 1;
   End;
If fDisplayMethod = DHOver Then         {*Display over mouse zone*}
   Begin
   fHintBox.Left := Left; fHintBox.Top := Top;
   End;
If fDisplayMethod = DHCorner Then       {*Display on the right bottom corner*}
   Begin
   fHintBox.Left := Right; fHintBox.Top := Bottom;
   End;
If fDisplayMethod = DHMouse Then        {*Display near the mouse*}
   Begin
   Mse_GetPos(mX,mY);
   fHintBox.Left := mX + CursorSizeX - T_Clip.X1;
   fHintBox.Top  := mY + CursorSizeY - T_Clip.Y1;
   End;

fHintBox.Draw;                          {*Draw nothing but need to Draw !*}

Bevel(fHintBox.X1, fHintBox.Y1, fHintBox.X2, fHintBox.Y2, CLBgInfo,BLUniformLine,BLNoLine,Yes);

Save_Context(Context,1);
Font_Set(Sys.FontHint,Sys.FontHintSize);
For A:=1 to fQttLines do Display(fHintBox.X1+4+T_Clip.X1, fHintBox.Y1+T_Clip.Y1+HaF*(A-1)+1, CLText, Sys.FontHint,Sys.FontHintSize, Str_Get(fCaption,A,'|'));
Restore_Context(Context);
End;

Procedure AHint.Hide;
Begin
fTimer.Show := No;                      {*Reinit Timer*}
fHintBox.Hide;                          {*Hide the message zone*}
Inherited Hide;                         {*Hide master zone*}
End;

Function AHint.GetMsg:TCompMessage;
Begin
Result := CMNone;

If Not Enabled Then Exit;               {*Component disabled*}
If Not Sys.DisplayHints Then Exit;      {*Interface does not display Hints*}
If fCaption='' Then Exit;               {*No text to display*}

If Mse_TestBox(X1+T_Clip.X1, Y1+T_Clip.Y1, X2+T_Clip.X1, Y2+T_Clip.Y1, MBNone) = Yes Then
   Begin
   If Time_TestTimer(fDelay,fTimer) Then
      Begin
      If Not Displayed Then
         Begin
         Draw;
         Message.PutToList(CMDisplayed);
         End;
      End;
   End
Else
   Begin
   fTimer.Show := No;                   {*Init Timer*}
   If Displayed Then
      Begin
      Hide;
      Message.PutToList(CMHidden);
      End;
   End;
Result := Message.GetFromList;          {*Return messages*}
End;

{****************************************************************************}
{*** TVisual Class                                                        ***}
{****************************************************************************}

Constructor TVisual.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fMouse := AMouseZone.Create(AOwner);
fHint  := AHint.Create(AOwner);
End;

Constructor TVisual.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fMouse := AMouseZone.Create(AOwner, pX1,pY1,pX2,pY2, MBLeft, P_Current);
fHint  := AHint.Create(AOwner, pX1,pY1,pX2,pY2, DHMouse,'');

fSupportColor := VESA_GetPixel(pX1,pY1);{*Support background color*}
{X1 := pX1;
Y1 := pY1;
X2 := pX2;
Y2 := pY2;}
End;

Destructor TVisual.Destroy;
Begin
fHint.Destroy;
fMouse.Destroy;
Inherited Destroy;
End;

Procedure TVisual.MoveTo(Xa,Ya:Integer);
Begin
fHint.MoveTo(Xa,Ya);
fMouse.MoveTo(Xa,Ya);
Inherited MoveTo(Xa,Ya);
End;

Procedure TVisual.SetWidth(W:Integer);
Begin
Inherited SetWidth(W);
fMouse.Width := W;
fHint.Width  := W;
End;

Procedure TVisual.SetHeight(H:Integer);
Begin
Inherited SetHeight(H);
fMouse.Height := H;
fHint.Height  := H;
End;

Procedure TVisual.SetX1(X:Integer);
Begin
Inherited SetX1(X);
fMouse.X1 := X;
fHint.X1  := X;
End;

Procedure TVisual.SetY1(Y:Integer);
Begin
Inherited SetY1(Y);
fMouse.Y1 := Y;
fHint.Y1  := Y;
End;

Procedure TVisual.SetX2(X:Integer);
Begin
Inherited SetX2(X);
fMouse.X2 := X;
fHint.X2  := X;
End;

Procedure TVisual.SetY2(Y:Integer);
Begin
Inherited SetY2(Y);
fMouse.Y2 := Y;
fHint.Y2  := Y;
End;

Procedure TVisual.SetLeft(L:Integer);
Begin
Inherited SetLeft(L);
fMouse.Left := L;
fHint.Left  := L;
End;

Procedure TVisual.SetTop(T:Integer);
Begin
Inherited SetTop(T);
fMouse.Top := T;
fHint.Top  := T;
End;

Procedure TVisual.Draw;
Begin
Inherited Draw;
//Bevel(X1,Y1,X2,Y2, CLLightBlue);
End;

Function TVisual.GetMsg:TCompMessage;
Begin
Result := CMNone;
If Not Enabled Then Exit;               {*Not enabled !*}

fMouse.GetMsg;                          {*Get messages from AMouse*}
fHint.GetMsg;
Message.MoveFrom(fMouse.Message);       {*Add messages from Mouse list*}
Message.MoveFrom(fHint.Message);        {*Add messages from Hint list*}

Result := Message.GetFromList;          {*Return messages*}
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}
{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}
{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}
{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}
{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

Constructor AButton.Define(X,Y,Sx,Sy:Integer; CoP,CoB:Longint; Id:Word; St,MPointer,StHint:String);
Begin
Button_Define(X,Y,Sx,Sy,CoP,CoB, Id, MyButton, St,MPointer,StHint);
End;

Constructor AButton.Create(X,Y,Sx,Sy:Integer; CoP,CoB:Longint; Id:Word; St:String);
Begin
Button(X,Y,Sx,Sy,CoP,CoB, Id, MyButton, St);
End;

Constructor AButton.Create(X,Y,Sx,Sy:Integer; CoP,CoB:Longint; Id:Word; St,MPointer,StHint:String);
Begin
Button(X,Y,Sx,Sy,CoP,CoB, Id, MyButton, St,MPointer,StHint);
End;

Destructor AButton.Free;
Begin
End;

Procedure AButton.Draw;
Begin
Button_Draw(MyButton);
End;

Procedure AButton.Freeze(Status:Boolean);
Begin
Button_Freeze(Status,MyButton);
End;

Function AButton.GetMsg:Byte;
Begin
Result:=Button_GetMsg(MyButton,1);
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

Constructor ACycler.Define(X,Y,Size,Num:Integer; CoP,CoB:LongInt; Id:Word; St,MPointer,SHint:String);
Begin
Cycler_Define(X,Y,Size,Num,CoP,CoB,Id,MyCycler,St,MPointer,SHint);
End;

Constructor ACycler.Create(X,Y,Size,Num:Integer; CoP,CoB:LongInt; Id:Word; St:String);
Begin
Cycler(X,Y,Size,Num,CoP,CoB,Id,MyCycler,St);
End;

Constructor ACycler.Create(X,Y,Size,Num:Integer; CoP,CoB:LongInt; Id:Word; St,MPointer,SHint:String);
Begin
Cycler(X,Y,Size,Num,CoP,CoB,Id,MyCycler,St,MPointer,SHint);
End;

Destructor ACycler.Free;
Begin
End;

Procedure ACycler.Draw;
Begin
Cycler_Draw(MyCycler);
End;

Procedure ACycler.Freeze(Status:Boolean);
Begin
Cycler_Freeze(Status,MyCycler);
End;

Function ACycler.GetMsg:Byte;
Begin
Result:=Cycler_GetMsg(MyCycler,1);
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

Constructor ARadioBox.Define(X,Y,Style:Integer; Id,Group,PLab:Byte; Status:Boolean; Cin,MPointer,SHint:String);
Begin
RadioBox_Define(MyRadioBox,X,Y,Style,Id,Group,PLab,Status,Cin,MPointer,SHint);
End;

Constructor ARadioBox.Create(X,Y,Style:Integer; Id,Group,PLab:Byte; Status:Boolean; Cin:String);
Begin
RadioBox(X,Y,Style,Id,Group,PLab,Status, MyRadioBox,Cin);
End;

Constructor ARadioBox.Create(X,Y,Style:Integer; Id,Group,PLab:Byte; Status:Boolean; Cin,MPointer,SHint:String);
Begin
RadioBox(X,Y,Style,Id,Group,PLab,Status, MyRadioBox,Cin,MPointer,SHint);
End;

Destructor ARadioBox.Free;
Begin
End;

Procedure ARadioBox.Draw;
Begin
RadioBox_Draw(MyRadioBox);
End;

Procedure ARadioBox.Freeze(Status:Boolean);
Begin
RadioBox_Freeze(Status,MyRadioBox);
End;

Function ARadioBox.GetMsg:Byte;
Begin
Result:=RadioBox_GetMsg(MyRadioBox,1);
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

Constructor ASlider.Define(X1,Y1,X2,Y2:Integer;Quantity,Displayed,Value,Orientation:Longint; Support,Shape:Byte; Graduations:Word; Cursor,ButtonsPos:Byte; ButtonsDef:Boolean; Id:Word; ShapeColor,ShapeShadeCol1,ShapeShadeCol2,CursorColor:Longint;
                           ShapeImage,CursorImage,MPointer,SHint:String);
Begin
Slider_Define(MySlider, X1,Y1,X2,Y2,Quantity,Displayed,Value,Orientation,Support,Shape,Graduations, Cursor,ButtonsPos, ButtonsDef,Id, ShapeColor,ShapeShadeCol1,ShapeShadeCol2,CursorColor,ShapeImage,CursorImage,MPointer,SHint);
End;

Constructor ASlider.Create(X1,Y1,Size:Integer; Quantity,Value,Orientation:Longint; Graduations:Word; Cursor:Byte; MPointer,SHint:String);
Begin
Slider(MySlider, X1,Y1,Size, Quantity,Value,Orientation,Graduations, Cursor, MPointer,SHint);
End;

Constructor ASlider.Create(X1,Y1,Size:Integer; Quantity,Value,Orientation:Longint; Cursor:Byte; MPointer,SHint:String);
Begin
Slider(MySlider, X1,Y1,Size, Quantity,Value,Orientation, Cursor, MPointer,SHint);
End;

Constructor ASlider.Create(X1,Y1,Size:Integer; Quantity,Displayed,Value,Orientation:Longint; ButtonsPos:Byte; ButtonsDef:Boolean; ShapeColor:Longint; MPointer,SHint:String);
Begin
Slider(MySlider, X1,Y1,Size, Quantity,Displayed,Value,Orientation, ButtonsPos, ButtonsDef, ShapeColor, MPointer,SHint);
End;

Constructor ASlider.Create(X1,Y1,Size:Integer; Quantity,Displayed,Value,Orientation:Longint; Support,Shape:Byte; Graduations:Word; Cursor,ButtonsPos:Byte; ButtonsDef:Boolean; Id:Word; ShapeColor,ShapeShadeCol1,ShapeShadeCol2,CursorColor:Longint;
                           ShapeImage,CursorImage,MPointer,SHint:String);
Begin
Slider(MySlider, X1,Y1,Size, Quantity,Displayed,Value,Orientation,Support,Shape, Graduations, Cursor,ButtonsPos,ButtonsDef, Id, ShapeColor,ShapeShadeCol1,ShapeShadeCol2,CursorColor, ShapeImage,CursorImage,MPointer,SHint);
End;

Destructor ASlider.Free;
Begin
End;

Procedure ASlider.Refresh;
Begin
Slider_Refresh(MySlider);
End;

Procedure ASlider.Draw;
Begin
Slider_Draw(MySlider);
End;

Procedure ASlider.DrawShape;
Begin
Slider_DrawShape(MySlider);
End;

Procedure ASlider.DrawSupport;
Begin
Slider_DrawSupport(MySlider);
End;

Procedure ASlider.DrawCursor;
Begin
Slider_DrawCursor(MySlider);
End;

Procedure ASlider.DrawGraduations;
Begin
Slider_DrawGraduations(MySlider);
End;

Function ASlider.GetMsg:Byte;
Begin
Result:=Slider_GetMsg(MySlider);
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

Constructor AInputText.Define(X,Y,Siz:Integer; Len:Byte; CoB,Co2:Longint;Sty:Byte; ID:Word; Var Cin:String);
Begin
InputText_Define(X,Y,Siz,Len, CoB,Co2,Sty,ID, MyInputText, Cin);
End;

Constructor AInputText.Create(X,Y,Siz,Len:Integer; CoB,Co2:Longint; Sty:Byte; Id:Word; Var Cin:String);
Begin
InputText(X,Y,Siz,Len,CoB,Co2,Sty,Id, MyInputText, Cin);
End;

Destructor AInputText.Free;
Begin
End;

Procedure AInputText.Draw;
Begin
InputText_Draw(MyInputText);
End;

Procedure AInputText.Freeze(Status:Boolean);
Begin
InputText_Freeze(Status,MyInputText);
End;

Function AInputText.GetMsg(Var Cin:String):Byte;
Begin
Result:=InputText_GetMsg(MyInputText,Cin);
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

Constructor AInputSlider.Create(X1,Y1,Sx:Integer; Col1,Col2,Style:Longint; Min,Max,Step:Longint; Var Value:Longint; Id:Word);
Begin
InputSlider(X1,Y1,Sx, Col1,Col2,Style, Min,Max,Step, Value, Id, MyInputSlider);
End;

Destructor AInputSlider.Free;
Begin
End;

Procedure AInputSlider.Draw;
Begin
InputSlider_Draw(MyInputSlider);
End;

Procedure AInputSlider.Freeze(Status:Boolean);
Begin
InputSlider_Freeze(Status,MyInputSlider);
End;

Function AInputSlider.GetMsg(Var VCin:Longint):Byte;
Begin
Result:=InputSlider_GetMsg(MyInputSlider,VCin);
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

Constructor AColorPalette.Define(Px,Py,CellX,CellY,InterCell,Columns:Integer; FirstColor,LastColor:Longint; Count:Integer; PaletteFile:String; Selected,BorderType:Integer; Id:Word; MPointer:ShortString);
Begin
ColorPalette_Define(MyColorPalette, Px,Py,CellX,CellY,InterCell,Columns, FirstColor,LastColor,Count, PaletteFile, Selected,BorderType,Id, MPointer);
End;

Constructor AColorPalette.Create(Px,Py,CellX,CellY,InterCell,Columns:Integer; FirstColor,LastColor:Longint; Count,Selected,BorderType:Integer; Id:Word; MPointer:ShortString);
Begin
ColorPalette(MyColorPalette, Px,Py,CellX,CellY,InterCell,Columns, FirstColor,LastColor, Count,Selected,BorderType, Id, MPointer);
End;

Constructor AColorPalette.Create(Px,Py,CellX,CellY,InterCell,Columns:Integer; PaletteFile:String; Selected,BorderType:Integer; Id:Word; MPointer:ShortString);
Begin
ColorPalette(MyColorPalette, Px,Py,CellX,CellY,InterCell,Columns, PaletteFile, Selected,BorderType, Id, MPointer);
End;

Destructor AColorPalette.Free;
Begin
End;

Procedure AColorPalette.Draw;
Begin
ColorPalette_Draw(MyColorPalette);
End;

Procedure AColorPalette.ToggleCell(Index:Integer; Display:Boolean; Border:Byte);
Begin
ColorPalette_ToggleCell(MyColorPalette,Index,Display,Border);
End;

Function AColorPalette.GetSelected:Longint;
Begin
Result:=ColorPalette_GetSelected(MyColorPalette);
End;

Procedure AColorPalette.Freeze(Status:Boolean);
Begin
ColorPalette_Freeze(MyColorPalette,Status);
End;

Function AColorPalette.GetMsg:Byte;
Begin
Result:=ColorPalette_GetMsg(MyColorPalette);
End;

{||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||}

{*------------------------ PAS DE PROG PRINCIPAL ---------------------------*}

Begin
End.
