{ͻ
                               UNIT MosDisks                                
 ͹
  CREATOR....: PLANAS Jean-Pierre                                           
  UPDATED BY.: PLANAS Jean-Pierre                                           
  CREATED THE: 20.07.1998                                                   
  COUNTRY....: Perpignan (FRANCE)                                           
  DESCRIPTION: Disk and directories procedures & functions.                 
  AUTONOMOUS.: No                                                           
  UNIT SIZE..: 871 Lines and (12.400 Bytes compiled)                        
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

{$ASMMODE intel}
{$MODE objfpc}

UNIT MosDisks;

INTERFACE

Const
{$ifdef linux}
       DirSep = '/';
{$else}
       DirSep = '\';
{$endif}

Type

   TString1=String[1];                  {*String 1*}

{|Main Directories Functions|}
Function  Dsk_Rmdir(St:String):Boolean;           {*Remove directories*}
Function  Dsk_ChDir(Path:String):Boolean;         {*Change directory*}
Function  Dsk_Mkdir(Dir:String):Boolean;          {*Create directories*}
Procedure Dsk_GetDir(D: Byte; var S: String);     {*Get Current Directories*}

{|Directories|}
Function  Dsk_TestEmptyDir(S:String):Boolean;     {*Test if directory is empty*}
Function  Dsk_TestDirectory(Dir:String):Boolean;  {*Test a directory*}
Function  Dsk_ValideDirectory(Path:String):String;{*Test the syntax of a path*}
Function  Dsk_FExpand(S:String):String;           {*Get exact path*}
Function  Dsk_GetCurrentDirectory:String;         {*Get current directory*}
Function  Dsk_GetParentDirectory(Dir:String):String;{*Get parent directory*}
Function  Dsk_TestRoot(Dir:String):Boolean;       {*Test if the path is the root*}
Function  Dsk_GetHome:String;                     {*Get directory of launch*}

{|Disk|}
Function  Dsk_TestWritableDrive(Drv:TString1):Boolean;{*Test if you can write on a drive*}
Function  Dsk_GetCurrentDrv:Byte;                 {*Return Current drive*}
Function  Dsk_GetDeviceName(Root:Char):String;    {*Return the label of a disk*}
Function  Dsk_TestDeviceExist(Drv:Char):Boolean;  {*Test if device exist*}
{*TO IMPLEMENT*}
Function  Test_CDRom(Drv:Char):Boolean;           {*Test if the drive is a CDROM drive*}

IMPLEMENTATION

Uses Dos,
     MosMemory,
     MosStrings;

{*--------------------------Main Directories Functions----------------------*}

{ͻ
                             FUNCTION Dsk_RmDir                             
 ͹
  IN.....: St (STRING) = Directory to remove.                               
  OUT....: (BOOLEAN) = TRUE if no errors, else return FALSE.                
  EXPLAIN: Remove an empty directory.                                       
           This function is much more powerful than Rmdir because if could  
           remove all the sub-directories of a directory if no files are    
           found.                                                           
 ͼ}
Function Dsk_Rmdir(St:String):Boolean;

  Function RemoveDir(S:String):Boolean;
  Var DirInfo : SearchRec;   {*For FindFirst/FindNext*}
      P       : String;      {*For PathName*}
      Pile    : TStack;      {*A Stack*}
      Res     : Boolean;     {*Partial Result*}
  Begin
  If Not Stack_Init(Pile,Sizeof(SearchRec),20) Then
    Begin
    RemoveDir:=False;
    Exit;
    End;                                   {*Not enougth memory*}

  S:=FExpand(S);                           {*Get full path*}
  FindFirst(Str_CorrectPath(S+DirSep+'*.*'), AnyFile, DirInfo); {*Begin search*}
  While (DosError=0) And (DirInfo.Name[1]='.') do FindNext(DirInfo);
  While DosError = 0 do
    Begin
    P:=Str_CorrectPath(S+DirSep+DirInfo.Name);{*Path Name*}

    If (DirInfo.Attr And Directory<> Directory) Then
      Begin
      RemoveDir:=False;
      Exit;
      End;                                 {*Contain a file...*}

    If Stack_TestFull(Pile) Then
      Begin
      RemoveDir:=False;
      Exit;
      End;                                 {*Stack full*}

    Stack_Push(Pile,DirInfo);
    Res:=Dsk_TestEmptyDir(P);
    Stack_Pop(Pile,DirInfo);

    If Not Res And (DirInfo.Name[1]<>'.') Then
      Begin
      Stack_Push(Pile,DirInfo);
      RemoveDir:=RemoveDir(P);             {*Recurse*}
      Stack_Pop(Pile,DirInfo);
      End;
    {$i-}
    Rmdir(P);                              {*Remove the directory*}


    {$i+}
    If DosError<>0 Then
      Begin
      RemoveDir:=False;
      DosError:=0;
      Exit;
      End;                                 {*Impossible to remove the directory*}
    FindNext(DirInfo);
    End;

  DosError:=0;
  RemoveDir:=True;
  Stack_Done(Pile);                        {*Kill the stack*}
  End;

Begin
If Not RemoveDir(St) Then                  {*Call recurse proc*}
  Begin
  Dsk_RmDir:=False; Exit;
  End;

St:=FExpand(St);
If (St[length(St)]=DirSep) And (Length(St)>3) Then Dec(St[0]);
{$i-}
Rmdir(St);                                 {*Remove the directory*}
{$i+}
If DosError<>0 Then
  Begin
  Dsk_RmDir:=False;
  DosError:=0;
  Exit;
  End;                                     {*Impossible to remove the directory*}
Dsk_Rmdir:=True;
End;

{ͻ
                             FUNCTION Dsk_ChDir                             
 ͹
  IN.....: Path(STRING) = Path where the change of the dir must be          
  OUT....: (BOOLEAN) = TRUE if no problem, else FALSE                       
  EXPLAIN: This function is like Chdir of Tp7. But with less bugs.          
 ͼ}
Function Dsk_ChDir(Path:String):Boolean;
Begin
Path:=FExpand(Path);
If (Path[Length(Path)]=DirSep) And (Length(Path)>3) Then Dec(Path[0]);
{$i-}
ChDir(Path);
{$i+}
Dsk_ChDir:=Ioresult=0;
End;

{ͻ
                             FUNCTION Dsk_Mkdir                             
 ͹
  IN.....: Dir (STRING) = Directory to create.                              
  OUT....: (BOOLEAN) = TRUE if OK, else FALSE.                              
  EXPLAIN: Create the specified directory on the disk.                      
  EXAMPLE: Dsk_Mkdir('C:\MOS\FAQ');                                         
  NB.....: Create the entire path, even if there is more than one directory 
           to create.                                                       
           Long filenames are supported.                                    
 ͼ}
Function Dsk_Mkdir(Dir:String):Boolean;
Var io:Integer;
    b:boolean;

Begin
If Dsk_TestRoot(Dir) Then Begin Dsk_Mkdir:=TRUE;Exit;End;
If Dsk_TestDirectory(Dir) Then Begin Dsk_Mkdir:=TRUE;Exit;End;

Dir:=Dsk_ValideDirectory(Dir);
Dec(Dir[0]);

{$i-}
MkDir(Dir);
{$i+}
io:=ioresult;

Dir:=Dir+DirSep;
If io=0 Then Begin Dsk_Mkdir:=TRUE;Exit;End;
If io=3 Then

  Begin
  b:=Dsk_Mkdir(Dsk_GetParentDirectory(Dir));
  If not b Then Begin Dsk_Mkdir:=FALSE;Exit;End;
  Dec(Dir[0]);

  {$i-}
  MkDir(Dir);
  {$i+}
  io:=ioresult;

  If io<>0 Then Begin Dsk_Mkdir:=FALSE;Exit;End;
  End;
If io=5 Then Begin Dsk_Mkdir:=FALSE;Exit;End;
Dsk_Mkdir:=TRUE;
End;

{ͻ
                            PROCEDURE Dsk_GetDir                            
 ͹
  IN.....: D (BYTE) =                                                       
  OUT....: S (STRING) =                                                     
  EXPLAIN: This function is the same as GetDir from Tp7.                    
 ͼ}
Procedure Dsk_GetDir(D: Byte; var S: String);
Begin
GetDir(D,S);
End;

{*------------------------------ Directories -------------------------------*}

{ͻ
                         FUNCTION Dsk_TestEmptyDir                          
 ͹
  IN.....: S (STRING) = Directory to test.                                  
  OUT....: (BOOLEAN) = TRUE if the directory is empty, else FALSE.          
  EXPLAIN: Test if a directory is empty or not.                             
  NB.....: Empty = contain no files. Sub-dirs are not files.                
 ͼ}
Function Dsk_TestEmptyDir(S:String):Boolean;
Var DirInfo:SearchRec;
    Res:Boolean;
Begin
Res:=True;
FindFirst(Str_CorrectPath(S+DirSep+'*.*'), AnyFile, DirInfo); {*Begin of search*}
while DosError = 0 do
begin
  If (DirInfo.Attr And VolumeId<>VolumeId) And
     (DirInfo.Name[1]<>'.') Then Res:=False;            {*One file founded*}
  FindNext(DirInfo);
end;
DosError:=0;                                            {*Doserror:=0*}
Dsk_TestEmptyDir:=Res;                                  {*Return result*}
End;

{ͻ
                         FUNCTION Dsk_TestDirectory                         
 ͹
  IN.....: Dir (STRING) Dir to check for                                    
  OUT....: (BOOLEAN) True if directory exist False elsewhere                
  EXPLAIN: Test if the string is really a directory!                        
  EXAMPLE: Dsk_TestDirectory('c:\utils')                                    
           Dsk_TestDirectory('..') etc                                      
 ͼ}
Function Dsk_TestDirectory(Dir:String):Boolean;
Var fHandle: File;
    wAttr: Word;
{    DirInfo: SearchRec; }    {*For FindFirst*}
Begin
If (Pos('*',Dir)>0) Or (Pos('?',Dir)>0) Then
  Begin
  Dsk_TestDirectory:=False;
  Exit;
  End;

Dir:=Dsk_ValideDirectory(Dir);
Dir := Dir + '.';
{Writeln(Dir);
FindFirst(Dir,Directory,DirInfo);
while DosError = 0 do
begin
  Writeln(DirInfo.Name);
  FindNext(DirInfo);
end;

Dsk_TestDirectory := (Doserror=0);
 }
Assign( fHandle, Dir );
GetFAttr( fHandle, wAttr );
Dsk_TestDirectory := ( (wAttr And Directory) = Directory );
End;

{ͻ
                        FUNCTION Dsk_ValideDirectory                        
 ͹
  IN.....: Path (STRING) = Input path.                                      
  OUT....: Path (STRING) = Output path.                                     
  EXPLAIN: Verify if the path is ended by a '\' and if it's not, it put it. 
  EXAMPLE: Dsk_ValideDirectory('c:\dos') = 'c:\dos\'                        
 ͼ}

Function Dsk_ValideDirectory(Path:String):String;
Begin
Path:=FExpand(Path);

If Path[Length(Path)]<>DirSep
   Then Dsk_ValideDirectory:=Path+DirSep
   Else Dsk_ValideDirectory:=Path;
End;

{ͻ
                            FUNCTION Dsk_FExpand                            
 ͹
  IN.....: S : (STRING) = String input (a filename or a directory)          
  OUT....: (STRING) = String result                                         
  EXPLAIN: Expands a file name into a fully-qualified file name.            
           There is only one difference between FExpand and Dsk_FileExpand :
           A directory will finish with '\' character on WIN32 and          
           '/' on Linux                                                     
  CF.....: FExpand from Dos Tp7 unit.                                       
 ͼ}
Function Dsk_FExpand(S:String):String;
Begin
S:=FExpand(S);
If Dsk_TestDirectory(S) And (S[length(S)]<>DirSep) then S:=S+DirSep;

Dsk_FExpand:=S;
End;

{ͻ
                      FUNCTION Dsk_GetCurrentDirectory                      
 ͹
  IN.....: /                                                                
  OUT....: (STRING) = Disk and current path.                                
  EXPLAIN: Return the current path ended by '\'                             
  EXAMPLE: Writeln('Chemin courrant: ',Dsk_GetCurrentDirectory);            
  SEEALSO: Dsk_GetParentDirectory                                           
 ͼ}

Function Dsk_GetCurrentDirectory:String;
Begin
Dsk_GetCurrentDirectory:=Dsk_FExpand('.');                 {*Get Drawer*}
End;

{ͻ
                      FUNCTION Dsk_GetParentDirectory                       
 ͹
  IN.....: Dir (STRING) = Input Path                                        
  OUT....: (STRING) = Parent of the input path                              
  EXPLAIN: Get the parent directory, without "..","." etc ...               
  EXAMPLE: Dsk_GetParentDirectory('C:\MOS\FAQ')=c:\mos                      
  NB 1   : If Input Path is the root directory, return the root directory   
  NB 2   : If a disk don't exist, works without problems.                   
  SEEALSO: Dsk_GetCurrentDirectory                                          
 ͼ}
Function Dsk_GetParentDirectory(Dir:String):String;
Begin
Dir:=Dsk_FExpand(Dir);
Dsk_GetParentDirectory:=Dsk_FExpand(Dir+'..');
End;

{ͻ
                           FUNCTION Dsk_TestRoot                            
 ͹
  IN.....: Dir (STRING) = Input Path                                        
  OUT....: (BOOLEAN) = TRUE if the path is the root directory, else FALSE.  
  EXPLAIN: Test if it's a root directory                                    
  EXAMPLE: Dsk_TestRoot('C:\MOS\FAQ')=FALSE.                                
  NB.....: If the drive is a substition, return TRUE if Dir is the root of  
           the substition.                                                  
           Subst c:\dos x: , Dsk_TestRoot('x:')= TRUE                       
 ͼ}
Function Dsk_TestRoot(Dir:String):Boolean;
Begin
Dir:=FExpand(Dir);
If length(Dir)=3 Then Dsk_TestRoot:=TRUE
                 Else Dsk_TestRoot:=FALSE;
End;

{ͻ
                            FUNCTION Dsk_GetHome                            
 ͹
  IN.....: /                                                                
  OUT....: (STRING) = Path of the application.                              
  EXPLAIN: Get the path where the application is launch.                    
  EXAMPLE: Writeln('Path of the application = ',Dsk_GetHome).               
  NB.....: Don't return the '\' on the end of the string.                   
 ͼ}
Function Dsk_GetHome:String;
Var I:Integer;
    Path:String;
Begin
Path:=FExpand(ParamStr(0));          {*Search Name Prog*}
I:=Length(Path)+1;                   {*Recherche Chemin Prog Appelant*}
   Repeat Dec(I);
   Until Copy(Path,I,1)=DirSep;
Dsk_GetHome:=Copy(Path,1,I-1);       {*Send Dsk_GetHome Directory... *}
End;

{*------------------------------- DISK -----------------------------------*}
{ͻ
                       FUNCTION Dsk_TestWritableDrive                       
 ͹
  IN.....: Drv (CHAR) = Drive letter.                                       
  OUT....: (BOOLEAN) = TRUE if you have the permission to write on this     
                       Drive.                                               
  EXPLAIN: Test if you can write on the drive Drv.                          
 ͼ}
Function Dsk_TestWritableDrive(Drv:TString1):Boolean;
Var F:File;
Begin
{Dsk_TestWritableDrive:=False;Exit;}
Assign(F,Drv+':\testmos.tmp');
{$I-}
Rewrite(F);
{$I+}
If Ioresult<>0 Then
  Begin
  Dsk_TestWritableDrive:=False;
  Exit;
  End;

{$I-}
Close(F);
{$I+}
If Ioresult<>0 Then
  Begin
  Dsk_TestWritableDrive:=False;
  Exit;
  End;

{$I-}
Erase(F);
{$I+}
If Ioresult<>0 Then
  Begin
  Dsk_TestWritableDrive:=False;
  Exit;
  End;

Dsk_TestWritableDrive:=True;
End;

{ͻ
                         FUNCTION Dsk_GetCurrentDrv                         
 ͹
  IN.....: /                                                                
  OUT....: (BYTE) = N of the current drive (A=1, B=2, C=3 ...)             
  EXPLAIN: Return the N of the drive.                                      
 ͼ}
Function Dsk_GetCurrentDrv:Byte;
Var S:String;
Begin
GetDir(0,S);
S:=UPPER(S);
Dsk_GetCurrentDrv:=ord(S[1])-64;
End;

{ͻ
                         FUNCTION Dsk_GetDeviceName                         
 ͹
  IN.....: Root (CHRA) = Label of the device (Ex: "C").                     
  OUT....: (STRING) = Name of the volume                                    
  EXPLAIN: Return the name ot the specified device.                         
 ͼ}

Function Dsk_GetDeviceName(Root:Char):String;
Var DirInfo:SearchRec;                      {*Struct Search Files & Dirs*}
    Name,Root2:String;

Begin
Name:='';                                           {*Clear Name Before*}
Root2:=Upper(Copy(Root,1,1))+':\';                  {*Correction ventuelle*}
If (Copy(Root2,1,1)<'A') Or (Copy(Root2,1,1)>'Z')   {*Error dans le nom du Device*}
   Then Begin {NError:=640;} Exit; End;
FindFirst(Root2+'*.*',AnyFile,DirInfo);             {*Found First Elment*}

While DosError=0 Do Begin
  If DirInfo.Attr And $8=$8 Then Name:=DirInfo.Name;{*Nom du Repertoire*}
FindNext(DirInfo);                                  {*Search Next Elment*}
End;

If Length(Name)>8 Then                              {*Nom > 8 Chars*}
   Name:=Copy(Name,1,8)+Copy(Name,10,Length(Name)-1);
Dsk_GetDeviceName:=Name;                            {*Transmet Rsultat*}
End;

{ͻ
                        FUNCTION Dsk_TestDeviceExist                        
 ͹
  IN.....: Root (CHAR) = Label of the device.                               
  OUT....: (BOOLEAN): True if the device exist.                             
  EXPLAIN: Test if device exist.                                            
  EXAMPLE: If Dsk_TestDeviceExist('C')=True Then Writeln('"C:" Exist');     
  BUGS...: If drive A is present this function return TRUE for drive b:     
 ͼ}
Function Dsk_TestDeviceExist(Drv:Char):boolean;
begin
  if (DiskSize(Ord(Drv)-64)=-1) and not Test_Cdrom(Drv) then
    Result:=false
  else
    Result:=true;
end;

{ͻ
                            FUNCTION TEST_CDROM                             
 ͹
  IN.....: Drv (CHAR) = Letter of the drive to test                         
  OUT....: (BOOLEAN) = True if Drv is a CDROM drive, else return False      
  EXPLAIN: Test if a drive is a CD-ROM.                                     
  NB.....: I have never test it with 2 Cdrom, but i think it will work.     
 ͼ}

Function Test_CDROM(Drv:Char):Boolean;
Begin
{*TO IMPLEMENT*}
{Drv:=Upcase(Drv);
   Asm
   mov R,0
   mov ax,1500h
   xor bx,bx
   xor cx,cx
   int 2Fh
   @Retry:
   or  bx,bx
   jz  @fin
   mov al,Drv
   sub drv,65
   cmp drv,cl
   je  @Trouve
   dec bx
   inc cl
   jmp @Retry
   @Trouve:
   mov R,1
   @fin:
   End;
Test_CDROM:=R;
}
Drv := Drv; // To avoid warnings !
Result:=False;
End;


{----------------------------------------------------------------------------}

Begin
End.
