{ͻ
                               UNIT MosErrors                               
 ͹
  CREATOR....: CHEVEAU Frdric                                             
  UPDATED BY.: CHEVEAU Frdric                                             
  CREATED THE: 21.04.1999                                                   
  COUNTRY....: Perpignan (FRANCE)                                           
  DESCRIPTION: Errors and exceptions mannagement.                           
  AUTONOMOUS.: Partial                                                      
  UNIT SIZE..: 427 Lines and (13.536 Bytes compiled)                        
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosErrors;

{$Mode Objfpc}

INTERFACE

Uses   SysUtils;

Type

    TError=Record                       {*Errors / Exceptions structure*}
    Id           :Integer;              {*Error Id*}
    Level        :Shortint;             {*Error level for interception*}
    Recurent     :Shortint;             {*Error displaying conditions*}
    EClass       :String[64];           {*Error class (Runtime, MOS...)*}
    ESource      :String[128];          {*Error source (Gfx, Misc...)*}
    ELabel       :String;               {*Error description*}
    Address      :String[10];           {*Error address*}
    Date         :String[32];           {*Error date*}
    Time         :String[32];           {*Error time*}
    End;

procedure HandleException(E:Exception);
Procedure Error(Err:Integer);
Procedure Error_Display (E:TError);              {*Display an error*}
Procedure Error_DisplayText(E:TError);           {*Display an error in TextMode*}
Procedure Error_DisplayLast;                     {*Display again the last Error*}
Function  Error_Add     (StError:String):Boolean;{*Add an error to database*}
Function  Error_Delete  (Id:Integer):Boolean;    {*Delete an error in database*}
Function  Error_Load    (Id:Integer; Var TE:TError):Boolean;{*Load an error and fill the structure*}
Procedure Error_Manage;                          {*Manage errors and display them if needs*}

{*----------------------------- Debut Library ------------------------------*}

IMPLEMENTATION

Uses
      MosBase,
      MosColors,
      MosConfig,
      MosCrt,
      MosDialogs,
      MosFiles,
      MosGadgets,
      MosGraph,
      MosGraphics,
      MosImages,
      MosStrings,
      MosSystem,
      MosText,
      MosTime,
      MosTools,
      MosWindows;

{ͻ
                            PROCEDURE Error                                 
 ͹
  IN.....: Err (INTEGER) = Error number to generate.                        
  OUT....: /                                                                
  EXPLAIN: Generate an error and display attache message.                   
  SEEALSO: Error_Display                                                    
 ͼ}

Procedure Error(Err:Integer);
Begin
MOS_Errors.NError:=Err;
MOS_Errors.LastError:=Err;
Error_Manage;
End;

{ͻ
                        PROCEDURE Error_DisplayLast                         
 ͹
  IN.....: ErrorId (INTEGER) = Error number to generate.                    
  OUT....: /                                                                
  EXPLAIN: Display the last error generated.                                
  SEEALSO: Error                                                            
 ͼ}

Procedure Error_DisplayLast;
Begin
MOS_Errors.NError:=MOS_Errors.LastError;
Error_Manage;
End;

{ͻ
                             FUNCTION Error_Add                             
 ͹
  IN.....: StError (STRING) = Error description to add.                     
  OUT....: (BOOLEAN) = TRUE: Ok, FALSE: Error.                              
  EXPLAIN: Add an error to the database error.                              
           Format of the Error description :                                
             The string line is composed with 6 columns.                    
             Columns are separated by ';'.                                  
                                                                            
             Column  Description                                            
                1    Error Id (Maximum 32767).                              
                2    Error gravity (1=Fatal error to 4=Information).        
                3    Error repetition (0=No limit, 1=Appears only one time).
                4    Error type (Runtime or MOS).                           
                5    Error source (Specified manager).                      
                6    Error description.                                     
  NB.....: User errors Id must be >=1000.                                   
  SEEALSO: Error_Delete                                                     
 ͼ}

Function Error_Add(StError:String):Boolean;
Var Id,FName:String;
    Line,VId,Code:Integer;

Begin
Error_Add:=FALSE;                       {*De base*}

FName := Sys.GetPath(SP_DATAS) + Language.ErrorName;
Id:=Str_Get(StError,1,';');
Line:=File_TestString(FName,Id,1,1);
If Line<>0 Then Exit;                   {*Id existe ?*}
If Str_Count(StError,';')<>6 Then Exit; {*Nombre de colonnes Ok ?*}
Val(Id,VId,Code);                       {*Id en integer*}
If (VId<1000) Then Exit;                {*Error number must be >=1000*}
Error_Add:=File_AppendString(FName,StError);
End;

{ͻ
                           FUNCTION Error_Delete                            
 ͹
  IN.....: Id (INTEGER) = Error Id.                                         
  OUT....: (BOOLEAN) = TRUE: Ok, FALSE: Error.                              
  EXPLAIN: Delete an error record in the database error.                    
  NB.....: Users errors Id must be >=1000.                                  
           Only users Id can be deleted.                                    
  SEEALSO: Error_Add                                                        
 ͼ}

Function Error_Delete(Id:Integer):Boolean;
Var FName,StId:String;
    Line:Integer;

Begin
Error_Delete:=FALSE;
If (Id<1000) Then Exit;                 {*Error number must be >=1000*}

Str(Id,StId);
FName := Sys.GetPath(SP_DATAS) + Language.ErrorName;
Line  := File_TestString(FName,StId,1,1);
If Line<=0 Then Exit;                   {*Erreur Inconnue*}
Error_Delete:=File_DeleteLine(FName,Line); {*Delete error line*}
End;

{ͻ
                            FUNCTION Error_Load                             
 ͹
  IN.....: Id (INTEGER) = Error Id to load.                                 
  OUT....: (BOOLEAN) = TRUE: Ok, FALSE: Error.                              
  EXPLAIN: Load an error and fill the Error structure.                      
  SEEALSO: Error_Add, Error_Delete                                          
 ͼ}

Function Error_Load(Id:Integer; Var TE:TError):Boolean;
Var FName,StId,St:String;
    Line,Code:Integer;

Begin
Error_Load:=FALSE;                      {*Init All*}
TE.EClass :='?'; TE.ESource:='?'; TE.ELabel:='?'; TE.Address:='?';
TE.Id:=0; TE.Level:=0; TE.Recurent:=0;
TE.Date:=Time_GetSDate;
TE.Time:=Time_GetSTime(0);
TE.Id:=Id;                              {*Fill Id de base*}

Str(Id,StId);
FName := Sys.GetPath(SP_DATAS) + Language.ErrorName;
Line  := File_TestString(FName,StId,1,1);
If Line<=0 Then Line:=File_TestString(FName,'XXXX',1,1);{*Erreur Inconnue*}
If Line<=0 Then Exit;                   {*Erreur Non Trouve*}

St:=File_GetLine(FName,Line);
If St='' Then Exit;                     {*Cannot load the line !*}
St:=Str_CorrectChars(St);               {*Convert all accentuated chars*}
Error_Load:=TRUE;

Val(Str_Get(St,1,';'),TE.Id,Code);      {*Fill the Structure*}
If Code<>0 Then TE.Id:=Id;
Val(Str_Get(St,2,';'),TE.Level,Code);
Val(Str_Get(St,3,';'),TE.Recurent,Code);
TE.EClass :=Str_Get(St,4,';');
TE.ESource:=Str_Get(St,5,';');
TE.ELabel :=Str_Get(St,6,';');
End;

{ͻ
                        PROCEDURE Error_DisplayText                         
 ͹
  IN.....: E (TError) = Pointer to the TError structure.                    
  OUT....: /                                                                
  EXPLAIN: Display an error in text mode using the TError structure.        
  SEEALSO: Error_Display                                                    
 ͼ}

Procedure Error_DisplayText(E:TError);
Var A:Char;
Begin
Writeln('Internal Error ',E.Id,' captured. Gravity ',E.Level);
Writeln('"',E.ELabel,'"');
Writeln('Exception raised at ',E.Time,' the ',E.Date);
Writeln('Source : ',E.ESource,'. Class ',E.EClass);
Writeln;
Write  ('(C)ontinue or (Q)uit ?');
   Repeat
   A:=Upcase(Readkey);
   If A='Q' Then Halt(1);               {*Exit Viril !*}
   Until A='C';
Writeln;
End;

{ͻ
                          PROCEDURE Error_Display                           
 ͹
  IN.....: E (TError) = Pointer to the TError structure.                    
  OUT....: /                                                                
  EXPLAIN: Display an error using the TError structure.                     
  SEEALSO: Error, Error_DisplayLast                                         
 ͼ}

Procedure Error_Display(E:TError);
Var W1            :TWindow;             {*Structure Fentre*}
    Even          :Byte;                {*Rception Evnements Fentre*}
    XOk,XCancel   :Boolean;             {*Flags de Sorties*}
    IconName      :String[12];          {*Nom de l'icne  afficher*}
    But1          :TButton;
    Bt            :Array[1..3] of TButton;
    Error_Type    :String;
    St,St1        :String;              {*Temp miscellaneous*}
    BName         :String;
    Text1         :TText;

{************************* AFFICHE CONTENU FENETRE **************************}

Procedure REDRAW_FRAME;                 {*Dfinition Contenu Fentre*}
Var A             :Byte;                {*Loop*}
Begin
Win_Clip(Yes,W1);                       {*Clipping de la Fentre*}
Font_Set('LITT.CHR',4);
Error_Type:=Language.Loc(1727);
St1:=Language.Loc(1728);
If E.Recurent=0 Then St:=Str_Get(St1,1,';') Else St:=Str_FillZero(E.Recurent,1)+Str_Get(St1,2,';');
BName:=Sys.GetPath(SP_IMAGES)+'ERR_TIT.BMP';

Bevel_Title (019,144,349,199, CLLightGrey,CLWhite, BLLoweredLine,BLRaisedLine,
             TJOnLeft,TJOnTop,Yes,'MODN.CHR',4,$0067,Str_Get(St1,3,';'));
Case E.Level of
   0   : IconName:='I_QUEST.ICO';
   1   : IconName:='I_BOMB.ICO';
   2..3: IconName:='I_WARN.ICO';
   4   : IconName:='I_INFORM.ICO';
   End;
Show_Icon (20,18,Sys.GetPath(SP_ICONS)+IconName);
If BMP_TestFile(BName) Then BMP_ShowTransparency (059,017,351,194,0,0,BName,$0080,CLTransparency);
St1:=Language.Loc(4);
Button  (259,052,80,-1, CLTextButton,CLBgButton,$0060,But1,Language.Loc(1668));
Button  (039,214,90,-1, CLTextButton,CLBgButton,$0110,Bt[1],'_OK');
Button  (144,214,90,-1, CLTextButton,CLBgButton,$0000,Bt[2],Str_Get(St1,1,'|'));
Button  (249,214,90,-1, CLTextButton,CLBgButton,$0000,Bt[3],Str_Get(St1,2,'|'));

St1:=Language.Loc(2);
Display (019,055, CLText,Str_Get(St1,1,'|'));
Display (019,065, CLText,Str_Get(St1,2,'|'));
St1:=Language.Loc(3);
Display (014,077, CLText,Str_Get(St1,1,';')+Str_Get(Error_Type,E.Level+1,';'));
Display (014,090, CLText,Str_Get(St1,2,';')+E.EClass);
Display (014,103, CLText,Str_Get(St1,3,';')+E.ESource);
Display (014,116, CLText,Str_Get(St1,4,';')+E.Address);
Display (209,077, CLText,Str_Get(St1,5,';')+E.Date);
Display (209,090, CLText,Str_Get(St1,6,';')+E.Time);
Display (209,103, CLText,Str_Get(St1,7,';')+St);
Display (209,116, CLText,Str_Get(St1,8,';')+Str_FillZero(E.Id,5));

For A:=1 to Str_Count(E.ELabel,'|') do
   Begin                                {*Display labels*}
   Display_Define(Text1,025,151+(A-1)*HaF, -1,CLText,'ITAL.CHR',4,TJOnLeft,$0022,Str_Get(E.ELabel,A,'|'));
   Text1.Width:=320;
   Display_Draw(Text1);
   End;
Font_Set('LITT.CHR',4);

Win_Clip(No,W1);                        {*Restaure le Clipping Normal*}
End;

{************************** PROCEDURE PRINCIPALE ****************************}

Begin
If Not Win_Init(0,0,LaF*74,HaF1+HaF*24,$55D9,Language.Loc(1726),W1) Then Exit;

Win_Sup(0,0,0,0,0,0,0,0,0,CLWinBorder,$0A94,W1);
Win_Draw(W1);                           {*Affiche la Fentre*}

REDRAW_FRAME;                           {*Affiche le Contenu de la Fentre*}
XOk:=No; XCancel:=No;                   {*Initialise Flags de Sortie*}

Repeat
If Win_Ready(W1) Then                   {*Teste si Fentre Prte*}
   Begin                                {*Test des Gadgets Personnels*}
   Win_Clip(Yes,W1);                    {*Clipping de la Fentre*}

   If Button_GetMsg(But1, 1)=CMSelected Then Sys_ErrorsHistory(W1.X1+20,W1.Y1+30);
   If Button_GetMsg(Bt[1],1)=CMSelected Then XOk:=TRUE;
   If Button_GetMsg(Bt[2],1)=CMSelected Then Begin Win_Done(W1); Application.Close; End;
   If Button_GetMsg(Bt[3],1)=CMSelected Then ;//Warm_Reboot;
   Win_Clip(No,W1);                     {*Restaure le Clipping Normal*}
   End;

Even:=Win_Test(W1);                     {*Rception venements fentre*}
If Even=CMClosed Then XCancel:=Yes;     {*Fentre Ferme   => Sortie*}
If Even=CMMoved Then REDRAW_FRAME;      {*Fentre Dplace => Rafficher Contenu*}
Until (XOk) Or (XCancel);               {*Flags de Sortie*}

Win_Done(W1);                           {*Supprime Fentre*}
End;

{ͻ
                           PROCEDURE Error_Manage                           
 ͹
  IN.....: /                                                                
  OUT....: /                                                                
  EXPLAIN: Manage the errors and display them if needs.                     
  SEEALSO: Error                                                            
 ͼ}

Procedure Error_Manage;
Var ASeg,AOf  :String;                  {*Segment et Offset de l'Erreur*}
    Num       :Integer;
    E         :TError;                  {*Erreur  charger*}
    FName     :String;                  {*Nom du fichier stats*}

Begin
If MOS_Errors.NError=540 Then Exit;     {*Erreur de CTRL-Break => Pas Actions*}
If MOS_Errors.NError=538 Then Exit;     {*Erreur de Reset=> Pas Actions*}

If Id_Err1 And $20=$20 Then Exit;       {*On Traite dja l'Erreur*}

If MOS_Errors.NError=0 Then Exit;       {*Pas d'erreurs finalement !*}

Log.Add('Error raised ('+Str_FillZero(MOS_Errors.NError,4)+')');

If Not File_TestExist(Sys.GetPath(SP_DATAS) + Language.ErrorName) Then
   Begin                                {*Pas de fichier "Erreurs"*}
   Num:=5;                              {*Icne Adrienne !*}
   Req_Dialog('CRITICAL ERROR',
   'LOCALISATIONS FILES NOT FOUND ! (PROBABLY DELETED DURING A DOS SESSION).|'+
   'ERRORS AND TEXT LABELS CANNOT BE DISPLAYED.||'+
   'YOU MUST HAVE THE FILE "ENGLISH.LOC" IN THE SYSTEM DIRECTORY||'+
   'SYSTEM IS NOW INSTABLE ! DO YOU WANT TO CONTINUE ?',
   ' _CONTINUE | _TERMINATE |',Num);
   If Num<>1 Then Application.Close;    {*Sort Aprs avoir tout Ferm...*}
   End;

Sys.WinMoved:=False;
Sys.WinSized:=False;
ErrorAddr:=Nil;                         {*Clear Error Flag now !*}
Id_Err1:=Id_Err1 Or $20;                {*On Traite l'Erreur*}

Inc(MOS_Errors.QttErrors);
Error_Load(MOS_Errors.NError,E);        {*Load the Error...*}

FName:=Sys.GetPath(SP_DATAS)+'STATS.DAT';
Ini_UpdateKey(FName,'Errors','Errors level '+Str_FillZero(E.Level,1),+1); {*1 Erreur de plus*}
Ini_UpdateKey(FName,'Errors','Errors and exceptions',+1); {*1 Erreur de plus*}

ASeg:=Str_L2H(Seg(ErrorAddr),4);        {*Segment et Offset de l'Erreur*}
AOf :=Str_L2H(Ofs(ErrorAddr),4);
If MOS_Errors.NError>=500 Then E.Address:='NA'
                          Else E.Address:=ASeg+':'+AOf;

File_AppendString(Sys.GetPath(SP_DATAS)+'ERRORS.HIS', {*Save Error in History*}
   Str_FillZero(E.Id,5)+';'+E.EClass+';'+E.ESource+';'+E.ELabel+';'+E.Date+';'+E.Time+';'+E.Address+';');

If E.Level>MOS_Errors.Level Then        {*Error level < : Non interception !*}
   Begin
   MOS_Errors.NError:=0;
   Exit;
   End;

Play_Beep;                              {*Flash Erreur*}
If Test_TextMode Then Error_DisplayText(E){*Display in Text*}
                 Else Error_Display(E);   {*Display in Gfx the Error requester*}

Id_Err1:=Id_Err1 And ($FFFF-$20);       {*On ne Traite plus l'Erreur*}
Id_Err1:=Id_Err1 And ($FFFF-$80);       {*Error - Qtt Max Handles Reached*}
End;

{ͻ
                           PROCEDURE HandleException                        
 ͹
  IN.....: E (Exception) = Exception to manage                              
  OUT....: /                                                                
  EXPLAIN: In old modal forms, when an exception is trapped, this procedure  SEEALSO: Error                                                            
           is called                                                        
 ͼ}

Procedure HandleException(E:Exception);
Var Err:TError;

Begin
  With Err do
  Begin
     Id       := 0;
     Level    := 1;
     Recurent := 0;
     EClass   := 'Runtime';
     ESource  := 'Kernel';
     ELabel   := E.Message;
     Address  := '';
     Date     := '';
     Time     := '';
  End;

  Error_Display(Err);
End;

{*-------------------------- PROGRAMME PRINCIPAL ---------------------------*}

Begin
MOS_Errors.NError:=0;
MOS_Errors.LastError:=0;
MOS_Errors.QttErrors:=0;
MOS_Errors.Level:=3;
End.
