{ͻ
                               UNIT MosGauges                               
 ͹
  CREATOR....: CHEVEAU Frdric                                             
  UPDATED BY.: CHEVEAU Frdric                                             
  CREATED THE: 08.05.2000                                                   
  COUNTRY....: Montpellier - Timeless (FRANCE)                              
  DESCRIPTION: Visual components "TGauge, TScope".                          
  AUTONOMOUS.: No                                                           
  UNIT SIZE..: 3.718 Lines and (93.136 Bytes compiled)                      
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosGauges;

{$Mode Objfpc}

INTERFACE

Uses
     Classes,
     MosBevels,
     MosControls,
     MosTexts;

Type                                    {*Types definition*}

{****************************************************************************}

TScopeStates = (                        {*TScope component state*}
   SSLeftToRight1,
   SSLeftToRight2,
   SSRightToLeft1,
   SSRightToLeft2);

{****************************************************************************}
{*** TGauge Class                                                         ***}
{****************************************************************************}

TGauge = Class(TVisual)                 {*TGauge class definition*}
   Private
      fMin              :Longint;       {*Minimum value*}
      fMax              :Longint;       {*Maximum value*}
      fStep             :Longint;       {*Step for progression*}
      fProgress         :Longint;       {*Progression value*}
      fPercent          :Boolean;       {*Work in percent or custom*}
      fShape            :AShape;        {*Shape to draw progression*}
      fBlock            :AShape;        {*Displayed blocks*}
      fText             :AText;         {*Text to see percentage*}
      fTextPosition     :THPosition;    {*Text position regarding to TGauge*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetMin(M:Longint);      {*Set minimum*}
      Procedure SetMax(M:Longint);      {*Set maximum*}
      Procedure SetStep(S:Longint);     {*Set step for progression*}
      Procedure SetProgress(P:Longint); {*Set progression value*}
      Procedure SetPercent(P:Boolean);  {*Set percent unit or no*}
      Procedure SetTextPosition(P:THPosition);  {*Set text position*}
      Procedure SetSaveBg(S:Boolean); Override; {*Refresh screen or no*}

   Public
      Property  Max          :Longint     Read fMax           Write SetMax;
      Property  Min          :Longint     Read fMin           Write SetMin;
      Property  Step         :Longint     Read fStep          Write SetStep;
      Property  Progress     :Longint     Read fProgress      Write SetProgress;
      Property  Percent      :Boolean     Read fPercent       Write SetPercent;
      Property  TextPosition :THPosition  Read fTextPosition  Write SetTextPosition;
      Property  Shape        :AShape      Read fShape;
      Property  Block        :AShape      Read fBlock;
      Property  Text         :AText       Read fText;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pMin,pMax:Longint; pColor:Longint); {*Define component*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** TScope Class                                                         ***}
{****************************************************************************}

TScope = Class(TVisual)                 {*TScope class definition*}
   Private
      fShade            :TShade;        {*Shade component*}
      fSpeed            :Integer;       {*Animation speed in pixels per round*}
      fState            :TScopeStates;  {*Component state for displaying*}
      fSpeedCounter     :Integer;       {*Loop counter for speed*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Property  SpeedCounter:Integer     Read fSpeedCounter;

      Procedure SetState(S:TScopeStates); {*Animation state*}

   Public
      Property  Shade       :TShade       Read fShade;
      Property  Speed       :Integer      Read fSpeed   Write fSpeed;
      Property  State       :TScopeStates Read fState   Write SetState;

      Constructor Create(AOwner:TComponent); Override; {*Create component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pFirstColor,pLastColor:Longint; pDirection:TDockOrientation); {*Define component*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Procedure   Hide; Override;                 {*Hide component*}
      Procedure   MoveTo(Xa,Ya:Integer); Override;{*Move component to another position*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{}
{}
{}

IMPLEMENTATION

Uses
     MosBase,
     MosColors,
     MosConfig,
     MosStrings,
     MosTools;

{****************************************************************************}
{*** TGauge Class                                                         ***}
{****************************************************************************}

Procedure TGauge.Define;
Begin
fBlock.Vacuum := 1;
fPercent      := Yes;
fMin          := 0;                     {*Needed for the SetMin, SetMax methods*}
fMax          := 100;
SetMin(fMin);
SetMax(fMax);

If Owner = Self Then Log.Add('Create new TGauge ('+')',2);
End;

Constructor TGauge.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fText  := AText.Create(AOwner);
fShape := AShape.Create(AOwner);
fBlock := AShape.Create(AOwner);

Define;
End;

Constructor TGauge.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pMin,pMax:Longint; pColor:Longint);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fText  := AText.Create (AOwner, pX1,pY1,'');
fShape := AShape.Create(AOwner, pX1,pY1,pX2,pY2, pColor);
fBlock := AShape.Create(AOwner, pX1,pY1,pX1+12,pY2, CLBlue);

Define;

SetMin(pMin);
SetMax(pMax);
End;

Destructor TGauge.Destroy;
Begin
fBlock.Destroy;
fShape.Destroy;
fText.Destroy;
Inherited Destroy;
End;

Procedure TGauge.Draw;
Var SaveBlockColor   :Longint;
    Loop             :Integer;
    NbBlocks         :Integer;

Begin
If Not Visible Then Exit;
Inherited Draw;

fShape.Draw;                            {*Draw main shape*}

If fPercent Then                        {*Draw value in percent*}
   Begin
   fText.Caption := Str_L2S(Round(Real(fProgress * 100) / (fMax-fMin))) + ' %';
   End
Else
   Begin                                {*Draw value in natural units*}
   fText.Caption := Str_L2S(fProgress);
   End;
fText.Draw;                             {*Draw text (optional)*}

SaveBlockColor := fBlock.Color;         {*Save block color*}

NbBlocks := Round(Real(fShape.Width) / fBlock.Width) + 1;

For Loop := 1 to NbBlocks do            {*Draw each block*}
   Begin
   fBlock.Draw;                         {*Draw one block*}
   fBlock.X1 := fBlock.X1 + NbBlocks * fBlock.Width;
   If fBlock.X2 > fShape.X2 Then fBlock.X2 := fShape.X2;
   If fBlock.X1 > Round(Real(fProgress * fShape.Width) / (fMax-fMin))
      Then fBlock.Color := fShape.Color;{*Set fill color to background color*}
   End;
fBlock.Color := SaveBlockColor;         {*Restore block color*}
End;

Function TGauge.GetMsg:TCompMessage;
Begin
Result := CMNone;
If Not Enabled Then Exit;

Result := Inherited GetMsg;
End;

Procedure TGauge.SetMax(M:Longint);
Begin
If M = fMax  Then Exit;
If M <= fMin Then Exit;
fMax := M;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TGauge.SetMin(M:Longint);
Begin
If M = fMin  Then Exit;
If M >= fMax Then Exit;
fMin := M;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TGauge.SetStep(S:Longint);
Begin
If S = fStep Then Exit;
fStep := S;
End;

Procedure TGauge.SetSaveBg(S:Boolean);
Begin
Inherited SetSaveBg(S);
//fText.SetSaveBg(S);
End;

Procedure TGauge.SetProgress(P:Longint);
Begin
If P = fProgress Then Exit;
If P < fMin Then Exit;
If P > fMax Then Exit;
fProgress := P;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TGauge.SetTextPosition(P:THPosition);
Begin
If P = fTextPosition Then Exit;
fTextPosition := P;
Invalidate;                             {*Redraw if needs*}
End;

Procedure TGauge.SetPercent(P:Boolean);
Begin
If P = fPercent Then Exit;
fPercent := P;
Invalidate;                             {*Redraw if needs*}
End;

{****************************************************************************}
{*** TScope Class                                                         ***}
{****************************************************************************}

Procedure TScope.Define;
Begin
fState        := SSLeftToRight1;
fSpeed        := 1;
fSpeedCounter := 0;

If Owner = Self Then Log.Add('Create new TScope ('+')',2);
End;

Constructor TScope.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fShade := TShade.Create(AOwner);

Define;
End;

Constructor TScope.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pFirstColor,pLastColor:Longint; pDirection:TDockOrientation);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
fShade := TShade.Create(AOwner, pX1,pY1,pX1,pY2, pFirstColor,pLastColor, pDirection);

Define;
End;

Destructor TScope.Destroy;
Begin
fShade.Destroy;
Inherited Destroy;
End;

Procedure TScope.Hide;
Begin
fShade.Hide;
Inherited Hide;
End;

Procedure TScope.MoveTo(Xa,Ya:Integer);
Begin
//Hide;                                   {*Before operate*}
Inherited MoveTo(Xa,Ya);
fShade.MoveTo(Xa,Ya);
End;

Procedure TScope.Draw;
Begin
If Not Visible Then Exit;
Inherited Draw;

fShade.Draw;                            {*Draw shade*}

If fShade.Direction = DOHorizontal Then {*Fill the rest with Bg color*}
   Begin
   If (fState = SSLeftToRight1) Or (fState = SSRightToLeft2) Then Bevel(fShade.X2+1,fShade.Y1,X2,fShade.Y2, SupportColor);
   If (fState = SSLeftToRight2) Or (fState = SSRightToLeft1) Then Bevel(X1,fShade.Y1,fShade.X1-1,fShade.Y2, SupportColor);
   End
Else
   Begin
   End;
End;

Procedure TScope.SetState(S:TScopeStates);
Begin
If S = fState Then Exit;
fState := S;
Invalidate;                             {*Redraw if needs*}
End;

Function TScope.GetMsg:TCompMessage;
Var HasChanged       :Boolean;
    ColTemp          :Longint;

Begin
Result := CMNone;
If Not Enabled Then Exit;

Inc(fSpeedCounter);
If fSpeedCounter >= Speed Then fSpeedCounter := 0;

HasChanged := No;
Case fState of
   SSRightToLeft2:
      Begin
      If fSpeedCounter = 0 Then
         Begin
         HasChanged := Yes;
         fShade.X2 := fShade.X2 - 1;
         End;
      If fShade.X2 < X1 Then
         Begin
         fShade.X2 := X1;
         fState := SSLeftToRight1;
         End;
//wait;
      End;
   SSRightToLeft1:
      Begin
      If fSpeedCounter = 0 Then
         Begin
         HasChanged := Yes;
         fShade.X1 := fShade.X1 - 1;
         End;
      If fShade.X1 < X1 Then
         Begin
(*         ColTemp := fShade.FirstColor;  {*Swap colors*}
         fShade.FirstColor := fShade.LastColor;
         fShade.LastColor := ColTemp;*)
         fShade.X1 := X1;
         fState := SSRightToLeft2;
         End;
//wait;
      End;
   SSLeftToRight2:
      Begin
      If fSpeedCounter = 0 Then
         Begin
         HasChanged := Yes;
         fShade.X1 := fShade.X1 + 1;
         End;
      If fShade.X1 > X2 Then
         Begin
         fShade.X1 := X2;
         fState := SSRightToLeft1;
         End;
//wait;
      End;
   SSLeftToRight1:
      Begin
      If fSpeedCounter = 0 Then
         Begin
         HasChanged := Yes;
         fShade.X2 := fShade.X2 + 1;
         End;
      If fShade.X2 > X2 Then
         Begin
         fShade.X2 := X2;
         fState := SSLeftToRight2;
         End;
//wait;
      End;
   End;

If HasChanged Then Draw;                {*Draw Scope permanently*}

Result := Inherited GetMsg;
End;

{******************************* NO MAIN CODE *******************************}

Begin
End.
