{ͻ
                              UNIT MosGraphics                              
 ͹
  CREATOR....: PLANAS Jean-Pierre, CHEVEAU Frdric                         
  UPDATED BY.: PLANAS Jean-Pierre                                           
  CREATED THE: 26.11.1994                                                   
  COUNTRY....: Perpignan (FRANCE)                                           
  DESCRIPTION: Management of video.                                         
  AUTONOMOUS.: Partial                                                      
  UNIT SIZE..: 521 Lines and (11.312 Bytes compiled)                        
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosGraphics;

{$MODE Objfpc}

INTERFACE

Uses
     MosGraph,
     MosConfig;

Type TPalette256 = Packed Array[0..255,1..3] of Byte;  {*256 Red/Green/Blue (RGB)*}

TRefresh = Class(TObject)               {*Refresh screen zone component*}
   Private
      fImage            :Pointer;       {*Pointer to the image array*}
      fBgStored         :Boolean;       {*Component background stored*}

   Public
      Property Image    :Pointer Read fImage    Write fImage;
      Property BgStored :Boolean Read fBgStored                   Default No;

      Constructor Create;               {*Create the component*}
      Destructor  Destroy; Override;    {*Destroy the component*}
      Procedure   GetImageFromScreen(X1,Y1,X2,Y2:Integer); {*Save image*}
      Procedure   PutImageToScreen(X1,Y1:Integer);         {*Display stored image*}
   End;

     TSaveScreen = Record            {*Type for Save/Restore_Screen*}
       What_save     : Word;         {*Bits of what you want to save*}
       ModeG         : Word;         {*GFX Mode*}
       Palette256    : Tpalette256;  {*256 colors palette*}
       ImageSwap     : TRefresh;     {*Picture displayed on screen*}
       MouseX,MouseY : Word;         {*Mouse coordinates*}
       ViewPort      : TViewPortType;{*VESA_SetViewPort parameters*}
     End;

{|Save/Restore Screen|}
Procedure Save_Screen(What_to_save:Word);          {*Save screen and it's state*}
Procedure Restore_Screen;                          {*Restore the screen like before a Save_Screen*}

{|Icons|}
Function  Test_Icon(S:String):Boolean;             {*Is file an icon?*}
Procedure Show_Icon(Wx,Wy:Word; S:String);         {*Show icon*}

{|GIF|}
Function  Test_GIF(NameFile:String):Boolean;       {*Is file a gif?*}
Procedure Get_GIFInfo(NameFile:String;Var ScreenX,ScreenY,Colors:Word);{*To get information about the gif*}

IMPLEMENTATION

Uses
     Dos,
     Graphix,
     GxImg,
     MosBase,
     MosCPUInstructions,
     MosErrors,
     MosFiles,
     MosMouse;

{*-------------------------------- TRefresh --------------------------------*}

{ͻ
                        Constructor TRefresh.Create                         
 ͹
  IN.....: /                                                                
  OUT....: /                                                                
  EXPLAIN: Initialize TRefresh object.                                      
 ͼ}

Constructor TRefresh.Create;
Begin
//If Assigned(Log) Then Log.Add('TRefresh.Create');
fImage:=Nil;                            {*Init pointer*}
fBgStored:=No;
End;

{ͻ
                       Destructor TRefresh.Destroy                          
 ͹
  IN.....: /                                                                
  OUT....: /                                                                
  EXPLAIN: Kill TRefresh object.                                            
 ͼ}

Destructor TRefresh.Destroy;
Begin
//If Assigned(Log) Then Log.Add('TRefresh.Destroy');
If fImage<>Nil Then DestroyImage(fImage);{*Free Mem & Pointers*}
fBgStored:=No;
End;

{ͻ
                         Procedure GetImageFromScreen                       
 ͹
  IN.....: X1,Y1,X2,Y2 (LONGINT) = Coordinates of a piece of screen.        
  OUT....: /                                                                
  EXPLAIN: Save a piece of screen.                                          
 ͼ}

Procedure TRefresh.GetImageFromScreen(X1,Y1,X2,Y2:Integer);
Begin
If fBgStored Then DestroyImage(fImage);  {*Free old image*}

Log.Add('TRefresh.GetImageFromScreen');

If (X2>VESA_GetMaxX) Or (Y2>VESA_GetMaxY) Then
   Begin
   Error(520);
   Exit;
   End;

Mse_Hide;
fImage:=CreateImage(x1,y1,x2,y2);
GetImage(x1,y1,fImage);
Mse_Show;
fBgStored:=Yes;
End;

{ͻ
                         Procedure PutImageToScreen                         
 ͹
  IN.....: X1,Y1 (WORD) = Coordinates on the screen.                        
  OUT....: /                                                                
  EXPLAIN: Put on the screen the saved piece of screen.                     
 ͼ}

Procedure TRefresh.PutImageToScreen(X1,Y1:Integer);
Begin
If Not fBgStored Then Exit;             {*Component is not yet stored*}

Log.Add('TRefresh.PutImageToScreen');
Mse_Hide;
If fImage<>Nil Then PutImage(X1,Y1, fImage);
Mse_Show;
End;

{*-------------------------- Save/Restore Screen ---------------------------*}

{ͻ
                           PROCEDURE SAVE_SCREEN                            
 ͹
  IN.....: What_to_save (Word) = Bit Mask                                   
           Bit  Num    Save or make if this bit is on : ...                 
            0   0001   Save Video Mode                                      
            1   0002   Color palette                                        
            2   0004   The picture of the screen                            
            3   0008   Mouse pointer                                        
            4   0010   Mouse coordinates                                    
            5   0020   Mouse speed                                          
            6   0040   Make a Mse_Show                                      
            7   0080   Make a Mse_Hide after the save_screen                
            8   0100   VESA_SetViewPort parameters saved                    
            9   0200                                                        
            10  0400                                                        
            11  0800                                                        
            12  1000                                                        
            13  2000                                                        
            14  4000                                                        
            15  8000                                                        
  OUT....: /                                                                
  EXPLAIN: This procedure save a picture, his video mode and when you will  
           call RESTORE_SCREEN, all will be put on the initial state.       
           This procedure save only what you want to save in the var        
           What_to_save.                                                    
  SEEALSO: Restore_Screen                                                   
 ͼ}

Var VSave_Screen:TSaveScreen;

Procedure SAVE_SCREEN(What_to_save:Word);
var G,M,D : Boolean; {*Boutons de la souris*}
begin
 With Vsave_Screen do
   begin
   What_save:=What_to_save;
   If What_Save and $100=$100 then             {*VESA_SetViewPort*}
     Begin
     VESA_GetViewSettings(ViewPort);                {*Get Old ViewPort*}
     VESA_SetViewPort(0,0,VESA_GetMaxX,VESA_GetMaxY,ClipOn);  {*Set All Screen ViewPort*}
     End;
   If What_save and $80=$80  then Mse_Hide;   {*Mse_Hide*}
   If What_Save and $10=$10 then               {*Save Mouse position*}
     Begin
     Mse_Get(MouseX,MouseY,G,M,D);
     End;
   If What_save and $4=$4 Then                 {*Save image screen*}
     Begin
     ImageSwap:=TRefresh.Create;
     ImageSwap.GetImageFromScreen(0,0,VESA_GetMaxX,VESA_GetMaxY);
     End;
(*   If What_save and $2=$2 Then                 {*Save palette*}
    If (Sys.GfxMode<20) And (Sys.GfxMode>=10) Then VESA_GetPalette256(Palette256)
      Else
      For i:=0 To 15 Do Vesa_getrgbpalette(col[i],Palette256[i,1],Palette256[i,2],Palette256[i,3]);
*)
   If What_save And $1=$1 Then ModeG:=Sys.GfxMode(*VESA_GetMode; {*Save Graphic Mode*}*)
 end;
end;

{ͻ
                          PROCEDURE RESTORE_SCREEN                          
 ͹
  IN.....: /                                                                
  OUT....: /                                                                
  EXPLAIN: This procedure restore the picture of screen, video like the     
           call of save_screen has define the parameters.                   
  SEEALSO: Save_Screen                                                      
 ͼ}

Procedure RESTORE_SCREEN;
begin
 With VSave_Screen do
   begin
   If What_save and $1=$1 then
     begin
     Sys.GfxMode:=ModeG;
     Init_Graph;
     (*VESA_SetMode(ModeG); {*VESA_SetMode Graphique*}*)
     end;
(*   If What_save and $2=$2 then                 {*Set palette*}
     If id_msg1 and $40=$40 then VESA_SetPalette256(Palette256) else
       For i:=0 to 15 do VESA_SetRGBPalette(col[i],Palette256[i,1],Palette256[i,2],Palette256[i,3]);
*)
   If What_save and $4=$4 then                 {*Restore image screen*}
     Begin
     imageswap.PutImageToScreen(0,0);
     imageswap.Destroy;
     End;
   If What_save and $40=$40 then               {*Replace Souris au Bon Endroit*}
     Begin
     Mse_Test;
     Mse_Put(MouseX,MouseY);
     Mse_Show;
     End;
   If What_save and $8=$8 then                 {*Set Pointer Souris*}
     Begin
     Mse_SetPointer(p_current);
     Mse_Speed(Sys.MouseSpeed,Sys.MouseSpeed);       {*Vitesse Souris*}
     End;
   If What_Save and $80=$80 then               {*Restore Old ViewPort*}
     With ViewPort do VESA_SetViewPort(X1,Y1,X2,Y2,ClipOn);
   End;
End;

{*-------------------------------- Icons -----------------------------------*}

{ͻ
                            PROCEDURE SHOW_ICON                             
 ͹
  IN.....: WX,WY (INTEGER) = Coordinates where to draw the Icon             
           S (STRING) = Path + Name of the icon file.                       
  OUT....: /                                                                
  EXPLAIN: Draw an icon at WX,WY coordinates. It is 32*32 pixel icons.      
           The icons files are the same like windows (.ICO)                 
  BUGS...: 1) This procedure don't load the icon palette. And the palette  
               of MOS is different of the icon palette..                    
           2) This procedure don't verify if S is an icon file...          
           3) Transparency not good                                        
  SEEALSO: Test_Icon                                                        
 ͼ}

Procedure SHOW_ICON(Wx,Wy:word; S:String);
Var P :Pointer;
    VP:TViewPortType;

Begin
S := FExpand(S);
If Not Test_Icon(S) Then
   Begin
   Error(730);                          {*Error File not found*}
   Exit;
   End;

If LoadImageFile(itICO,S,p,1) > 0 Then  {*Load from file...*}
   Begin
   Error(730);
   Exit;
   End;

VESA_GetViewSettings(VP);          {*Save ViewPort*}

   If BTST(Id_Scr1,3) Then         {*Window Clipped*}
   Begin
      VESA_SetViewPort(T_Clip.X1, T_Clip.Y1, T_Clip.X2, T_Clip.Y2, TRUE);
   End;

Mse_Hide;
VESA_PutImageT(T_Clip.X1+Wx-1, T_Clip.Y1+Wy-6, p^,normalput,getimagetransparencycolor(p));
Mse_Show;

VESA_SetViewPort(VP.X1,VP.Y1,VP.X2,VP.Y2,VP.Clip); {*Restore ViewPort*}

DestroyImage(p);
End;

{ͻ
                             FUNCTION TEST_ICON                             
 ͹
  IN.....: S (STRING) = Path + Name of the icon file to test                
  OUT....: (BOOLEAN) = TRUE if it's a windows icon file (readable by MOS)   
                       FALSE if the file is not an icon file.               
  EXPLAIN: Answer the question : Is this file an icon file ?                
  OPT....: File_TestHeader                                                  
  SEEALSO: Show_Icon                                                        
 ͼ}

Function TEST_ICON(S:String):boolean;
Type TWinIconColor = Packed Record
       vBlue,
       vGreen,
       vRed,
       vUnused : Byte;
     End;

     TIcone=Packed Record
       vIdReserved,
       vIdType,
       vIdCount     :Word;
       vIconWidth,                    (* 1 For each icon in the File *)
       vIconHeight,
       vColorCount,
       vReserved    : Byte;
       vPlanes,
       vBitCount    : Word;
       vBytesInRes,
       vImageoffset : LongInt;
       vHdrSize,                 (* Always 40 For Windows icons *)
       vPixelWidth,
       vPixelHeight   : LongInt;
       vColorPlanes,             (* Should always be 1 *)
       vBitCount2     : Word;
       vCompression,
       vImageSize,
       vXPelsPerMeter,
       vYPelsPerMeter,
       vClrUsed,
       vClrImportant  : LongInt;
       vRGBTable   : Array[0..15] of TWinIconColor;
       vxorMask    : Array[0..511] of Byte;
       vandMask    : Array[0..127] of Byte;
     End;

var f:file of TIcone;
    icon:TIcone;
    DirInfo: SearchRec;

begin
If not File_TestExist(s) then test_icon:=false else Test_Icon:=True;
Exit;
 If not File_TestExist(s) then test_icon:=false else
  begin
   assign(f,s);
   {$i-}
   reset(f);
   {$i+}
   if ioresult=0 then

    begin
     findfirst(s, Archive, DirInfo);
     if Dirinfo.size<>766 then TEST_ICON:=false else

      begin
       {$i-}
       read(f,icon);
       {$i+}
       if (ioresult<>0) or (icon.vHdrSize<>40) or (icon.vColorPlanes<>1) then TEST_ICON:=false
       else TEST_ICON:=true;
      end;
     close(f);
    end else test_icon:=false;
  end;
end;

{*--------------------------------- GIF ------------------------------------*}

{ͻ
                             FUNCTION TEST_GIF                              
 ͹
  IN.....: NameFile (STRING) = Path + Name of the file to test.             
  OUT....: (BOOLEAN) = TRUE : if it's a GIF picture, else FALSE.            
  EXPLAIN: Answer the question : Is this file a GIF picture ?               
  SEEALSO: Get_GIFInfo                                                      
 ͼ}

Function TEST_GIF(NameFile:string):Boolean;
Begin
TEST_GIF:=File_TestHeader(NameFile,'GIF87A',0) Or
          File_TestHeader(NameFile,'GIF89A',0);
End;

{ͻ
                           PROCEDURE Get_GIFInfo                            
 ͹
  IN.....: NameFile (STRING) = Path + Name of the GIF file to get infos.    
  OUT....: ScreenX (WORD) = Horizontal Size (Width) of the GIF file.        
           ScreenY (WORD) = Vertical Size (Heigth) of the GIF file.         
           Colors  (WORD) = Number of colors in the palette.                
  EXPLAIN: Get informations on a GIF File.                                  
  RQ.....: If the file is not a GIF file, ScreenX,ScreenY and Colors are    
           all equal to 0                                                   
  OPT....: Perharps.                                                        
  SEEALSO: Test_Gif                                                         
 ͼ}

Procedure Get_GIFInfo(NameFile:string;Var ScreenX,ScreenY,Colors:Word);
Type Gif_Header = Record
	   Sig, Ver                        : Array[1..3] of Char;
	   Screen_X,Screen_Y               : Word;
	   _Packed,Background,Pixel_Aspect : Byte;
     end;
 { first 13 Bytes of a Gif                                               }
 { sig='GIF' and Ver=version number (87a or 89a)                         }

Procedure Notgif;
begin
 ScreenX:=0;
 ScreenY:=0;
 Colors:=0;
end;

Var  F : File;
     G : GIF_Header;i:byte;
     E : Word;
begin
 If TEST_GIF(Namefile) then
  begin
   Assign(F,Namefile);
   {$i-}
   Reset(F, 1);
   {$i+}
   if ioresult=0 then
    begin
     {$i-}
     Blockread(F,G,SizeOf(G),e);
     {$i+}
     If (ioresult<>0) or (e<>Sizeof(G)) then
      begin
       Notgif;
       close(f);
       exit;
      end;
     Close(F);
     With G DO
     begin
      for i:=1 to 3 do
       begin
        sig[i]:=upcase(sig[i]);
        ver[i]:=upcase(ver[i]);
       end;
      If (sig='GIF') and ((ver='87A') or (ver='89A')) then
       begin
        ScreenX:=Screen_X;
        ScreenY:=Screen_Y;
        colors:=2 SHL (_Packed and 7);
       end
       else notgif;
     end;
    end else notgif;
 end else notgif;
end;

{*------------------------------ MAIN PROGRAM ------------------------------*}

Begin
End.
