{ͻ
                               UNIT MosImages                               
 ͹
  CREATOR....: PLANAS Jean-Pierre                                           
  UPDATED BY.: PLANAS Jean-Pierre                                           
  CREATED THE: 15.07.1995                                                   
  COUNTRY....: Perpignan (FRANCE)                                           
  DESCRIPTION: Procedures to manage images (Display and information).       
  AUTONOMOUS.: Partial                                                      
  UNIT SIZE..: 1.505 Lines and (24.416 Bytes compiled)                      
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

{$ASMMODE intel}
{$MODE objfpc}

UNIT MosImages;

INTERFACE

Uses
     Classes,
     MosConfig,
     MosControls,
     MosGraph;

Type TImage=Record                      {*TImage structure*}
     C                  :TCoordinates;  {*Coordinates of the picture*}
     OffsetX,OffsetY    :Integer;       {*X,Y offsets*}
     End;

     TBMPHeader=Packed Record   { Header of a BMP file}
       ID    : Array[1..2] of char; { 'BM' for a Windows BitMaP }
       FSize : LongInt;  { Size of file }
       Ver   : LongInt;  { BMP version (?), currently 0 }
       Image : LongInt;  { Offset of image into file }
       Misc  : LongInt;  { Unknown, appears to be 40 for all files }
       Width : LongInt;  { Width of image }
       Height: LongInt;  { Height of image }
       Num   : Word;     { Not sure,possibly number of images or planes (1) }
       Bits  : Word;     { Number of bits per pixel }
       Comp  : LongInt;  { Type of compression, 0 = uncompressed, 1,2 = RLE }
       ISize : LongInt;  { Size of image in bytes }
       XRes  : LongInt;  { X dots per metre }
       YRes  : LongInt;  { Y dots per metre }
       PSize : LongInt;  { Palette size (number of colours) If not zero }
       Res   : LongInt;  { Probably reserved, currently 0 }
     End;                { 54 bytes }

     TBMPPalette256 = Record { BMP uses a 4 bytes for the palette }
       B,G,R,X : Byte;       { Last byte is not used }
     End;

     TBMPPalette=Array[0..255,1..4] of Byte;

{****************************************************************************}

AImage = Class (TControl)
   Private
      fFileName         :String;        {*Path and filename*}
      fColors           :Longint;       {*Number of colors*}
      fOffsetX          :Integer;       {*Horizontal offset*}
      fOffsetY          :Integer;       {*Vertical offset*}
      fDisplay          :TDisplayPosition; {*Method to display image*}
      fLoaded           :Boolean;       {*Image loaded or no*}
      fFormat           :Integer;       {*Image format*}
      fTransparency     :Longint;       {*Transparency color*}
      fTransparent      :Boolean;       {*Transparent background*}
      fImage            :Pointer;       {*Pointer to image in memory*}

   Protected
      Property  Loaded  :Boolean Read fLoaded;
      Property  Format  :Integer Read fFormat Write fFormat;

      Procedure SetFilename(F:String);         {*Set new image filename*}
      Procedure SetOffsetX(O:Integer);         {*Set new image offset X*}
      Procedure SetOffsetY(O:Integer);         {*Set new image offset Y*}
      Procedure SetDisplay(D:TDisplayPosition);{*Set display image method*}
      Procedure SetTransparent(T:Boolean);     {*Set transparent background*}
      Procedure SetTransparency(T:Longint);    {*Set transparency color*}

   Public
      Property Transparent:Boolean       Read fTransparent Write SetTransparent;
      Property Transparency:Longint      Read fTransparency Write SetTransparency;
      Property Display :TDisplayPosition Read fDisplay  Write SetDisplay;
      Property FileName:String           Read fFileName Write SetFileName;
      Property OffsetX :Integer          Read fOffsetX  Write SetOffsetX;
      Property OffsetY :Integer          Read fOffsetY  Write SetOffsetY;
      Property Colors  :Integer          Read fColors;
      Property Image   :Pointer          Read fImage;

      Constructor Create(AOwner:TComponent); Override;          {*Create new component*}
      Constructor Create(AOwner:TComponent; pFileName:String; pLeft,pTop:Integer); {*Create new matrix image*}
      Destructor  Destroy; Override;                            {*Free Image resources*}
      Procedure   LoadFromFile;                                 {*Load using defined file*}
      Procedure   SaveToFile(FName:String);                     {*Save image in file*}
      Procedure   Draw; Override;                               {*Draw Image*}
      Function    Recognised:Boolean;                           {*Test if Image is a recognised format*}
      Function    GetFormat:String;                             {*Get the image format*}
      Procedure   DoScreenShot(pX1,pY1,pX2,pY2:Integer);        {*Get screenshot as BMP*}
   End;

{****************************************************************************}

Function  JPG_TestFile(FN:string):Boolean;         {*Is a JPG file ?*}
Function  BMP_TestFile(FN:String):Boolean;         {*Is a BMP file ?*}
Procedure BMP_GetInfo(NameFile:String;Var ScreenX,ScreenY:word;var Colors:longint); {To get information about the BMP}
Procedure BMP_Create(FileName:String;x1,y1,x2,y2:Integer);  {Capture the screen}
Procedure BMP_Show(x1,y1,x2,y2,OffsetX,OffsetY:Integer;Namefile:string;BmpFlags:Word); {Display a BMP on screen}

Procedure BMP_ShowTransparency(x1,y1,x2,y2,OffsetX,OffsetY:Integer;Namefile:string;BmpFlags:Word;TColor:Longint);

IMPLEMENTATION

Uses
     Graphix,
     GxImg,
     MosBase,
     MosColors,
     MosCPUInstructions,
     MosErrors,
     MosFiles,
     MosKeyboard,
     MosMouse,
     MosStrings;

{****************************************************************************}

Constructor AImage.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
End;

Constructor AImage.Create(AOwner:TComponent; pFileName:String; pLeft,pTop:Integer);
Begin
Inherited Create(AOwner, pLeft,pTop,pLeft,pTop);

fFileName        := pFileName;
fTransparency    := CLTransparency;
fTransparent     := No;
fLoaded          := No;                 {*Image currently not loaded*}
fImage           := Nil;
fDisplay         := DIClient;

LoadFromFile;                           {*Load image from file*}
End;

Destructor AImage.Destroy;
Begin
DestroyImage(fImage);
Inherited Destroy;
End;

Function AImage.GetFormat:String;
Begin
Result := '';
{METTRE ICI LE CODE QUI RENVOIS LE TYPE DE L'IMAGE. Ex: 'BMP', 'JPG'...}
End;

Function AImage.Recognised:Boolean;
Begin
Result := BMP_TestFile(FileName) Or JPG_TestFile(FileName);
End;

Procedure AImage.Draw;
Var VP               :TViewPortType;
    SX,SY            :Word;
    Dx,Dy            :Integer;
    Xd,Yd            :Integer;
    fX1,fX2,fY1,fY2  :LongInt;          {*Image coordinates*}

Begin
If Not Visible Then Exit;
Inherited Draw;

VESA_GetViewSettings(VP);               {*Save ViewPort*}
Mse_Hide;

SX := GetImageWidth (fImage);
SY := GetImageHeight(fImage);

Case fDisplay of
  DIStretch : ;     {*Stretched image*}
  DIMosaic  : ;     {*Mosaic images*}
  DICenter  :       {*Centered image*}
    Begin

    Dx:=(VESA_GetMaxX-Sx) div 2;
    Dy:=(VESA_GetMaxY-Sy) div 2;

    VESA_SetViewPort(Dx,Dy,VESA_GetMaxX-Dx,VESA_GetMaxY-Dy,True);
    VESA_SetFillStyle(1,CLBackground);       {*Background Color*}
    VESA_Bar(0,0,VESA_GetMaxX,VESA_GetMaxY); {*Fill Background*}

    If fTransparent Then VESA_PutImageT(Dx,Dy,fImage^,NormalPut,fTransparency)
        Else PutImage(Dx,Dy,fImage);
    End;
  DIClient  :       {*Normal (Client) image*}
    Begin
    fX1:=X1;
    fY1:=Y1;
    fX2:=X2;
    fY2:=Y2;
    If BTST(Id_Scr1,3) Then
      Begin
      Xd:=T_Clip.X1;
      Yd:=T_Clip.Y1;                    {*Set Relative Coordinates*}

      fX1:=X1+Xd;
      fX2:=X2+Xd;

      fY1:=Y1+Yd;
      fY2:=Y2+Yd;
      If fX2>T_Clip.X2 Then fX2:=T_Clip.X2;
      If fY2>T_Clip.Y2 Then fY2:=T_Clip.Y2;

      If fX1<T_Clip.X1 Then fX1:=T_Clip.X1;
      If fY1<T_Clip.Y1 Then fY1:=T_Clip.Y1;

      End;

    VESA_SetViewPort(fx1,fy1,fx2,fy2,True);
    If fTransparent Then
      VESA_PutImageT(fx1-OffsetX,fy1-OffsetY,fImage^,NormalPut,fTransparency)
    Else
      VESA_PutImage(fx1-OffsetX,fy1-OffsetY,fImage^,NormalPut);
    End;

End; {Case}

Mse_Show;

VESA_SetViewPort(VP.X1,VP.Y1,VP.X2,VP.Y2,VP.Clip); {*Restore ViewPort*}
End;

Procedure AImage.LoadFromFile;
Begin
If Not File_TestExist(fFileName) Then
  Begin
  fFileName := Sys.GetPath(SP_BUTTONS)+Sys.ImgError;
  End;

If Not (BMP_TestFile(fFileName) or JPG_TestFile(fFileName)) Then
  Begin
  Error(710);                           {*Error Not a BMP*}
  fLoaded := No;
  Exit;
  End;

LoadImageFile(itdetect,fFileName,fImage,1);
fLoaded := Yes;
End;

Procedure AImage.SaveToFile(FName:String);
Begin
{Met ton code de sauvegarde ici}
FName := FName; // To avoid warnings !
End;

Procedure AImage.DoScreenShot(pX1,pY1,pX2,pY2:Integer);
Begin
BMP_Create(FileName, pX1,pY1,pX2,pY2);
{A arranger pour sauvegarder dans System\Snaphots}
End;

Procedure AImage.SetFilename(F:String);
Begin
If fFilename = F Then Exit;             {*Identical filename*}
fFileName := F;
LoadFromFile;
If Displayed Then Draw;
End;

Procedure AImage.SetOffsetX(O:Integer);
Begin
If fOffsetX = O Then Exit;              {*Identical offset*}
fOffsetX := O;
If Displayed Then Draw;
End;

Procedure AImage.SetOffsetY(O:Integer);
Begin
If fOffsetY = O Then Exit;              {*Identical offset*}
fOffsetY := O;
If Displayed Then Draw;
End;

Procedure AImage.SetDisplay(D:TDisplayPosition);
Begin
If fDisplay = D Then Exit;              {*Display method not changed*}
fDisplay := D;
If Displayed Then Draw;
End;

Procedure AImage.SetTransparent(T:Boolean);
Begin
If T = fTransparent Then Exit;
fTransparent := T;
If Displayed Then Draw;
End;

Procedure AImage.SetTransparency(T:Longint);
Begin
If T = fTransparency Then Exit;
fTransparency := T;
If Displayed Then Draw;
End;

{****************************************************************************}

{ͻ
                           FUNCTION BMP_TestFile                            
 ͹
  IN.....: FN : (STRING) = Path + Name of the file to test                  
  OUT....: (BOOLEAN) = TRUE if file is a BMP else return FALSE.             
  EXPLAIN: Is the file a BMP ??                                             
           If file not found, return FALSE.                                 
 ͼ}

Function BMP_TestFile(FN:string):Boolean;
Begin
 BMP_TestFile:=File_TestHeader(FN,'BM',0);
End;

{ͻ
                           FUNCTION JPG_TestFile                            
 ͹
  IN.....: FN : (STRING) = Path + Name of the file to test                  
  OUT....: (BOOLEAN) = TRUE if file is a JPG else return FALSE.             
  EXPLAIN: Is the file a JPG ??                                             
           If file not found, return FALSE.                                 
 ͼ}

Function JPG_TestFile(FN:string):Boolean;
Begin
 JPG_TestFile:=File_TestHeader(FN,#$FF#$D8#$FF#$E0#$00#$10+'JFIF',0);
End;

{ͻ
                           PROCEDURE BMP_GetInfo                            
 ͹
  IN.....: NameFile : (STRING) = Path + Name of the file to test            
  OUT....: ScreenX : (WORD) = Horizontal Size in pixel                      
           ScreenY : (WORD) = Vertical Size in pixel                        
           Colors : (LONGINT) = Number of colors in bmp file.               
  EXPLAIN: Get information about a bmp file.                                
           If file is not a BMP file, the 3 parameters = 0.                 
 ͼ}

Procedure BMP_GetInfo(NameFile:string;Var ScreenX,ScreenY:word;var Colors:longint);

Procedure notbmp;
Begin
 ScreenX:=0;
 ScreenY:=0;
 Colors:=0;
End;

Var  F : File;
     G : TBMPHeader;
     E : Word;
Begin
 If BMP_TestFile(Namefile) then
   Begin
   Assign(F,Namefile);
   {$i-}
   Reset(F, 1);
   {$i+}
   If ioresult=0 then
     Begin
     {$i-}
     Blockread(F,G,SizeOf(G),E);
     {$i+}
     Close(F);
     If (ioresult<>0) or (e<>Sizeof(g)) then
       Begin
       notbmp;
       exit;
       End;
     With G DO
       Begin
       If UPPER(ID)='BM' then
         Begin
         ScreenX:=width;
         ScreenY:=height;
         If PSize=0 Then
           Case Bits Of
           1 : colors:=2;         { These are the only valid bits in a BMP }
           4 : colors:=16;
           8 : colors:=256;
           24: colors:=16777216;  { A 24 bit image does not have a palette }
           End;
         End
       Else notbmp;
     End;
     End
   Else notbmp;
   End
 Else notbmp;
End;

{ͻ
                            PROCEDURE BMP_Create                            
 ͹
  IN.....: FileName : (STRING) = Path + Name of a file to save.             
           X1,Y1,X2,Y2 : (INTEGER) = Box where to get pixel to save.        
  OUT....: /                                                                
  EXPLAIN: Create or overwrite a BMP file.Make a "snapshot" of the screen.  
           This Procedure save a screen or a part of screen to a file.      
  BUGS...: Parameters X1,Y1,X2,Y2 must be correct.                          
  EXAMPLE: BMP_Create('Image1.bmp',0,0,VESA_GetMaxX,VESA_GetMaxY)           
 ͼ}

Procedure BMP_Create(FileName:String;x1,y1,x2,y2:Integer);
Var p:pointer;
Begin
Write(^g);
p:=CreateImage(x1,y1,x2,y2);
CaptureImage(x1,y1,x2,y2,p);
saveimagebmp(FileName,p);
DestroyImage(p);
Write(^g);
End;

{ͻ
                             PROCEDURE BMP_Show                             
 ͹
  IN.....: X1,Y1,X2,Y2 : (INTEGER) = Box where the picture will be          
                                     displayed.                             
           OffsetX,OffsetY : (INTEGER) = Offset or shift of the picture.    
               The picture will be displayed on x1,y1, but at this place,   
               the pixel shown is (offsetx,offsety) from the bmp file.      
               This 2 parameters are use only if we are in mosaic, centered 
               or viewer mode.                                              
           Namefile : (STRING) = Path + Name of the bmp file.               
           BmpFlags : (WORD) =                                              
        Bit  Num                                                            
     *   0   0001  Viewer mode.                                             
     *   1   0002  Make a wait at the end                                   
         2   0004  Mosaic                                                   
         3   0008  Centered                                                 
     *   4   0010  Load bmp palette of the bmp of 0..255                    
     *   5   0020  Load bmp palette of the bmp of 16..255                   
     *   6   0040  In viewer mode, don't save screen.                       
     *   7   0080  Show in a window (Mse_Hide + Win_Clip )                  
  OUT....: /                                                                
  EXPLAIN: This procedure show a picture or photo from a BMP file.          
  BUGS...: Show only 16 and 256 colors pictures.                            
           x1..y2 and BmpFlags are use only if it's 256 colors BMP files.   
  SEEALSO: BMP_TestFile, BMP_GetInfo, BMP_LoadPalette                       
 ͼ}


Procedure BMP_Show(x1,y1,x2,y2,OffsetX,OffsetY:Integer;Namefile:string;BmpFlags:Word);
Var Img:AImage;
Begin
(*
If Not File_TestExist(NameFile) Then
  Begin
  NameFile:=Sys.GetPath(SP_BUTTONS)+Sys.ImgError;
  End;

If Not BMP_TestFile(Namefile) Then
  Begin
  Error(710);                           {*Error Not a BMP*}
  Exit;
  End;
*)
Img:=AImage.Create(nil,NameFile,x1,y1);
Img.x1:=x1;
Img.y1:=y1;
Img.x2:=x2;
Img.y2:=y2;
Img.OffsetX:=OffsetX;
Img.OffsetY:=OffsetY;

If BTST(BmpFlags,2) Then Img.SetDisplay(DIMosaic);{*MOSAIC*}
If BTST(BmpFlags,3) Then Img.SetDisplay(DICenter);{*CENTERED*}
If BTST(BmpFlags,7) Then Img.SetDisplay(DIClient);{*NORMAL*}

Img.Draw;
If BTST(BmpFlags,1) Then Wait;             {*Wait*}
Img.Destroy;
End;
(*
Procedure BMP_Show(x1,y1,x2,y2,OffsetX,OffsetY:Integer;Namefile:string;BmpFlags:Word);
Var p:pointer;
    VP:TViewPortType;
    SX,SY:Word;
    Dx,Dy:Integer;
    Xd,Yd:Integer;

Begin
If Not File_TestExist(NameFile) Then
  Begin
  NameFile:=Sys.GetPath(SP_BUTTONS)+Sys.ImgError;
  End;

If Not BMP_TestFile(Namefile) Then
  Begin
  Error(710);                           {*Error Not a BMP*}
  Exit;
  End;

LoadImage(itBMP,NameFile,p);
VESA_GetViewSettings(VP);               {*Save ViewPort*}

Mse_Hide;

SX:=GetImageWidth(p);
SY:=GetImageHeight(p);

If BTST(BmpFlags,2) Then
  Begin                                 {*MOSAIC*}
  End
Else
If BTST(BmpFlags,3) Then
  Begin                                 {*CENTERED*}
  Dx:=(VESA_GetMaxX-Sx) div 2;
  Dy:=(VESA_GetMaxY-Sy) div 2;
  VESA_SetViewPort(Dx,Dy,VESA_GetMaxX-Dx,VESA_GetMaxY-Dy,True);
  VESA_SetFillStyle(1,CLBackground);       {*Background Color*}
  VESA_Bar(0,0,VESA_GetMaxX,VESA_GetMaxY); {*Fill Background*}
  PutImage(Dx,Dy,p);
  End
Else
  Begin                                  {*NORMAL*}
  If BTST(BmpFlags,7) Then
    Begin
    If BTST(Id_Scr1,3) Then
      Begin
      Xd:=T_Clip.X1;
      Yd:=T_Clip.Y1;                     {*Set Relative Coordinates*}

      X1:=X1+Xd;
      X2:=X2+Xd;

      Y1:=Y1+Yd;
      Y2:=Y2+Yd;
      If X2>T_Clip.X2 Then X2:=T_Clip.X2;
      If Y2>T_Clip.Y2 Then Y2:=T_Clip.Y2;

      If X1<T_Clip.X1 Then X1:=T_Clip.X1;
      If Y1<T_Clip.Y1 Then Y1:=T_Clip.Y1;
      End;

    End;
  VESA_SetViewPort(x1,y1,x2,y2,True);
  VESA_PutImage(x1-OffsetX,y1-OffsetY,p^,NormalPut);
  End;

Mse_Show;

VESA_SetViewPort(VP.X1,VP.Y1,VP.X2,VP.Y2,VP.Clip); {*Restore ViewPort*}
RemoveImage(p);
If BTST(BmpFlags,1) Then Wait;             {*Wait*}

End;
  *)
{ͻ
                       PROCEDURE BMP_ShowTransparency                       
 ͹
  IN.....: X1,Y1,X2,Y2 : (INTEGER) =        SEEALSO BMP_Show                
           OffsetX,OffsetY : (INTEGER) =    SEEALSO BMP_Show                
           Namefile : (STRING) =            SEEALSO BMP_Show                
           BmpFlags : (WORD) =              SEEALSO BMP_Show                
           TColor (LONGINT) = Transparency color.                           
  OUT....: /                                                                
  EXPLAIN: SEEALSO BMP_Show                                                 
  SEEALSO: BMP_Show                                                         
 ͼ}
Procedure BMP_ShowTransparency(x1,y1,x2,y2,OffsetX,OffsetY:Integer;Namefile:string;BmpFlags:Word;TColor:Longint);
Var Img:AImage;
Begin
(*
If Not File_TestExist(NameFile) Then
  Begin
  NameFile:=Sys.GetPath(SP_BUTTONS)+Sys.ImgError;
  End;

If Not BMP_TestFile(Namefile) Then
  Begin
  Error(710);                           {*Error Not a BMP*}
  Exit;
  End;
*)
Img := AImage.Create(NIL,NameFile,x1,y1);
Img.x1:=x1;
Img.y1:=y1;
Img.x2:=x2;
Img.y2:=y2;
Img.OffsetX:=OffsetX;
Img.OffsetY:=OffsetY;
Img.SetTransparent(True);
Img.SetTransparency(TColor);

If BTST(BmpFlags,2) Then Img.SetDisplay(DIMosaic);{*MOSAIC*}
If BTST(BmpFlags,3) Then Img.SetDisplay(DICenter);{*CENTERED*}
If BTST(BmpFlags,7) Then Img.SetDisplay(DIClient);{*NORMAL*}

Img.Draw;
If BTST(BmpFlags,1) Then Wait;             {*Wait*}
Img.Destroy;
End;

(*Procedure BMP_ShowTransparency(x1,y1,x2,y2,OffsetX,OffsetY:Integer;Namefile:string;BmpFlags:Word;TColor:Longint);
Var p:pointer;
    VP:TViewPortType;
    SX,SY:Word;
    Dx,Dy:Integer;
    Xd,Yd:Integer;

Begin
If Not File_TestExist(NameFile) Then
  Begin
  NameFile:=Sys.GetPath(SP_BUTTONS)+Sys.ImgError;
  End;

If Not BMP_TestFile(Namefile) Then
  Begin
  Error(710);                           {*Error Not a BMP*}
  Exit;
  End;

LoadImage(itBMP,NameFile,p);
VESA_GetViewSettings(VP);               {*Save ViewPort*}

Mse_Hide;

SX:=GetImageWidth(p);
SY:=GetImageHeight(p);

If BTST(BmpFlags,2) Then
  Begin                                 {*MOSAIC*}
  End
Else
If BTST(BmpFlags,3) Then
  Begin                                 {*CENTERED*}
  Dx:=(VESA_GetMaxX-Sx) div 2;
  Dy:=(VESA_GetMaxY-Sy) div 2;
  VESA_SetViewPort(Dx,Dy,VESA_GetMaxX-Dx,VESA_GetMaxY-Dy,True);
  VESA_SetFillStyle(1,CLBackground);       {*Background Color*}
  VESA_Bar(0,0,VESA_GetMaxX,VESA_GetMaxY); {*Fill Background*}
  VESA_PutImageT(Dx,Dy,p^,NormalPut,TColor);
  End
Else
  Begin                                 {*NORMAL*}
  If BTST(BmpFlags,7) Then
    Begin
//    If BTST(Id_Scr1,3) Then
      Begin
      Xd:=T_Clip.X1;
      Yd:=T_Clip.Y1;                    {*Set relative coordinates*}

      X1:=X1+Xd;
      X2:=X2+Xd;

      Y1:=Y1+Yd;
      Y2:=Y2+Yd;
      If X2>T_Clip.X2 then X2:=T_Clip.X2;
      If Y2>T_Clip.Y2 then Y2:=T_Clip.Y2;

      If X1<T_Clip.X1 Then X1:=T_Clip.X1;
      If Y1<T_Clip.Y1 Then Y1:=T_Clip.Y1;
      End;

    End;
  VESA_SetViewPort(x1,y1,x2,y2,True);
  VESA_PutImageT(x1-OffsetX,y1-OffsetY,p^,NormalPut,TColor);
  End;

Mse_Show;

VESA_SetViewPort(VP.X1,VP.Y1,VP.X2,VP.Y2,VP.Clip); {*Restore ViewPort*}
RemoveImage(p);
If BTST(BmpFlags,1) Then Wait;             {*Wait*}
End;*)

Begin
End.
