{ͻ
                               UNIT MosLists                                
 ͹
  CREATOR....: CHEVEAU Frdric                                             
  UPDATED BY.: CHEVEAU Frdric                                             
  CREATED THE: 08.05.2000                                                   
  COUNTRY....: Montpellier - Timeless (FRANCE)                              
  DESCRIPTION: Visual components "List, ListBox".                           
  AUTONOMOUS.: No                                                           
  UNIT SIZE..: 3.718 Lines and (93.136 Bytes compiled)                      
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosLists;

{$Mode Objfpc}

INTERFACE

Uses
     Classes,
     MosControls,
     MosMemory;

{
TListBox
TListHeader
TListColumn
TListItem
}

Type                                    {*Types definition*}

{****************************************************************************}

{****************************************************************************}
{*** BList Class                                                          ***}
{****************************************************************************}

BList = Class(TComponent)               {*TList class definition*}
   Private
      fCapacity         :Longint;       {*List capacity (maximum of items)*}
      fCount            :Longint;       {*Count defined items*}
      fList             :TArray;        {*Pointer to the pointer array*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetCapacity(C:Longint); {*Set the list capacity*}
      Procedure SetItem(Index:Longint; Value:Pointer); {*Set a new value in item*}
      Function  GetItem(Index:Longint):Pointer;        {*Get an item value*}

   Public
      Property Capacity            :Longint  Read fCapacity  Write SetCapacity;
      Property Count               :Longint  Read fCount;
      Property List                :TArray   Read fList;
      Property Items[Index:Longint]:Pointer  Read GetItem    Write SetItem;

      Constructor Create(AOwner:TComponent); Override; {*Create a new component*}
      Destructor  Destroy; Override;              {*Destroy component*}
      Function    Add(pItem:Pointer):Longint;     {*Add an item to the list*}
      Function    Remove(pItem:Pointer):Longint;  {*Remove an item for list*}
      Function    IndexOf(pItem:Pointer):Longint; {*Get the item index*}
      Procedure   Delete(Index:Longint);          {*Delete an item in array*}
      Procedure   Insert(Index:Longint; pItem:Pointer);{*Insert new item*}
      Procedure   Move(CurIndex,NewIndex:Longint);{*Move item to new index*}
      Procedure   Exchange(Index1,Index2:Longint);{*Exchange two items*}
      Procedure   Clear;                          {*Clear all items in array*}
      Function    Expand:Boolean;                 {*Expand list capacity*}
      Function    First:Pointer;                  {*Get the first item*}
      Function    Last:Pointer;                   {*Get the last item*}
      Procedure   Pack;                           {*Pack items in list*}
      Procedure   Sort;                           {*Sort the list*}
   End;

{****************************************************************************}
{*** TListItem Class                                                      ***}
{****************************************************************************}

TListItem = Class(TCollectionItem)      {*TListItem class definition*}
   Private
      fCaption          :String;        {*Item caption*}
      fChecked          :Boolean;       {*Item checked*}
      fSeparator        :Char;          {*Cell tag separator*}

      fOnChange         :TNotifyEvent;  {*Item changed*}

   Protected
      Procedure Change; Dynamic;
      Procedure SetCaption(C:String);   {*Set a new caption*}
      Procedure SetChecked(C:Boolean);  {*Check or uncheck item*}
      Procedure SetCell(pIndex:Integer; Value:String); {*Set a value in cell*}
      Function  GetCell(pIndex:Integer):String;{*Get a cell value*}
      Procedure SetSeparator(S:Char);   {*Define the column separator*}

   Public
      Property  Cell[_Index:Integer]:String Read GetCell   Write SetCell;

      Constructor Create(pCollection:TCollection); Override;
      Destructor  Destroy; Override;

   Published
      Property  Caption     :String       Read fCaption   Write SetCaption;
      Property  OnChange    :TNotifyEvent Read fOnChange  Write fOnChange;
      Property  Separator   :Char         Read fSeparator Write SetSeparator;
   End;

{****************************************************************************}
{*** TListItems Class                                                     ***}
{****************************************************************************}

TListItems = Class(TCollection)         {*TListItems class definition*}
   Private

   Protected
      Function  GetItem(Index:Integer): TListItem;
      Procedure SetItem(Index:Integer; Value:TListItem);

   Public
      Constructor Create(AOwner:TListItem);
      Function    Add:TListItem;

      Property Items[Index:Integer]:TListItem   Read GetItem Write SetItem;

   Published
   End;

{****************************************************************************}
{*** TListBox Class                                                       ***}
{****************************************************************************}

TListBox = Class(TVisual)               {*TListBox class definition*}
   Private
   Protected
   Public
   End;

{}
{}
{}

IMPLEMENTATION

Uses
     MosBase,
     MosConfig,
     MosCPUInstructions,
     MosStrings;

{****************************************************************************}
{*** BList Class                                                          ***}
{****************************************************************************}

Procedure BList.Define;
Begin
fCapacity := 50;
fCount    := 0;

If Owner = Self Then Log.Add('Create new TList ('+')',2);
End;

Constructor BList.Create(AOwner:TComponent);
Var OneItem          :Pointer;
Begin
Inherited Create(AOwner);
OneItem := NIL;
Define;
Array_Init(fList, 0, fCapacity-1, OneItem, Sizeof(OneItem));
End;

Destructor BList.Destroy;
Begin
Array_Done(fList);
Inherited Destroy;
End;

Function BList.Add(pItem:Pointer):Longint;
Begin
Inc(fCount);
If fCount > fCapacity - 1 Then Expand;

Array_Put(fList, fCount, pItem);        {*Warning ! Index shift between TArray and TList*}
Result := fCount - 1;
End;

Procedure BList.Delete(Index:Longint);
Var Loop             :Longint;
    OneItem          :Pointer;
Begin
If fCount <= 0 Then Exit;               {*No items in list*}
If (Index < 0) Or (Index > fCount - 1) Then Exit; {*Out of range*}
For Loop := Index + 1 to fCount - 1 do
   Begin
   Array_Get(fList, Loop + 1, OneItem); {*Move items values*}
   Array_Put(fList, Loop + 1 - 1, OneItem);
   End;

OneItem := NIL;
SetItem(fCount - 1, OneItem);           {*Clear last value*}
Dec(fCount);                            {*Decrease items count*}
End;

Procedure BList.Insert(Index:Longint; pItem:Pointer);
Var Loop             :Longint;
    OneItem          :Pointer;
Begin
If Index < 0 Then Exit;                 {*Out of range*}
If Index > fCapacity - 1 Then SetCapacity(Index);
For Loop := fCount - 1 DownTo Index do
   Begin
   Array_Get(fList, Loop + 1, OneItem); {*Move items values*}
   Array_Put(fList, Loop + 1 + 1, OneItem);
   End;

If Index <= fCount Then Inc(fCount);    {*Increase items count*}
SetItem(Index, pItem);                  {*Add new item*}
End;

Function BList.Remove(pItem:Pointer):Longint;
Var Index            :Longint;
Begin
Index := IndexOf(pItem);
If Index >=0 Then Delete(Index);
Result := Index;
End;

Procedure BList.Move(CurIndex,NewIndex:Longint);
Var OneItem          :Pointer;
Begin
If (CurIndex < 0) Or (NewIndex < 0) Or (CurIndex = NewIndex) Then Exit;
OneItem := Items[CurIndex];
Delete(CurIndex);
Insert(NewIndex, OneItem);
End;

Procedure BList.Clear;
Var Loop             :Longint;
    OneItem          :Pointer;

Begin
OneItem := NIL;
For Loop := 0 to fCount - 1 do
   Begin
   Array_Put(fList, Loop + 1, OneItem);
   End;
fCount := 0;
End;

Procedure BList.Exchange(Index1,Index2:Longint);
Var Item1,Item2      :Pointer;          {*To store temporary items*}
Begin
If (Index1 < 0) Or (Index2 < 0) Then Exit; {*Out of ranges*}
If (Index1 > fCapacity - 1) Or (Index2 > fCapacity - 1) Then SetCapacity(Max(Index1,Index2));

Array_Get(fList, Index1 + 1, Item1);    {*Get items values*}
Array_Get(fList, Index2 + 1, Item2);

SetItem(Index1, Item2);                 {*Swap items values*}
SetItem(Index2, Item1);
End;

Function BList.Expand:Boolean;
Begin
SetCapacity(fCapacity * 2);             {*Doble capacity*}
Result := Yes;                          {!}
End;

Function BList.First:Pointer;
Begin
Result := Items[0];
End;

Function BList.Last:Pointer;
Begin
Result := Items[fCount - 1];
End;

Function BList.IndexOf(pItem:Pointer):Longint;
Var Loop             :Longint;          {*For main loop*}
    OneItem          :Pointer;          {*To store temporary item*}
Begin
For Loop := 0 to fCount - 1  do
   Begin
   Array_Get(fList, Loop + 1, OneItem); {*Get item value*}
   If pItem = OneItem Then
      Begin
      Result := Loop;                   {*Return index*}
      Exit;
      End;
   End;
Result := -1;                           {*Item not found*}
End;

Procedure BList.Pack;
Var BuildIndex       :Longint;          {*Index to store new packed values*}
    Loop             :Longint;          {*For main loop*}
    PlaceFound       :Boolean;          {*Search free place one time only*}
    OneItem          :Pointer;          {*To store temporary item*}

Begin
BuildIndex := -1;                       {*Initialise*}
PlaceFound := No;

For Loop := 0 to fCount - 1  do
   Begin
   Array_Get(fList, Loop + 1, OneItem); {*Get item value*}
   If (OneItem = NIL) And (Not PlaceFound) Then
      Begin                             {*Found free place*}
      BuildIndex := Loop;
      PlaceFound := Yes;
      End;
   If (OneItem <> NIL) And (BuildIndex <> Loop) And (BuildIndex <> -1) Then
      Begin                             {*Move item to free place*}
      Array_Put(fList, BuildIndex + 1, OneItem);
      Inc(BuildIndex);
      Inc(PlaceFound);                  {*Next place to store*}
      End;
   End;

If PlaceFound Then                      {*Modification(s) has been done*}
   Begin
   OneItem := NIL;                      {*Clear with NIL*}
   For Loop := BuildIndex to fCount - 1 do
      Begin                             {*Clear residual values*}
      Array_Put(fList, Loop + 1, OneItem);
      End;
   fCount := BuildIndex;                {*Set new count*}
   End;
End;

Procedure BList.Sort;
Begin
End;

Procedure BList.SetCapacity(C:Longint);
Begin
If C < 1 Then Exit;
If C = fCapacity Then Exit;
fCapacity := C;
{Redimensionner Array}
End;

Procedure BList.SetItem(Index:Longint; Value:Pointer);
Begin
If Index < 0 Then Exit;
If Index > fCapacity - 1 Then Expand;

If Index > fCount Then  fCount := Index + 1;
Array_Put(fList, Index + 1, Value);
End;

Function BList.GetItem(Index:Longint):Pointer;
Var OneItem          :Pointer;
Begin
Result := NIL;
If Index < 0 Then Exit;                 {*Negative indexes not allowed*}
If Index > fCapacity - 1 Then Exit;     {*Index out of ranges*}

Array_Get(fList, Index + 1, OneItem);
Result := OneItem;                      {*Return pointer*}
End;

{****************************************************************************}
{*** TListItem Class                                                      ***}
{****************************************************************************}

Constructor TListItem.Create(pCollection:TCollection);
Begin
Inherited Create(pCollection);
End;

Destructor TListItem.Destroy;
Begin
Inherited Destroy;
End;

Procedure TListItem.SetCaption(C:String);
Begin
If C = fCaption Then Exit;
fCaption := C;
Change;
End;

Procedure TListItem.SetChecked(C:Boolean);
Begin
If C = fChecked Then Exit;
fChecked := C;
Change;
End;

Procedure TListItem.SetSeparator(S:Char);
Begin
If S = fSeparator Then Exit;
fSeparator := S;
Change;
End;

Procedure TListItem.SetCell(pIndex:Integer; Value:String);
Begin
Str_Put(fCaption, Value, pIndex - 1, fSeparator);
Change;
End;

Function TListItem.GetCell(pIndex:Integer):String;
Begin
Result := Str_Get(fCaption, pIndex - 1, fSeparator);
End;

Procedure TListItem.Change;
Begin
If Assigned(fOnChange) Then fOnChange(Self);
End;

{****************************************************************************}
{*** TListItems Class                                                     ***}
{****************************************************************************}

Constructor TListItems.Create(AOwner:TListItem);
Begin
Inherited Create(TListItem);
AOwner := AOwner; {!}
End;

Function TListItems.GetItem(Index:Integer): TListItem;
Begin
Result := TListItem(Inherited GetItem(Index));
End;

Procedure TListItems.SetItem(Index:Integer; Value:TListItem);
Begin
Inherited SetItem(Index, Value);
End;

Function TListItems.Add:TListItem;
Begin
Result := TListItem(Inherited Add);
End;

{****************************************************************************}
{*** TListBox Class                                                       ***}
{****************************************************************************}

{******************************* NO MAIN CODE *******************************}

Begin
End.
