{ͻ
                               UNIT MosMemory                               
 ͹
  CREATOR....: PLANAS Jean-Pierre                                           
  UPDATED BY.: PLANAS Jean-Pierre                                           
  CREATED THE: 22.12.1998                                                   
  COUNTRY....: Perpignan                                                    
  DESCRIPTION: Memory management. (containers)                              
  AUTONOMOUS.: Yes                                                          
  UNIT SIZE..: 374 Lines and (5.168 Bytes compiled)                         
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosMemory;

{$MODE Objfpc}

INTERFACE

Type TArray = Record              {*Dynamic array record*}
       p_Array      : Pointer;    {*Pointer on the array*}
       Size         : Longint;    {*Size of the array*}
       Min,Max      : Longint;    {*Min and Max index*}
     End;

     TStack = Record              {*Stack record*}
       p_Stack      : Pointer;    {*@ of the current item in stack*}
       ItemSize     : LongInt;    {*Size of 1 item*}
       Qtt_Items    : LongInt;    {*Number of items*}
       Qtt_MaxItems : LongInt;    {*Max number of items*}
       Size         : LongInt;    {*Size in byte of the stack*}
     End;

     TVector = Record             {*Dynamic array resizable*}
       p_Vector     : Pointer;    {*Pointer on the array*}
       Size         : Longint;    {*Size of 1 item*}
       Last         : Longint;    {*Last item of the array*}
       GranPage     : Longint;    {*Qtt of item by pages*}
       QttPage      : LongInt;    {*Qtt of pages allocated*}
     End;

     TMemText = Record            {*Memory Text File*}
       Vector       : TVector;    {*Data*}
       Current_Line : LongInt;    {*Current line to read*}
       Qtt_Lines    : Longint;    {*Qtt of lines in the array*}
     End;


{|Dynamic Array|}
Function  Array_Init(Var A:TArray;Min,Max:Longint;Var v;Size:LongInt):boolean; {*Initialise dynamic array*}
Procedure Array_Done(A:TArray);                    {*Free memory allocated memory by the array*}
Procedure Array_Get(A:TArray;index:longint;Var v); {*Get an item from the array*}
Procedure Array_Put(A:TArray;index:longint;Var v); {*Put an item on the array*}

{|Vector|}
Function  Vector_Init(Var A:TVector;GranPage:Longint;Size:LongInt):boolean; {*Init dynamic Vector*}
Procedure Vector_Kill(Var A:TVector);                     {*Kill the vector*}
Procedure Vector_Get(Const A:TVector;index:longint;Var v);{*Get one item of the vector*}
Procedure Vector_Put(Var A:TVector;index:longint;Var v);  {*Put one item in the vector*}
Procedure Vector_Remove(Var A:TVector;index:longint);     {*Remove one item of the vector*}

{|Memory Text File|}
Procedure MemText_Init(Var V:TMemText);                 {*Init Text Memory File*}
Procedure MemText_Done(Var V:TMemText);                 {*Kill MemText*}
Procedure MemText_Writeln(Var V:TMemText;St:String);    {*Writeln a string at the end*}
Procedure MemText_Readln(Var V:TMemText;Var St:String); {*Readln a string*}
Procedure MemText_Seek(Var V:TMemText;Pos:LongInt);     {*Set current position*}
Function  MemText_Eof(Var V:TMemText):Boolean;          {*Test end of MemFile*}
Function  MemText_GetLines(Var V:TMemText):LongInt;     {*Get how many lines*}
Procedure MemText_Load(Var V:TMemText;FileName:String); {*Load a text file*}
Procedure MemText_Save(Var V:TMemText;FileName:String); {*Save a text file*}

{|Stack|}
Function  Stack_Init(Var Stack:TStack;ItemSize:LongInt;QttMax:LongInt):Boolean; {*Initialize the stack*}
Procedure Stack_Done(Var Stack:TStack);               {*Kill the stack*}
Function  Stack_Push(Var Stack:TStack;Var V):Boolean; {*Put an item on the stack*}
Function  Stack_Pop(Var Stack:TStack;Var V):Boolean;  {*Get an item from the stack*}
Function  Stack_TestEmpty(Stack:TStack):Boolean;      {*Test if the stack is empty*}
Function  Stack_TestFull (Stack:TStack):Boolean;      {*Test if the stack is full*}
Procedure Stack_Read(Stack:TStack;Num:LongInt;Var V); {*Read an item from the stack*}

{|CRC|}
Function Mem_CheckSum(Var data; size : LongInt) : LongInt; {*Get a CRC*}

{|Memory Blocks|}
Function Mem_CompBlock(Var P1,P2;Size:LongInt):Boolean;

{|Miscelaneous|}
Procedure Mem_Gap(Var P1,P2;Size:LongInt;B:Byte);
Procedure Mem_ReAlloc(Var P:Pointer; NewSize:LongInt); {*Change size of the allocated memory*}
Function MaxAvail: Longint;
Function MemAvail: Longint;

IMPLEMENTATION

Uses
  MosCPUInstructions
  {$ifdef GO32V2}
  ,go32
  {$endif}
  {$ifdef MSWINDOWS}
  ,windows
  {$endif}
  {$ifdef linux}
  ,linux
  {$endif}
  ;

function maxavail: Longint;
var info: {$ifdef GO32V2}tmeminfo{$endif}{$ifdef MSWINDOWS}TMemoryStatus{$endif}{$ifdef linux}TSysInfo{$endif};
begin
  {$ifdef GO32V2}
  get_meminfo(info);
  maxavail := info.available_memory;
  {$endif}
  {$ifdef MSWINDOWS}
  GlobalMemoryStatus(info);
  maxavail := info.dwAvailphys;
  {$endif}
  {$ifdef linux}
  if SysInfo(Info) then maxavail := Info.freeram else maxavail := 0;
  {$endif}
end;

Function MemAvail: Longint;
begin
  MemAvail := MaxAvail;
end;

{*-------------------------------- Dynamic Array ---------------------------*}

{ͻ
                            FUNCTION Array_Init                             
 ͹
  IN.....: Min (LONGINT) = Minimun index of the array.                      
           Max (LONGINT) = Maximun index of the array.                      
           V (UNTYPED) = The variable of the same type of the array         
                         witch contain the initilizated value of the array  
           Size (WORD) = Always = Sizeof(V).                                
  OUT....: A (TArray) = The array. (Put the name of the array in)           
           (BOOLEAN) = TRUE if array is created, else FALSE.                
  EXPLAIN: Create a dynamic array.                                          
  SEEALSO: Array_Done, Array_Get, Array_Put                                 
 ͼ}

Function Array_Init(Var A:TArray;Min,Max:Longint;Var v;Size:LongInt):boolean;
Var i:longint;
Begin

If (Set_Even(Size)*(max-min+1))>=maxavail Then {*Test if enought memory*}
  Begin
  A.p_Array:=Nil;
  Array_Init:=false;
  Exit;
  End;

GetMem(A.p_array,Set_Even(Size)*(max-min+1));  {*Get memory*}

A.Min:=Min;
A.Max:=Max;
A.Size:=Size;

For i:=Min to Max do        {*Init all the array with the same value*}
  Begin
  Array_Put(A,i,v)
  End;
Array_Init:=true;
End;

{ͻ
                            PROCEDURE Array_Done                            
 ͹
  IN.....: A (TArray) = Name of the array                                   
  OUT....: /                                                                
  EXPLAIN: Kill the array (Free the memory allocated by Array_Init)         
           Don't forget to call this procedure because If you don't call    
           in the program, this memory will be lost.                        
  SEEALSO: Array_Init                                                       
 ͼ}

Procedure Array_Done(A:TArray);
Begin
If A.p_array=Nil then exit;                          {*Error*}
FreeMem(A.p_array,Set_Even(A.Size)*(A.Max-A.Min+1)); {*Free memory allocated*}
A.p_array:=Nil;
End;

{ͻ
                            PROCEDURE Array_Get                             
 ͹
  IN.....: A (TArray) = Name of the array                                   
           Index (LONGINT) = Is the index in the array.                     
  OUT....: V (UNTYPED) = get the variable from the array                    
  EXPLAIN: Get the item at Index Position in the array.                     
  SEEALSO: Array_Init, Array_Put                                            
 ͼ}

Procedure Array_Get(A:TArray;index:longint;Var v);
Var p:pointer;
Begin
p:=A.p_array;
incp(p,(index-A.Min)*A.Size);
move(p^,v,A.Size);
End;

{ͻ
                            PROCEDURE Array_Put                             
 ͹
  IN.....: A (TArray) = Name of the array                                   
           Index (LONGINT) = Index in the array.                            
           V (UNTYPED) = put the variable in the array.                     
  OUT....: /                                                                
  EXPLAIN: Set an item in the array at Index position.                      
  SEEALSO: Array_Init, Array_Get                                            
 ͼ}

Procedure Array_Put(A:TArray;index:longint;Var v);
Var p:pointer;
Begin
p:=A.p_array;
incp(p,(index-A.Min)*A.Size);
move(v,p^,A.Size);
End;

{*-------------------------------- Vectors ---------------------------------*}

{ͻ
                            Function Vector_Init                            
 ͹
  IN.....: A (TVector) = Record witch contains all infos on the vector.     
           GranPage (LongInt) = Set quantity of items by pages.             
           Size (LongInt) = Size of an item.                                
  OUT....: A (TVector) = Record set.                                        
           (Boolean) = TRUE if OK, else FALSE if an error occurs.           
  EXPLAIN: Create a Vector. A vector is a dynamic array. It resize itself   
           when you put new items in the array.                             
           When a vector allocate memory, it allocate always a Page.        
           A page contains "GranPage" items...                              
  EXAMPLE: Var V:TVector;                                                   
           Vector_Init(V,256,Sizeof(Real))                                  
           This create a new dynamic array of Real.                         
  NB.....: The array begins at 0.                                           
  SEEALSO: Vector_Kill, Vector_Get, Vector_Put, Vector_Remove               
 ͼ}
Function  Vector_Init(Var A:TVector;GranPage:Longint;Size:LongInt):boolean;
Var P:Pointer;
    I:LongInt;
Begin
If (Size+1)>=maxavail Then {*Test if enought memory*}
  Begin
  A.p_Vector:=Nil;
  Vector_Init:=False;
  {!!}
  Exit;
  End;

A.Size:=Size+1;
A.Last:=0;
A.QttPage:=1;
A.GranPage:=GranPage;

GetMem(A.p_Vector,A.Size*A.GranPage*A.QttPage);  {*Get memory*}

P:=A.p_Vector;

For i:=0 to A.GranPage-1 do
  Begin
  Poke(P,0);
  IncP(P,A.Size);
  End;

Vector_Init:=True;
End;

{ͻ
                           Procedure Vector_Kill                            
 ͹
  IN.....: A (TVector) = Record witch contains all infos on the vector      
  OUT....: /                                                                
  EXPLAIN: Remove from memory the vector                                    
  SEEALSO: Vector_Init                                                      
 ͼ}
Procedure Vector_Kill(Var A:TVector);
Begin
If A.p_Vector=Nil Then Exit;  {!!}                  {*Error*}
FreeMem(A.p_Vector,A.Size*A.GranPage*A.QttPage);    {*Free memory allocated*}
A.p_Vector:=Nil;
End;

{ͻ
                            Procedure Vector_Get                            
 ͹
  IN.....: A (TVector) = Record witch contains all infos on the vector      
           Index (LongInt) = Index in the vector of the item to get         
           V (UNTYPED) = Variable witch will contain the data               
  OUT....: /                                                                
  EXPLAIN: Get the item at Index position                                   
  SEEALSO: Vector_Init                                                      
 ͼ}
Procedure Vector_Get(Const A:TVector;index:longint;Var v);
Var P:Pointer;
Begin
If Index>A.Last Then Exit;{!!}
P:=A.p_Vector;
IncP(P,A.Size*Index);
If Peek(P)=0 Then Exit;   {!!}
IncP(P,1);
Move(P^,v,A.Size-1);
End;

{ͻ
                            Procedure Vector_Put                            
 ͹
  IN.....: A (TVector) = Record witch contains all infos on the vector      
           Index (LongInt) = Index in the vector of the item to put         
  OUT....: V (UNTYPED) = Variable witch contains the data                   
  EXPLAIN: Set an item at index position                                    
  SEEALSO: Vector_Init                                                      
 ͼ}
Procedure Vector_Put(Var A:TVector;index:longint;Var v);
Var P:Pointer;
    NumPage:Longint;
    I:LongInt;
Begin
{*Test if need to allocate memory*}
NumPage:=(index div A.GranPage);
If NumPage>A.QttPage Then
  Begin
{  Mem_Realloc(A.P_Vector, A.Size*A.GranPage*A.QttPage,
                          A.Size*A.GranPage*NumPage);}
  Mem_Realloc(A.P_Vector, A.Size*A.GranPage*NumPage);
  A.QttPage:=NumPage;

  P:=A.P_Vector;
  IncP(P,A.Last*A.Size);
  For i:=A.Last to Index do
    Begin
    Poke(P,0);
    IncP(P,A.Size);
    End;

  End;

P:=A.p_Vector;
IncP(P,Index*A.Size);
Poke(p,1);
IncP(p,1);
Move(v,p^,A.Size-1);
If A.Last<Index Then A.Last:=Index;
End;

{ͻ
                          Procedure Vector_Remove                           
 ͹
  IN.....: A (TVector) = Record witch contains all infos on the vector      
           Index (LongInt) = Index of the item to remove                    
  OUT....: A (TVector) = Record set                                         
  EXPLAIN: Remove an item and free memory if it's possible...               
  BUGS...: Not finished                                                     
  SEEALSO: Vector_Init                                                      
 ͼ}
Procedure Vector_Remove(Var A:TVector;index:longint);
Var P:Pointer;
Begin
P:=A.p_Vector;
IncP(P,Index*A.Size);
Poke(p,0);
{!!}
{*Search if allocation can be reduced*}
End;

{*------------------------------- MemText ----------------------------------*}

{ͻ
                           Procedure MemText_Init                           
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
  OUT....: /                                                                
  EXPLAIN: Create a text memory file.                                       
  SEEALSO: MemText_Done, MemText_Writeln, MemText_Readln, MemText_Seek,     
           MemText_Eof, MemText_GetLines, MemText_Load, MemText_Save        
 ͼ}
Procedure MemText_Init(Var V:TMemText);
Begin

V.Qtt_Lines:=0;
V.Current_Line:=0;
Vector_Init(V.Vector,32,Sizeof(String));  {* 32*256 = 2*4Ko *}
End;

{ͻ
                           Procedure MemText_Done                           
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
  OUT....: /                                                                
  EXPLAIN: Remove MemText from memory                                       
  SEEALSO: MemText_Init                                                     
 ͼ}
Procedure MemText_Done(Var V:TMemText);
Begin
Vector_Kill(V.Vector);
V.Qtt_Lines:=0;
V.Current_Line:=0;
End;

{ͻ
                         Procedure MemText_Writeln                          
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
           St (String) = String to write at the end of the text memory file.
  OUT....: /                                                                
  EXPLAIN: Set current line to the end and writeln the string in the file.  
  SEEALSO: MemText_Init                                                     
 ͼ}
Procedure MemText_Writeln(Var V:TMemText;St:String);
Begin
Vector_Put(V.Vector,V.Qtt_Lines,St);
V.Qtt_Lines:=V.Qtt_Lines+1;
V.Current_Line:=V.Qtt_Lines;
End;

{ͻ
                          Procedure MemText_Readln                          
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
  OUT....: St (String) = String result from the read of MemText             
  EXPLAIN: Readln a string in the MemText at the current line.              
  SEEALSO: MemText_Init                                                     
 ͼ}
Procedure MemText_Readln(Var V:TMemText;Var St:String);
Begin
Vector_Get(V.Vector,V.Current_Line,St);
V.Current_Line:=V.Current_Line+1;
End;

{ͻ
                           Procedure MemText_Seek                           
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
           Pos (LongInt) = Line number                                      
  OUT....: /                                                                
  EXPLAIN: Set the current line in the file to Pos.                         
  SEEALSO: MemText_Init                                                     
 ͼ}
Procedure MemText_Seek(Var V:TMemText;Pos:LongInt);
Begin

V.Current_Line:=Pos;
End;

{ͻ
                           Procedure MemText_Eof                            
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
  OUT....: (Boolean) = Test if current line is at the end of the MemText.   
  EXPLAIN: Test if the end of MemText is reached.                           
  SEEALSO: MemText_Init                                                     
 ͼ}
Function  MemText_Eof(Var V:TMemText):Boolean;
Begin
MemText_Eof:=(V.Current_Line>=V.Qtt_Lines);
End;

{ͻ
                         Procedure MemText_GetLines                         
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
  OUT....: (LongInt) = Quantity of lines in MemText                         
  EXPLAIN: Get how many lines in MemText file.                              
  SEEALSO: MemText_Init                                                     
 ͼ}
Function  MemText_GetLines(Var V:TMemText):LongInt;
Begin
MemText_GetLines:=V.Qtt_Lines;
End;

{ͻ
                           Procedure MemText_Load                           
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
           FileName (String) = Path+ File name                              
  OUT....: /                                                                
  EXPLAIN: Load a text file in memory.                                      
  SEEALSO: MemText_Init                                                     
 ͼ}
Procedure MemText_Load(Var V:TMemText;FileName:String);
Var S:String;
    F:Text;
Begin
Assign(F,FileName);
ReSet(F);
MemText_Seek(V,0);
While (Not Eof(F)) do
  Begin
  Readln(F,S);
  MemText_Writeln(V,S);
  End;
Close(F);
End;

{ͻ

                           Procedure MemText_Save                           
 ͹
  IN.....: V (TMemText) = Record on the memory text file.                   
           FileName (String) = Path+ File name                              
  OUT....: /                                                                
  EXPLAIN: Save a MemText to a file.                                        
  SEEALSO: MemText_Init                                                     
 ͼ}
Procedure MemText_Save(Var V:TMemText;FileName:String);
Var S:String;
    F:Text;
Begin
Assign(F,FileName);
ReWrite(F);
MemText_Seek(V,0);
While (Not MemText_Eof(V)) do
  Begin
  MemText_Readln(V,S);
  Writeln(F,S);
  End;
Close(F);
End;

{*--------------------------------- Stack ----------------------------------*}

{ͻ
                            FUNCTION Stack_Init                             
 ͹
  IN.....: Stack (TSTACK) = Stack Structure                                 
           ItemSize (LONGINT) = Size of an Item                             
           QttMax (LONGINT) = Maximum quantity of items                     
  OUT....: (BOOLEAN) = TRUE if all is OK, FALSE if not enougth memory       
  EXPLAIN: Create a stack of items. You fixe the size of the item with      
           ItemSize and the size of the stack with QttMax.                  
           Don't forget to kill the stack after use.                        
  EXAMPLE: Var S:Stack;St:String;                                           
           Stack_Init(S,Sizeof(St),20); Create a stack witch can contain    
                                        20 strings.                         
  SEEALSO: Stack_Done, Stack_Push, Stack_Pop, Stack_TestEmpty, Stack_Read   
 ͼ}
Function Stack_Init(Var Stack:TStack;ItemSize:LongInt;QttMax:LongInt):Boolean;
Var L:Longint;
Begin
Stack.ItemSize:=ItemSize;                 {*Fix stack parameters*}
Stack.Qtt_Items:=0;
Stack.Qtt_MaxItems:=QttMax;
L:=QttMax*ItemSize;
Stack.Size:=L;
If (MaxAvail>Stack.Size) And (L<>0) Then  {*Ok, Memory available*}
  Begin
  Stack_Init:=True;
  Getmem(Stack.P_Stack,Stack.Size);
  End
Else
  Begin                                   {*Not enougth memory*}
  Stack.P_Stack:=Nil;
  Stack_Init:=False;
  End;
End;

{ͻ
                            PROCEDURE Stack_Done                            
 ͹
  IN.....: Stack (TSTACK) = Stack Structure                                 
  OUT....: /                                                                
  EXPLAIN: Kill the stack and free allocated memory.                        
  SEEALSO: Stack_Init                                                       
 ͼ}
Procedure Stack_Done(Var Stack:TStack);
Begin
Decp(Stack.P_Stack,Stack.ItemSize*Stack.Qtt_Items);
Stack.Qtt_Items:=0;
FreeMem(Stack.P_Stack,Stack.Size);       {*No Comments*}
End;

{ͻ
                            FUNCTION Stack_Push                             
 ͹
  IN.....: Stack (TSTACK) = Stack Structure                                 
           V (UNTYPED) = Variable witch contain one item.                   
  OUT....: (BOOLEAN) = TRUE if all is OK, FALSE if the stack is full.       
  EXPLAIN: Stack_Push on the stack an item.                                 
  SEEALSO: Stack_Pop, Stack_Init                                            
 ͼ}
Function Stack_Push(Var Stack:TStack;Var V):Boolean;
Begin
If Stack.Qtt_Items=Stack.Qtt_MaxItems Then  {*Stack full*}
  Begin
  Stack_Push:=False;
  Exit;
  End;
Move(V,Stack.P_Stack^,Stack.ItemSize);      {*Copy datas*}
Inc(Stack.Qtt_Items);                       {*Set stack values*}
Incp(Stack.P_Stack,Stack.ItemSize);
Stack_Push:=True;
End;

{ͻ
                             FUNCTION Stack_Pop                             
 ͹
  IN.....: Stack (TSTACK) = Stack Structure                                 
           V (UNTYPED) = Variable where to put the item.                    
  OUT....: (BOOLEAN) = TRUE if all is OK, FALSE if the stack is empty.      
  EXPLAIN: Stack_Pop on the stack an item.                                  
  SEEALSO: Stack_Push, Stack_Init                                           
 ͼ}
Function Stack_Pop(Var Stack:TStack;Var V):Boolean;
Begin
If Stack.Qtt_Items=0 Then                   {*Stack Empty*}
  Begin
  Stack_Pop:=False;
  Exit;
  End;

Dec(Stack.Qtt_Items);                       {*Set stack values*}
Decp(Stack.P_Stack,Stack.ItemSize);
Move(Stack.P_Stack^,V,Stack.ItemSize);      {*Copy stack*}
Stack_Pop:=True;
End;

{ͻ
                          FUNCTION Stack_TestEmpty                          
 ͹
  IN.....: Stack (TSTACK) = Stack Structure                                 
  OUT....: (BOOLEAN) = TRUE if the stack is empty, else FALSE.              
  EXPLAIN: Test if the stack is empty or not.                               
  SEEALSO: Stack_Init                                                       
 ͼ}
Function Stack_TestEmpty(Stack:TStack):Boolean;
Begin

Stack_TestEmpty:=Stack.Qtt_Items=0;
End;

{ͻ
                          FUNCTION Stack_TestFull                           
 ͹
  IN.....: Stack (TSTACK) = Stack Structure                                 
  OUT....: (BOOLEAN) = TRUE if the stack is full, else FALSE.               
  EXPLAIN: Test if the stack is full or not.                                
  SEEALSO: Stack_Init                                                       
 ͼ}
Function Stack_TestFull(Stack:TStack):Boolean;
Begin
Stack_TestFull:=Stack.Qtt_Items=Stack.Qtt_MaxItems;
End;

{ͻ
                            PROCEDURE Stack_Read                            
 ͹
  IN.....: Stack (TSTACK) = Stack Structure                                 
           Num (LONGINT) = N of the item to read.                          
  OUT....: V (UNTYPED) = Variable wehre to put the item.                    
  EXPLAIN: Read one item of the stack.                                      
           The first item is the 1.                                         
  EXAMPLE: if the stack is like this :        Stack_Read(S,1,V)             
           Item 3                             returns in V Item 1.          
           Item 2                                                           
           Item 1                                                           
  SEEALSO: Stack_Init                                                       
 ͼ}
Procedure Stack_Read(Stack:TStack;Num:LongInt;Var V);
Var P:Pointer;
Begin
p:=Stack.P_Stack;
Decp(p,Stack.ItemSize*(Stack.Qtt_Items-Num+1));
Move(p^,V,Stack.ItemSize);               {*Copy from stack*}
End;

{*---------------------------------- CRC -----------------------------------*}

{ͻ
                           FUNCTION Mem_CheckSum                            
 ͹
  IN.....: Data (UNTYPED) = "Pointer" on memory data                        
           Size (LONGINT) = Size in bytes of the memory data block.         
  OUT....: (LONGINT) = Checksum result.                                     
  EXPLAIN: Get a checksum of a part of memory.                              
  NB.....: This CheckSum calculated is different of the true CRC calculation
  CF.....: File_CheckSum                                                    
  FROM...: Swag.                                                            
  TO.....: Passer en 32 bits au lieu de 8 bits la boucle...                 
 ͼ}
Function Mem_CheckSum(Var data; size : LongInt) : LongInt;
Var Res:LongInt;
    i:LongInt;
    P:TPByte;
Begin
P:=@Data;
Res:=0;
For i:=1 to Size do
  Begin
  Res:=Res xor (P^ xor $ABCD);
  Inc(P);
  End;

Mem_CheckSum:=Res;
End;
(*                    ATTENTION Ce calcul donne pas le mme resultat
Assembler;            en pascal qu'en asm...
Asm
  push ds
  lds  si, data
  mov  cx, size
  mov  bx, 0
  mov  dx, $ABCD
  cld
  jcxz @X
 @L:
  lodsb
  rol  bx, 1
  xor  bx, dx
  xor  bl, al
  loop @L
 @X:
  mov  ax, bx
  pop  ds
end;*)

{*------------------------------- Memory Blocks ----------------------------*}

{ͻ
                           FUNCTION Mem_CompBlock                           
 ͹
  IN.....: P1 (UNTYPED) = First block to compare                            
           P2 (UNTYPED) = Second block to compare                           
           Size (LONGINT) = Size of the 2 blocks to compare                 
  OUT....: (BOOLEAN) = TRUE if the blocks are the same, else return FALSE   
  EXPLAIN: Test 2 blocks of memory if they are the sames.                   
  EXAMPLE: Const s1='toto';s2='titi';                                       
           Writeln(Mem_CompBlock(s1,s2,5)) = False                          
           Writeln(Mem_CompBlock(s1,s1,5)) = True                           
  OPT....: Can be optimized for 32 bits instructions                        
 ͼ}
Function Mem_CompBlock(Var P1,P2;Size:LongInt):Boolean;
Var B1,B2:TPByte;
    i:LongInt;
Begin
B1:=@P1;
B2:=@P2;
For i:=1 to Size do
  Begin
  If B1^<>B2^ Then
    Begin
    Mem_CompBlock:=False;
    Exit;
    End;
  Inc(B1);
  Inc(B2);
  End;
Mem_CompBlock:=True;
End;
{Assembler;
Asm
  push ds
  cld
  lds si,p1
  les di,p2
  mov cx,Size
  repz cmpsb
  jz @OK
  mov ax,0
  jmp @fin
@OK:
  mov ax,1
@fin:
  pop ds
End;}

{*------------------------------- Miscelaneous -----------------------------*}

{ͻ
                             PROCEDURE Mem_Gap                              
 ͹
  IN.....: P1 (UNTYPED) = Memory buffer 1                                   
           P2 (UNTYPED) = Memory buffer 2                                   
           Size (LONGINT) = Size of Memory buffer (Size of P1 must be = P2) 
           B (BYTE) = Byte to change                                        
  OUT....: P1 (UNTYPED) = Memory buffer 1 modified                          
  EXPLAIN: Return the memory buffer 1. All the bytes B in buffer 1 are      
           replaced by the bytes of memory buffer 2.                        
  EXAMPLE: P1= 00 05 06 00 33                                               
           P2= 11 22 33 44 55                                               
           Mem_Gap(P1,P2,5,0)                                               
           P1= 11 05 06 44 33                                               
  NB.....: If you have an image (IMA1) with the color 0 = background        
           GetImage(...,IMA2)                                               
           Mem_Gap(IMA1,IMA2,Sizeof(IMA1),0);                               
           PutImage(...,IMA1);                                              
           You will see the image in transparency                           
 ͼ}
Procedure Mem_Gap(Var P1,P2;Size:LongInt;B:Byte);
Var pb1,pb2:TPByte;
    i:Longint;
Begin
pb1:=@P1;
pb2:=@P2;

For i:=1 to Size do
  Begin
  If pb1^=b Then pb1^:=pb2^;
  Inc(pb1);
  Inc(pb2);
  End;
End;

{ͻ
                           Procedure Mem_ReAlloc                            
 ͹
  IN.....: P (Pointer) = An input pointer                                   
           NewSize (LongInt) = New size you want to allocate                
  OUT....: P (Pointer) = OutPut pointer                                     
  EXPLAIN: Allocate a new size to a pointer keeping data...                 
           You can change the size to memory allocated without you lost data
  NB.....: If OldSize>NewSize then you could lost some data !!!             
  EXAMPLE: Mem_ReAlloc(p,60);                                               
 ͼ}

Procedure Mem_ReAlloc(Var P:Pointer; NewSize:LongInt);
{Var P2:Pointer;}
Begin
(*
GetMem(p2,NewSize);                   {*Allocate new mem*}
Move(p2^,p^,Min(OldSize,NewSize));    {*Copy old mem --> new mem*}
FreeMem(p,OldSize);                   {*Free old mem*}
p:=p2;                                {*Result*}
*)
ReAllocMem(p,NewSize);
End;

Begin
End.
