{ͻ
                               UNIT MosCombos                               
 ͹
  CREATOR....: CHEVEAU Frdric                                             
  UPDATED BY.: CHEVEAU Frdric                                             
  CREATED THE: 08.05.2000                                                   
  COUNTRY....: Montpellier - Timeless (FRANCE)                              
  DESCRIPTION: Visual components "TComboBox, TSpot, TCanvas".               
  AUTONOMOUS.: No                                                           
  UNIT SIZE..: 3.718 Lines and (93.136 Bytes compiled)                      
  LAST UPDATE: 12.12.2001 at 23:25                                          
 ͹
                                 DISCLAIMER                                 
 ͹
    This file is a part of the MOS (Master Operating System) library        
    Copyright (C) 1994-2001 Frdric Cheveau and Jean-Pierre Planas         
                                                                            
    This library is Free software since the 4.00 version. You can           
    redistribute it and / or modify it under the terms of the GNU           
    General Public License as published by the Free Software Foundation.    
                                                                            
    This library is distributed in the hope that it will be useful,         
    but WITHOUT ANY WARRANTY; without even the implied warranty of          
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU        
    General Public License for more details. This file should be in         
    the MOS package, if not, write to the Free Software Foundation,         
    Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA           
 ͼ}

UNIT MosCombos;

{$Mode Objfpc}

INTERFACE

Uses
     Classes,
     MosBevels,
     MosControls;

Const
   MaxSpots     = 8;                    {*Number of spots in Canvas*}

Type                                    {*Types definition*}

{****************************************************************************}

{****************************************************************************}
{*** AComboBox Class                                                      ***}
{****************************************************************************}

AComboBox = Class(TVisual)              {*ComboBox class definition*}
   Private
   Protected
   Public
   End;

{****************************************************************************}
{*** TSpot Class                                                          ***}
{****************************************************************************}

TSpot = Class(TBevelBox)                {*TSpot class definition*}
   Private
      fMouse            :AMouseZone;    {*Mouse pointer manager*}
      fSize             :Integer;       {*Spot square size in pixels*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure SetXorMode(M:Boolean);  {*Set the draw mode*}
      Procedure SetSize(S:Integer);     {*Set the spot size*}
      Function  GetXorMode:Boolean;     {*Get the draw mode*}

   Public
      Property Mouse    :AMouseZone Read fMouse;
      Property XorMode  :Boolean    Read GetXorMode  Write SetXorMode;
      Property Size     :Integer                     Write fSize;

      Constructor Create(AOwner:TComponent); Override; {*Create the component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,pSize:Integer; pXorMode:Boolean);{*Create component with parameters*}
      Destructor  Destroy; Override;              {*Destroy the component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{****************************************************************************}
{*** ACanvas Class                                                        ***}
{****************************************************************************}

TSpots = Array[1..MaxSpots] of TSpot;

ACanvas = Class(TVisual)                {*TCanvas class definition*}
   Private
      fSpots            :TSpots;        {*Array of spots*}
      fLineSize         :Integer;       {*Box line size in pixels*}
      fLineVisible      :Boolean;       {*Box line is visible or no*}

      Procedure Define;                 {*Define base variables*}

   Protected
      Procedure DisableSpots;              {*Disable all the spots*}
      Procedure EnableSpots;               {*Enable all the spots*}
      Procedure SetLineSize(S:Integer);    {*Set the box line size*}
      Procedure SetLineVisible(V:Boolean); {*Set the box line visible*}
      Procedure CreateSpots;               {*Create the spots*}
      Function  CalculeSpotCoordinates(pIndex:Integer; Var pX1,pY1:Integer):Boolean; {*Calcule coordinates for a spot*}

   Public
      Property  Spots      :TSpots   Read fSpots;
      Property  LineSize   :Integer  Read fLineSize     Write SetLineSize;
      Property  LineVisible:Boolean  Read fLineVisible  Write SetLineVisible;

      Constructor Create(AOwner:TComponent); Override; {*Create the component*}
      Constructor Create(AOwner:TComponent; pX1,pY1,px2,pY2:Integer; pLineVisible:Boolean);{*Create component with parameters*}
      Destructor  Destroy; Override;              {*Destroy the component*}
      Procedure   Draw; Override;                 {*Draw component*}
      Function    GetMsg:TCompMessage; Override;  {*Get messages from component*}
   End;

{}
{}
{}

IMPLEMENTATION

Uses
     MosBase,
     MosColors,
     MosConfig,
     MosGraph;

{****************************************************************************}
{*** AComboBox Class                                                      ***}
{****************************************************************************}

{****************************************************************************}
{*** TSpot Class                                                          ***}
{****************************************************************************}

Procedure TSpot.Define;
Begin
If Owner = Self Then Log.Add('Create new TSpot ('+')',2);
End;

Constructor TSpot.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
fMouse := AMouseZone.Create(AOwner);

Define;

SetXorMode(Yes);
fSize := 0;
SetSize(5);                             {*Initialise square spot size*}
End;

Constructor TSpot.Create(AOwner:TComponent; pX1,pY1,pSize:Integer; pXorMode:Boolean);
Begin
If pSize < 2 Then Exit;

Inherited Create(AOwner, pX1,pY1,pX1+pSize,pY1+pSize, CLBlack);
fMouse := AMouseZone.Create(AOwner, pX1,pY1,pX1+pSize,pY1+pSize, MBLeft, MPSizeLR);

Define;

SetXorMode(pXorMode);
End;

Destructor TSpot.Destroy;
Begin
fMouse.Destroy;
Inherited Destroy;
End;

Procedure TSpot.Draw;
Var OldMode          :Smallint;

Begin
If Not Visible Then Exit;

OldMode := VESA_GetWriteMode;           {*Get curent write mode*}
If Not SaveBg Then VESA_SetWriteMode(XORPut);
Inherited Draw;
If Not SaveBg Then VESA_SetWriteMode(OldMode);
End;

Function TSpot.GetMsg:TCompMessage;
Begin
Result := CMNone;
If Not Enabled Then Exit;               {*Not enabled !*}

fMouse.GetMsg;                          {*Get messages from AMouse*}
Message.MoveFrom(fMouse.Message);       {*Add messages from Mouse list*}

Result := Message.GetFromList;          {*Return messages*}
End;

Function TSpot.GetXorMode:Boolean;
Begin
Result := Not SaveBg;
End;

Procedure TSpot.SetXorMode(M:Boolean);
Begin
If M = SaveBg Then Exit;
SaveBg := M;
Invalidate;
End;

Procedure TSpot.SetSize(S:Integer);
Begin
If S = fSize Then Exit;
If S < 2 Then Exit;                     {*Minimum size*}
fSize := S;

Width         := fSize;
Height        := fSize;
fMouse.Width  := fSize;
fMouse.Height := fSize;

Invalidate;
End;

{****************************************************************************}
{*** ACanvas Class                                                        ***}
{****************************************************************************}

Procedure ACanvas.Define;
Begin
fLineSize    := 1;
fLineVisible := No;

If Owner = Self Then Log.Add('Create new TCanvas ('+')',2);
End;

Constructor ACanvas.Create(AOwner:TComponent);
Begin
Inherited Create(AOwner);
CreateSpots;

Define;
End;

Constructor ACanvas.Create(AOwner:TComponent; pX1,pY1,pX2,pY2:Integer; pLineVisible:Boolean);
Begin
Inherited Create(AOwner, pX1,pY1,pX2,pY2);
CreateSpots;

Define;

fLineVisible := pLineVisible;
End;

Function ACanvas.CalculeSpotCoordinates(pIndex:Integer; Var pX1,pY1:Integer):Boolean;
Begin
pX1 := 0;
pY1 := 0;
Result := No;
If (pIndex < 1) Or (pIndex > MaxSpots) Then Exit;

Case pIndex of                          {*Calcule coordinates*}
   1 : Begin                            {*Left top*}
       End;
   2 : Begin                            {*Center top*}
       End;
   3 : Begin                            {*Right top*}
       End;
   4 : Begin                            {*Right middle*}
       End;
   5 : Begin                            {*Right bottom*}
       End;
   6 : Begin                            {*Center bottom*}
       End;
   7 : Begin                            {*Left bottom*}
       End;
   8 : Begin                            {*Left middle*}
       End;
   End;

Result := Yes;
End;

Procedure ACanvas.CreateSpots;
Var cX1,cY1          :Integer;          {*Spots coordinates*}
    Loop             :Integer;          {*Loop for spots array*}
Begin
For Loop := 1 to MaxSpots do
   Begin
   CalculeSpotCoordinates(Loop, cX1,cY1);
   Spots[Loop] := TSpot.Create(Owner, cX1,cY1,5, Yes);
   End;
End;

Destructor ACanvas.Destroy;
Var Loop             :Integer;          {*Loop for spots array*}
Begin
For Loop := 1 to MaxSpots do
   Begin
   Spots[Loop].Destroy;
   End;
Inherited Destroy;
End;

Procedure ACanvas.Draw;
Var Loop             :Integer;          {*Loop for spots array*}

Begin
If Not Visible Then Exit;
Inherited Draw;

For Loop := 1 to MaxSpots do
   Begin
   Spots[Loop].Draw;
   End;
End;

Function ACanvas.GetMsg:TCompMessage;
Var Loop             :Integer;          {*Loop for spots array*}

Begin
If Not Enabled Then Exit;
Result := Inherited GetMsg;

For Loop := 1 to MaxSpots do
   Begin
   Spots[Loop].GetMsg;
   End;
End;

Procedure ACanvas.DisableSpots;
Var Loop:Integer;
Begin
For Loop := 1 to MaxSpots do
   Begin
   Spots[Loop].Enabled := No;
   End;
Invalidate;
End;

Procedure ACanvas.EnableSpots;
Var Loop:Integer;
Begin
For Loop := 1 to MaxSpots do
   Begin
   Spots[Loop].Enabled := Yes;
   End;
Invalidate;
End;

Procedure ACanvas.SetLineSize(S:Integer);
Begin
If S = fLineSize Then Exit;
If S < 1 Then Exit;
fLineSize := S;
Invalidate;
End;

Procedure ACanvas.SetLineVisible(V:Boolean);
Begin
If V = fLineVisible Then Exit;
fLineVisible := V;
Invalidate;
End;

{******************************* NO MAIN CODE *******************************}

Begin
End.
